/* $Id: apsttest.c,v 1.16 2004/04/09 06:16:46 jhayes Exp $ */
/*
 * Copyright  2002 The Regents of the University of California. 
 * All Rights Reserved. 
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for educational, research and non-profit purposes, without
 * fee, and without a written agreement is hereby granted, provided that the
 * above copyright notice, this paragraph and the following three paragraphs
 * appear in all copies. 
 *
 * Permission to incorporate this software into commercial products may be
 * obtained by contacting
 * Technology Transfer Office 
 * 9500 Gilman Drive 
 * 411 University Center 
 * University of California 
 * La Jolla, CA 92093-0093
 * (858) 534-5815
 * invent@ucsd.edu
 *
 * This software program and documentation are copyrighted by The Regents of
 * the University of California. The software program and documentation are
 * supplied "as is", without any accompanying services from The Regents. The
 * Regents does not warrant that the operation of the program will be
 * uninterrupted or error-free. The end-user understands that the program was
 * developed for research purposes and is advised not to rely exclusively on
 * the program for any reason. 
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
 * LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION,
 * EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED
 * HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO
 * OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
 * MODIFICATIONS. 
 */


/*
 * This program is a test driver for the apstd daemon.  It starts daemons and
 * pushes various commands to them through the client interface.  Usage:
 *
 * apsttest [-h] [-p path] [xml ...]
 *
 * where #path# is the path to the apstd binary (default ../bin/apstd) and each
 * #xml# is a path to an APST XML file that specifies resources (i.e., disks
 * and hosts) that the program should use for testing.  If given no XML file,
 * the program will test the APST protocol and the ability of the daemon to
 * Providing XML specifications for hosts and disks that use other supported
 * infrastructures (e.g. globus, ssh) will extend the scope of the tests.  See
 * the accompanying file testssh.xml for one example.
 */


#include "config.h"
#include <signal.h>   /* signal */
#include <stdarg.h>   /* va_* */
#include <stdio.h>    /* file functions */
#include <stdlib.h>   /* free system */
#include <string.h>   /* strcmp */
#include <oshseed.h>  /* rmdir sleep symlink */
#include <sys/stat.h> /* mkdir */
#include <sys/wait.h> /* wait */
#define ASDIAG_SHORT_NAMES
#include <diagnosticseed.h>
#define ASENV_SHORT_NAMES
#include <envseed.h>
#define ASFILE_SHORT_NAMES
#include <fileseed.h>
#define ASIP_SHORT_NAMES
#include <ipseed.h>
#include <propertyseed.h>
#define ASSTR_SHORT_NAMES
#include <strseed.h>
#define ASXML_SHORT_NAMES
#include <xmlseed.h>
#include <elbase.h>
#include "client.h"

#define ENVVAR_NAME "HOME"
#define LINK_NAME "myls"
#define OUTFILE_NAME "apsttestout"
#define ABSDIR_PATH "/tmp"
#define RELDIR_PATH "./apsttestdir"
#define VARDIR_PATH "${" ENVVAR_NAME "}"
#define ABSFILE_PATH ABSDIR_PATH "/" OUTFILE_NAME
#define RELFILE_PATH RELDIR_PATH "/" OUTFILE_NAME
#define VARFILE_PATH VARDIR_PATH "/" OUTFILE_NAME
#define LINK_PATH RELDIR_PATH "/" LINK_NAME
#define LOCALDISK_ID "_localdisk"
#define ABSDISK_ID "_absdisk"
#define RELDISK_ID "_reldisk"
#define VARDISK_ID "_vardisk"

typedef enum {
  CANCELED, DONE, FAILED, NEW, RUNNING, UNKNOWN
} TaskStatuses;

static const char *XML_ARGS =
  "<task executable='/bin/cp' arguments='/dev/null " ABSFILE_PATH "'/>";
static const char *XML_DEPENDEE =
  "<task executable='/bin/cp' arguments='/dev/null /tmp/apsttestdepend' "
  "output='/tmp/apsttestdepend'/>";
static const char *XML_DEPENDENT =
  "<task executable='/bin/rm' arguments='/tmp/apsttestdepend' "
  "input='/tmp/apsttestdepend'/>";
static const char *XML_EMPTY = "<apst></apst>";
static const char *XML_LOCAL_DISK =
  "<storage><disk id='" LOCALDISK_ID "'/></storage>";
static const char *XML_LOCAL_DISK_DATADIR_ABSOLUTE =
  "<storage><disk id='" ABSDISK_ID "' datadir='" ABSDIR_PATH "'><local/>"
  "</disk></storage>";
static const char *XML_LOCAL_DISK_DATADIR_RELATIVE =
  "<storage><disk id='" RELDISK_ID "' datadir='" RELDIR_PATH "'><local/>"
  "</disk></storage>";
static const char *XML_LOCAL_DISK_DATADIR_VARIABLE =
  "<storage><disk id='" VARDISK_ID "' datadir='" VARDIR_PATH "'><local/>"
  "</disk></storage>";
static const char *XML_LOCAL_HOST =
  "<compute><host id='_localhost' disk='" LOCALDISK_ID "'><local/>"
  "</host></compute>";
static const char *XML_LOCAL_HOST_DATADIR_ABSOLUTE =
  "<compute><host id='_abshost' disk='" ABSDISK_ID "'><local/>"
  "</host></compute>";
static const char *XML_LOCAL_HOST_DATADIR_RELATIVE =
  "<compute><host id='_relhost' disk='" RELDISK_ID "'><local/>"
  "</host></compute>";
static const char *XML_LOCAL_HOST_DATADIR_VARIABLE =
  "<compute><host id='_varhost' disk='" VARDISK_ID "'><local/>"
  "</host></compute>";
static const char *XML_LOCAL_HOST_WD_ABSOLUTE =
  "<compute><host id='_awdhost' disk='" LOCALDISK_ID "' wd='/bin'><local/>"
  "</host></compute>";
static const char *XML_LOCAL_HOST_WD_RELATIVE =
  "<compute><host id='_rwdhost' disk='" LOCALDISK_ID "' wd='" RELDIR_PATH "'>"
  "<local/></host></compute>";
static const char *XML_NOOP = "<task executable='/bin/echo'/>";
static const char *XML_PATH_STDERR_ABSOLUTE =
  "<task executable='/bin/rm' arguments='/abcdefg' stderr='" ABSFILE_PATH "'/>";
static const char *XML_PATH_STDERR_RELATIVE =
  "<task executable='/bin/rm' arguments='/abcdefg' stderr='" RELFILE_PATH "'/>";
static const char *XML_PATH_STDERR_VARIABLE =
  "<task executable='/bin/rm' arguments='/abcdefg' stderr='" VARFILE_PATH "'/>";
static const char *XML_PATH_STDOUT_ABSOLUTE =
  "<task executable='/bin/ls' stdout='" ABSFILE_PATH "'/>";
static const char *XML_PATH_STDOUT_DEFAULT =
  "<task executable='/bin/ls' stdout='" OUTFILE_NAME "'/>";
static const char *XML_PATH_STDOUT_RELATIVE =
  "<task executable='/bin/ls' stdout='" RELFILE_PATH "'/>";
static const char *XML_PATH_STDOUT_VARIABLE =
  "<task executable='/bin/ls' stdout='" VARFILE_PATH "'/>";
static const char *XML_PATH_TASK_ABSOLUTE =
  "<task executable='./ls' wd='/bin' stdout='" ABSFILE_PATH "'/>";
static const char *XML_PATH_TASK_RELATIVE =
  "<task executable='./" LINK_NAME "' wd='" RELDIR_PATH "' "
  "stdout='" ABSFILE_PATH "'/>";
static const char *XML_PATH_TASK_BINNAME =
  "<task executable='./ls' stdout='" ABSFILE_PATH "'/>";
static const char *XML_PATH_TASK_LINKNAME =
  "<task executable='./" LINK_NAME "' stdout='" ABSFILE_PATH "'/>";
static const char *XML_PRIORITY_HIGH =
  "<task executable='/bin/cp' arguments='/dev/null /tmp/apsttestpriority' "
  "priority='2'/>";
static const char *XML_PRIORITY_LOW =
  "<task executable='/bin/rm' arguments='/tmp/apsttestpriority' "
  "priority='1'/>";
static const char *XML_RELATIVE_CLEANUP =
  "<task executable='/bin/rm' arguments='-fr apsttestdata'/>";
static const char *XML_RELATIVE =
  "<task executable='/bin/cat' stdin='apsttestdata/in' "
  "stdout='apsttestdata/out' stderr='apsttestdata/err'/>";
static const char *XML_RELATIVE_SETUP =
  "<task executable='/bin/mkdir' arguments='apsttestdata' priority='2'/>"
  "<task executable='/bin/cp' arguments='/dev/null apsttestdata/in' "
  "priority='1'/>";
static const char *XML_STDERR =
  "<task executable='/bin/rm' arguments='/abcdefg' " "stderr='" ABSFILE_PATH
  "'/>";
static const char *XML_STDIN =
  "<task executable='/bin/cat' stdin='/dev/null' stdout='" ABSFILE_PATH "'/>";
static const char *XML_STDOUT =
  "<task executable='/bin/cat' arguments='/dev/null' stdout='" ABSFILE_PATH
  "'/>";
static const char *XML_WD =
  "<task executable='/bin/cp' wd='/etc' arguments='./passwd " ABSFILE_PATH
  "'/>";

const char *COMPUTE_SERVICE_IMAGES[ELBASE_COMPUTE_SERVICE_COUNT + 1] = {
  "local", "globus", "netsolve", "rash", "ssh"
};
const char *STORAGE_SERVICE_IMAGES[ELBASE_STORAGE_SERVICE_COUNT + 1] = {
  "local", "ftp", "gass", "scp", "sftp", "srb"
};

static int
EnableHost(Socket sock,
           const char *host) {
  return APSTClient_Disable(sock, "all") &&
         (host == NULL || APSTClient_Enable(sock, host));
}

static int
SpawnApstd(const char *path,
           const char *port) {
  int status;
  if(fork() == 0) {
    execlp(path, path, "--port", port, NULL);
    return -1;
  }
  return wait(&status) >= 0 && WIFEXITED(status) && WEXITSTATUS(status) == 0;
}

static XmlNode
FindListItemByAttribute(XmlNodeList list,
                        const char *attribute,
                        const char *value) {
  unsigned i;
  XmlNode node;
  for(i = 0;
      (node = NodeListItem(list, i)) != NULL &&
      strcmp(ElementGetAttribute(node, (char *)attribute), value) != 0;
      i++)
    ; /* Nothing more to do. */
  return node;
}

static XmlNode
FindListItemByChildTag(XmlNodeList list,
                       const char *tag) {
  XmlNode child;
  unsigned i;
  XmlNode node;
  for(i = 0; (node = NodeListItem(list, i)) != NULL; i++) {
    if(tag == NULL) {
      if(NodeFirstChildByType(node, SHOW_ELEMENT) == NULL)
        return node;
    }
    else {
      for(child = NodeFirstChildByType(node, SHOW_ELEMENT);
          child != NULL;
          child = NodeNextSiblingByType(child, SHOW_ELEMENT)) {
        if(strcmp(NodeName(child), tag) == 0)
          return node;
      }
    }
  }
  return NULL;
}
       
static char *
SubmitTask(Socket sock,
           const char *task) {

  XmlDocument doc;
  unsigned i;
  XmlNodeList list;
  XmlNode node;
  char *result = NULL;
  char *state;
  char *xml = StrAppend(NULL, "<apst><tasks>", task, "</tasks></apst>", NULL);

  if(!APSTClient_AddXml(sock, xml) ||
     !APSTClient_GetState(sock, "all", &state)) {
    free(xml);
    return NULL;
  }
  free(xml);
  doc = XmlFromString(state, NULL, 1, 1, NULL, NULL, NULL);
  list = DocumentGetElementsByTagName(doc, "task");
  for(i = 0; (node = NodeListItem(list, i)) != NULL; i++) {
    char *taskId = ElementGetAttribute(node, "position");
    if(result == NULL || atoi(taskId) > atoi(result))
      result = taskId;
  }
  result = strdup(result);
  NodeListFree(list);
  NodeFree(doc);
  free(state);
  return result;

}

static TaskStatuses
TaskStatus(Socket sock,
           const char *taskId) {

  const char *IMAGES[] = {
    "canceled", "done", "failed", "new", "running", NULL
  };
  XmlDocument doc;
  unsigned i;
  XmlNodeList list;
  XmlNode node;
  char *state;
  char *status;

  if(!APSTClient_GetState(sock, taskId, &state))
    return UNKNOWN;
  doc = XmlFromString(state, NULL, 1, 1, NULL, NULL, NULL);
  list = DocumentGetElementsByTagName(doc, "task");
  status = (node = NodeListItem(list, 0)) == NULL ?
           "" : ElementGetAttribute(node, "status");
  for(i = 0; IMAGES[i] != NULL && strcmp(status, IMAGES[i]) != 0; i++)
    ; /* Nothing more to do. */
  NodeListFree(list);
  NodeFree(doc);
  free(state);
  return IMAGES[i] == NULL ? UNKNOWN : (TaskStatuses)i;

}

static int
WaitForTask(Socket sock,
            const char *taskId) {
  TaskStatuses status;
  while((status = TaskStatus(sock, taskId)) == NEW || status == RUNNING)
    sleep(1);
  return status == DONE;
}

static int
TryRun(Socket sock,
       XmlNode hostNode,
       const char *task) {
  int result;
  char *taskId;
  if(!EnableHost(sock, ElementGetAttribute(hostNode, "id")) ||
     (taskId = SubmitTask(sock, task)) == NULL)
    return 0;
  result = WaitForTask(sock, taskId);
  free(taskId);
  return result;
}

int
main(int argc,
     char **argv) {

  const char *VALID =
    "-a path      path to apstd binary\n"
    "-h void      display this text\n"
    "-p int       port for apstd\n"
    "   path ...  path to resource XML file";

  Address addr;
  XmlNode disk;
  XmlNodeList diskList;
  XmlDocument doc;
  DiagnosticState ds = DiagnosticStateNew();
  char envvarValue[256];
  XmlNode host;
  char *highId;
  XmlNodeList hostList;
  unsigned i;
  int passed;
  ParsedArgv pa;
  const char *path;
  char *port;
  Socket sock;
  char *s;
  char *t;
  char *taskId;
  char varPath[256];

  DirectDiagnostics(ds, ERROR_MESSAGE, fileno(stderr));
  DirectDiagnostics(ds, LOG_MESSAGE, fileno(stdout));
  signal(SIGPIPE, SIG_IGN);

  /* Verify any command-line switches. */
  if((pa = ParseArgv((const char*const *)argv, VALID, NULL)) == NULL)
    FAILRETURN1(ds, "%s\n", VALID, 1);
  for(i = 1; (path = SwitchValueN(pa, "", i, NULL)) != NULL; i++) {
    const char *c;
    XmlParseErrorCodes error;
    unsigned long errorPos;
    unsigned long line;
    if((s = FileContents(path)) == NULL)
      FAILRETURN1(ds, "Unable to open %s\n", path, 1);
    if((doc = XmlFromString(s, NULL, 1, 1, NULL, &error, &errorPos)) == NULL) {
      for(line = 1, c = strchr(s, '\n');
          c != NULL && c - s < errorPos;
          line++, c = strchr(c + 1, '\n'))
        ; /* Nothing more to do. */
      free(s);
      FAILRETURN2(ds, "Parse error in %s on line %d\n", path, line, 1);
    }
    NodeFree(doc);
    free(s);
  }

  port = strdup(SwitchValue(pa, "-p", DEFAULT_APSTD_PORT));
  if((s = getenv("HOME")) != NULL) {
    char *apstrcPath;
    FILE *f;
    ASPROP_Property p;
    ASPROP_PropertyList pl;
    apstrcPath = StrAppend(NULL, s, "/.apstrc", NULL);
    if((f = fopen(apstrcPath, "r")) != NULL) {
      pl = ASPROP_ReadPropertyList(f);
      fclose(f);
      if(pl != NULL) {
        p = ASPROP_FindPropertyByName(pl, "APSTD_PORT");
        ASPROP_PropertyListFree(&pl);
        if(p != NULL) {
          free(port);
          port = strdup(ASPROP_PropertyValue(p));
        }
      }
    }
    free(apstrcPath);
  }

  AddressValue(MyMachineName(), &addr);
  path = SwitchValue(pa, "-a", "apstd");
  if(!SpawnApstd(path, port))
    FAILRETURN1(ds, "Unable to run %s\n", path, 1);
  if((sock = ConnectToPort(addr, strtol(port, NULL, 10))) ==
     NO_SOCKET)
    FAILRETURN(ds, "Unable to contact daemon\n", 1);

  /* Combine local XML and all the listed XML documents into one big doc. */
  s = StrAppend
    (NULL, "<apst>",
     XML_LOCAL_DISK, XML_LOCAL_HOST,
     XML_LOCAL_DISK_DATADIR_ABSOLUTE, XML_LOCAL_HOST_DATADIR_ABSOLUTE,
     XML_LOCAL_DISK_DATADIR_RELATIVE, XML_LOCAL_HOST_DATADIR_RELATIVE,
     XML_LOCAL_DISK_DATADIR_VARIABLE, XML_LOCAL_HOST_DATADIR_VARIABLE,
     XML_LOCAL_HOST_WD_ABSOLUTE,      XML_LOCAL_HOST_WD_RELATIVE,
     "</apst>", NULL);
  doc = XmlFromString(s, NULL, 1, 1, NULL, NULL, NULL);
  free(s);
  for(i = 1; (path = SwitchValueN(pa, "", i, NULL)) != NULL; i++) {
    XmlNode n;
    XmlDocument subdoc;
    s = FileContents(path);
    subdoc = XmlFromString(s, NULL, 1, 1, NULL, NULL, NULL);
    for(n = NodeFirstChild(DocumentElement(subdoc));
        n != NULL;
        n = NodeNextSibling(n))
      NodeAppendChild(DocumentElement(doc), DocumentImportNode(doc, n, 1));
    NodeFree(subdoc);
    free(s);
  }

  fprintf(stdout, "\nProtocol tests...\n");

  s = NULL;
  passed = APSTClient_Noop(sock) && APSTClient_GetId(sock, &s) &&
           s != NULL && *s != '\0';
  fprintf(stdout, "%s basic messages\n", passed ? "Pass" : "Fail");
  if(s != NULL)
    free(s);

  s = NULL;
  passed = APSTClient_GetLog(sock, &s);
  fprintf(stdout, "%s log\n", passed ? "Pass" : "Fail");
  if(s != NULL)
    free(s);

  s = XmlToString(doc, NULL);
  t = NULL;
  passed = APSTClient_AddXml(sock, s) &&
           APSTClient_GetState(sock, "all", &t) &&
           strcmp(t, XML_EMPTY) != 0 &&
           APSTClient_Clear(sock) &&
           APSTClient_GetState(sock, "all", &t) &&
           strcmp(t, XML_EMPTY) == 0;
  fprintf(stdout, "%s clear\n", passed ? "Pass" : "Fail");
  if(s != NULL)
    free(s);
  if(t != NULL)
    free(t);

  s = NULL;
  passed = APSTClient_Stop(sock) && !APSTClient_GetId(sock, &s);
  fprintf(stdout, "%s stop\n", passed ? "Pass" : "Fail");
  if(s != NULL)
    free(s);

  Disconnect(&sock);
  sleep(3); /* Let the daemon finish shutting down. */
  path = SwitchValue(pa, "-a", "../bin/apstd");
  if(!SpawnApstd(path, port))
    FAILRETURN1(ds, "Unable to run %s\n", path, 1);
  if((sock = ConnectToPort(addr, strtol(port, NULL, 10))) ==
     NO_SOCKET)
    FAILRETURN(ds, "Unable to contact daemon\n", 1);

  fprintf(stdout, "\nSecurity tests...\n");

  APSTClient_AllowFile(0);
  passed = !APSTClient_AddXml(sock, XML_EMPTY) &&
           strstr(APSTClient_Error(), "denied") != NULL;
  fprintf(stdout, "%s minimal security\n", passed ? "Pass" : "Fail");

  APSTClient_AllowFile(1);
  passed = APSTClient_AddXml(sock, XML_EMPTY);
  fprintf(stdout, "%s file connection\n", passed ? "Pass" : "Fail");

  s = XmlToString(doc, NULL);
  APSTClient_AddXml(sock, s);
  free(s);
  diskList = DocumentGetElementsByTagName(doc, "disk");
  hostList = DocumentGetElementsByTagName(doc, "host");

  fprintf(stdout, "\nSpawn tests...\n");

  for(i = 0; i < ELBASE_COMPUTE_SERVICE_COUNT; i++) {
    const char *image = COMPUTE_SERVICE_IMAGES[i];
    host = FindListItemByChildTag(hostList, image);
    passed = host != NULL && TryRun(sock, host, XML_NOOP);
    fprintf(stdout, "%s %s run\n",
            passed ? "Pass" : host == NULL ? "N/A" : "Fail", image);
  }

  fprintf(stdout, "\nTask feature tests...\n");

  host = FindListItemByChildTag(hostList, "local");

  passed = TryRun(sock, host, XML_ARGS) && unlink(ABSFILE_PATH) == 0;
  fprintf(stdout, "%s arguments\n", passed ? "Pass" : "Fail");

  passed = TryRun(sock, host, XML_STDOUT) && unlink(ABSFILE_PATH) == 0;
  fprintf(stdout, "%s stdout\n", passed ? "Pass" : "Fail");

  passed = TryRun(sock, host, XML_STDIN) && unlink(ABSFILE_PATH) == 0;
  fprintf(stdout, "%s stdin\n", passed ? "Pass" : "Fail");

  passed = !TryRun(sock, host, XML_STDERR) && unlink(ABSFILE_PATH) == 0;
  fprintf(stdout, "%s stderr\n", passed ? "Pass" : "Fail");

  passed = TryRun(sock, host, XML_WD) && unlink(ABSFILE_PATH) == 0;
  fprintf(stdout, "%s wd\n", passed ? "Pass" : "Fail");

  taskId = SubmitTask(sock, XML_DEPENDENT);
  sleep(2); /* Give apstd some time to reschedule. */
  passed = taskId != NULL && TaskStatus(sock, taskId) == NEW &&
           TryRun(sock, host, XML_DEPENDEE) && WaitForTask(sock, taskId);
  fprintf(stdout, "%s dependency\n", passed ? "Pass" : "Fail");
  if(taskId != NULL)
    free(taskId);

  EnableHost(sock, NULL);
  if((taskId = SubmitTask(sock, XML_PRIORITY_LOW)) != NULL) {
    if((highId = SubmitTask(sock, XML_PRIORITY_HIGH)) != NULL)
      free(highId);
  }
  EnableHost(sock, ElementGetAttribute(host, "id"));
  passed = taskId != NULL && WaitForTask(sock, taskId);
  fprintf(stdout, "%s priority\n", passed ? "Pass" : "Fail");
  if(taskId != NULL)
    free(taskId);

  passed = TryRun(sock, host, XML_RELATIVE_SETUP) &&
           TryRun(sock, host, XML_RELATIVE);
  fprintf(stdout, "%s relative paths\n", passed ? "Pass" : "Fail");

  TryRun(sock, host, XML_RELATIVE_CLEANUP);

  fprintf(stdout, "\nFile transfer tests...\n");

  for(i = 0; i < ELBASE_STORAGE_SERVICE_COUNT; i++) {
    const char *image = STORAGE_SERVICE_IMAGES[i];
    host = (disk = FindListItemByChildTag(diskList, image)) == NULL ? NULL :
           FindListItemByAttribute
             (hostList, "disk", ElementGetAttribute(disk, "id"));
    passed = host != NULL && TryRun(sock, host, XML_STDIN);
    fprintf(stdout, "%s %s input\n",
            passed ? "Pass" : host == NULL ? "N/A" : "Fail", image);
  }
  unlink(ABSFILE_PATH);

  fprintf(stdout, "\nPath tests...\n");

  host = FindListItemByAttribute(hostList, "disk", LOCALDISK_ID);
  strcpy(envvarValue, getenv(ENVVAR_NAME));
  strcpy(varPath, envvarValue);
  strcat(varPath, "/" OUTFILE_NAME);
  mkdir(RELDIR_PATH, 0777);
  symlink("/bin/ls", LINK_PATH);

  passed = TryRun(sock, host, XML_PATH_STDOUT_ABSOLUTE) &&
          unlink(ABSFILE_PATH) == 0;
  fprintf(stdout, "%s absolute stdout test\n", passed ? "Pass" : "Fail");

  passed = TryRun(sock, host, XML_PATH_STDOUT_RELATIVE) &&
           unlink(RELFILE_PATH) == 0;
  fprintf(stdout, "%s relative stdout test\n", passed ? "Pass" : "Fail");

  passed = TryRun(sock, host, XML_PATH_STDOUT_VARIABLE) &&
           unlink(varPath) == 0;
  fprintf(stdout, "%s variable stdout test\n", passed ? "Pass" : "Fail");

  passed = !TryRun(sock, host, XML_PATH_STDERR_ABSOLUTE) &&
           unlink(ABSFILE_PATH) == 0;
  fprintf(stdout, "%s absolute stderr test\n", passed ? "Pass" : "Fail");

  passed = !TryRun(sock, host, XML_PATH_STDERR_RELATIVE) &&
           unlink(RELFILE_PATH) == 0;
  fprintf(stdout, "%s relative stderr test\n", passed ? "Pass" : "Fail");

  passed = !TryRun(sock, host, XML_PATH_STDERR_VARIABLE) &&
           unlink(varPath) == 0;
  fprintf(stdout, "%s variable stderr test\n", passed ? "Pass" : "Fail");

  passed = TryRun(sock, host, XML_PATH_TASK_ABSOLUTE) &&
           unlink(ABSFILE_PATH) == 0;
  fprintf(stdout, "%s absolute task test\n", passed ? "Pass" : "Fail");

  passed = TryRun(sock, host, XML_PATH_TASK_RELATIVE) &&
           unlink(ABSFILE_PATH) == 0;
  fprintf(stdout, "%s relative task test\n", passed ? "Pass" : "Fail");

  host = FindListItemByAttribute(hostList, "disk", ABSDISK_ID);
  passed = TryRun(sock, host, XML_PATH_STDOUT_DEFAULT) &&
           unlink(ABSFILE_PATH) == 0;
  fprintf(stdout, "%s absolute datadir test\n", passed ? "Pass" : "Fail");

  host = FindListItemByAttribute(hostList, "disk", RELDISK_ID);
  passed = TryRun(sock, host, XML_PATH_STDOUT_DEFAULT) &&
           unlink(RELFILE_PATH) == 0;
  fprintf(stdout, "%s relative datadir test\n", passed ? "Pass" : "Fail");

  host = FindListItemByAttribute(hostList, "disk", VARDISK_ID);
  passed = TryRun(sock, host, XML_PATH_STDOUT_DEFAULT) &&
           unlink(varPath) == 0;
  fprintf(stdout, "%s variable datadir test\n", passed ? "Pass" : "Fail");

  host = FindListItemByAttribute(hostList, "wd", "/bin");
  passed = TryRun(sock, host, XML_PATH_TASK_BINNAME) &&
           unlink(ABSFILE_PATH) == 0;
  fprintf(stdout, "%s absolute host wd test\n", passed ? "Pass" : "Fail");

  host = FindListItemByAttribute(hostList, "wd", RELDIR_PATH);
  passed = TryRun(sock, host, XML_PATH_TASK_LINKNAME) &&
           unlink(ABSFILE_PATH) == 0;
  fprintf(stdout, "%s relative host wd test\n", passed ? "Pass" : "Fail");

  unlink(LINK_PATH);
  rmdir(RELDIR_PATH);

  APSTClient_Stop(sock);
  NodeFree(doc);
  return 0;

}
