/* $Id: em.c,v 1.6 2004/03/05 22:00:10 jhayes Exp $ */
/*
 * Copyright  2004 The Regents of the University of California. 
 * All Rights Reserved. 
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for educational, research and non-profit purposes, without
 * fee, and without a written agreement is hereby granted, provided that the
 * above copyright notice, this paragraph and the following three paragraphs
 * appear in all copies. 
 *
 * Permission to incorporate this software into commercial products may be
 * obtained by contacting
 * Technology Transfer Office 
 * 9500 Gilman Drive 
 * 411 University Center 
 * University of California 
 * La Jolla, CA 92093-0093
 * (858) 534-5815
 * invent@ucsd.edu
 *
 * This software program and documentation are copyrighted by The Regents of
 * the University of California. The software program and documentation are
 * supplied "as is", without any accompanying services from The Regents. The
 * Regents does not warrant that the operation of the program will be
 * uninterrupted or error-free. The end-user understands that the program was
 * developed for research purposes and is advised not to rely exclusively on
 * the program for any reason. 
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
 * LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION,
 * EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED
 * HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO
 * OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
 * MODIFICATIONS. 
 */

 /*
  * This program provides a command-line interface to Elagi's metric query
  * services.  Usage:
  *
  *   em [--server [type#]server] [entity[#metric]] ...
  *
  * type defaults to ganglia, entity to the server and metric to cpuload.
  */


#include <stdio.h>
#include <string.h>
#include <strseed.h>
#include "elmetric.h"


static const char *METRIC_TYPE_NAMES[ELMETRIC_METRIC_COUNT + 1] = {
  "cpucount", "cpuload", "bandwidth", "latency", NULL
};
static const char *SERVICE_TYPE_NAMES[ELMETRIC_METRIC_SERVICE_COUNT + 1] = {
  "local", "ganglia", "mds", "nws", NULL
};


int
ShowMetric(ELMETRIC_MetricServiceTypes serviceType,
           char *server,
           char *entity) {

  char *c;
  unsigned i;
  ELMETRIC_Key key;
  ELMETRIC_Metric metric;
  const char *metricName;
  ELMETRIC_MetricTypes metricType;

  if((c = strchr(entity, '#')) == NULL)
    metricType = ELMETRIC_CPU_LOAD;
  else {
    *(c++) = '\0';
    for(i = 0;
        METRIC_TYPE_NAMES[i] != NULL && strcmp(METRIC_TYPE_NAMES[i], c) != 0;
        i++)
      ; /* empty */
    if(METRIC_TYPE_NAMES[i] == NULL) {
      fprintf(stderr, "Unknown metric '%s'\n", c);
      return 0;
    }
    metricType = (ELMETRIC_MetricTypes)i;
  }
  metricName = METRIC_TYPE_NAMES[metricType];
  if((key = ELMETRIC_GetKey(serviceType, server, entity, metricType)) ==
     ELMETRIC_NO_KEY) {
    fprintf(stderr, "%s %s unable to retrieve key\n", entity, metricName);
    return 0;
  }
  else {
    if((metric = ELMETRIC_GetMetric(key)) == ELMETRIC_NO_METRIC) {
      ELMETRIC_FreeKey(key);
      fprintf(stderr, "%s %s unable to retrieve metric\n", entity, metricName);
      return 0;
    }
    printf("%s %s=%s\n", entity, metricName, ELMETRIC_GetMetricValue(metric));
    ELMETRIC_FreeMetric(metric);
    ELMETRIC_FreeKey(key);
  }

  return 1;

}


int
main(int argc,
     char **argv) {

  char *c;
  unsigned i;
  unsigned j;
  char *entity = NULL;
  char *server = "localhost";
  ELMETRIC_MetricServiceTypes serviceType = ELMETRIC_LOCAL;

  for(i = 1; argv[i] != NULL; i++) {
    if(strcmp(argv[i], "--server") != 0) {
      entity = argv[i];
      ShowMetric(serviceType, server, entity);
      continue;
    }
    if((server = argv[++i]) == NULL)
      break;
    if((c = strchr(server, '#')) == NULL)
      serviceType = ELMETRIC_GANGLIA;
    else {
      *c = '\0';
      for(j = 0;
          SERVICE_TYPE_NAMES[j] != NULL &&
          strcmp(SERVICE_TYPE_NAMES[j], argv[i]) != 0;
          j++)
        ; /* empty */
      if(SERVICE_TYPE_NAMES[j] == NULL) {
        fprintf(stderr, "Unknown service '%s'\n", argv[i]);
        return 1;
      }
      serviceType = (ELMETRIC_MetricServiceTypes)j;
      server = c + 1;
    }
    if(!ELMETRIC_ValidServer(serviceType, server)) {
      fprintf(stderr, "Unable to talk to server '%s'\n", server);
      return 1;
    }
    entity = NULL;
  }
  if(server != NULL && entity == NULL)
    ShowMetric(serviceType, server, server);

  return 0;

}
