/* $Id: bulktest.c,v 1.7 2004/03/26 06:52:53 jhayes Exp $ */
/*
 * Copyright  2004 The Regents of the University of California. 
 * All Rights Reserved. 
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for educational, research and non-profit purposes, without
 * fee, and without a written agreement is hereby granted, provided that the
 * above copyright notice, this paragraph and the following three paragraphs
 * appear in all copies. 
 *
 * Permission to incorporate this software into commercial products may be
 * obtained by contacting
 * Technology Transfer Office 
 * 9500 Gilman Drive 
 * 411 University Center 
 * University of California 
 * La Jolla, CA 92093-0093
 * (858) 534-5815
 * invent@ucsd.edu
 *
 * This software program and documentation are copyrighted by The Regents of
 * the University of California. The software program and documentation are
 * supplied "as is", without any accompanying services from The Regents. The
 * Regents does not warrant that the operation of the program will be
 * uninterrupted or error-free. The end-user understands that the program was
 * developed for research purposes and is advised not to rely exclusively on
 * the program for any reason. 
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
 * LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION,
 * EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED
 * HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO
 * OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
 * MODIFICATIONS. 
 */

 /*
  * This is a test program for the ELAGI bulk copy services found in bulkcopy.
  * It takes no arguments, but it does read server information from the file
  * servers.xml (see testutils.h) if available.
  */

#include <stdio.h>  /* File functions */
#include <stdlib.h> /* atoi free malloc */
#include <oshseed.h>
#define ASSTR_SHORT_NAMES
#include <strseed.h>
#define ASXML_SHORT_NAMES
#include <xmlseed.h>
#include "config.h"
#include "elbase.h"
#include "bulkcopy.h"
#include "testutils.h"


int
main(int argc,
     char **argv) {

  char *diskName;
  XmlNode diskNode;
  const char *diskServer;
  ELBASE_StorageServiceTypes diskService;
  XmlNode doc;
  FILE *f;
  XmlNode hostNode;
  const char *hostServer;
  ELBASE_ComputeServiceTypes hostService;
  unsigned i;
  unsigned j;
  unsigned long idx;
  ELBC_BulkLink link;
  int passed;
  ELBASE_Process pid;
  const char **rmArgs;
  XmlNodeList servers;
  int status;

  if((f = fopen("servers.xml", "r")) == NULL)
    doc = ImplementationCreateDocument(NULL, NULL, "doc", NULL);
  else {
    doc = ParseServers(f, stderr);
    fclose(f);
    if(doc == NULL)
      return 1;
  }
 
  servers = DocumentGetElementsByTagName(doc, "host");

  for(i = 0; (hostNode = NodeListItem(servers, i)) != NULL; i++) {

    /* Skip to next interactive host that has a "disk" attribute. */
    if(*(diskName = ElementGetAttribute(hostNode, "disk")) == '\0')
      continue;
    hostNode = NodeFirstChild(hostNode);
    if(hostNode == NULL ||
       !StrArrayFind(COMPUTE_TAGS, NodeName(hostNode), &idx) ||
       NodeNextSibling(hostNode) != NULL) /* Batch host */
      continue;

    hostServer = ElementGetAttribute(hostNode, "server");
    hostService = (ELBASE_ComputeServiceTypes)idx;

    if((diskNode = FindElementByAttribute(doc, "id", diskName)) == NULL ||
       (diskNode = NodeFirstChild(diskNode)) == NULL ||
       !StrArrayFind(STORAGE_TAGS, NodeName(diskNode), &idx)) {
      fprintf(stdout, "Error: missing disk '%s'\n", diskName);
      continue;
    }

    diskServer = ElementGetAttribute(diskNode, "server");
    diskService = (ELBASE_StorageServiceTypes)idx;

    /* Establish a bulk link between local and remote host. */
    link = ELBC_BulkLinkNew
      (ELBASE_CP, "", ELBASE_FORK, "",
       diskService, diskServer, hostService, hostServer);
    if(link == NULL) {
      fprintf(stdout, "Error: unable to establish link to '%s'\n", diskName);
      continue;
    }

    /* Create some local source files. */
    for(j = 0; SOURCE_PATHS[j] != NULL; j++)
      CreateTestFile(SOURCE_PATHS[j]);

    /* Test outbound bulk copy. */
    passed = ELBC_BulkCopy(link, SOURCE_PATHS, DEST_PATHS,
                           StrArrayLen(SOURCE_PATHS), &pid) &&
             ELBASE_Poll(pid, 1, &status) &&
             status == 0;
    fprintf(stdout, "%s bulk copy to %s (%s)\n",
            passed ? "Pass" : "Fail", STORAGE_TAGS[diskService], diskName);
    ELBC_BulkLinkFree(link);

    /* Test inbound bulk copy. */
    link = ELBC_BulkLinkNew
      (diskService, diskServer, hostService, hostServer,
       ELBASE_CP, "", ELBASE_FORK, "");
    if(link == NULL)
      fprintf(stdout, "Error: unable to establish link from '%s'\n", diskName);
    else {
      passed = ELBC_BulkCopy(link, DEST_PATHS, SOURCE_PATHS,
                             StrArrayLen(SOURCE_PATHS), &pid) &&
               ELBASE_Poll(pid, 1, &status) &&
               status == 0;
      fprintf(stdout, "%s bulk copy from %s (%s)\n",
              passed ? "Pass" : "Fail", STORAGE_TAGS[diskService], diskName);
      passed = 1;
      for(j = 0; SOURCE_PATHS[j] != NULL; j++) {
        if(!CheckTestFile(SOURCE_PATHS[j]))
          passed = 0;
      }
      fprintf(stdout, "%s content check from %s (%s)\n",
              passed ? "Pass" : "Fail", STORAGE_TAGS[diskService], diskName);
      ELBC_BulkLinkFree(link);
    }

    /* Clean up test files. */
    for(j = 0; SOURCE_PATHS[j] != NULL; j++)
      unlink(SOURCE_PATHS[j]);
    rmArgs = (const char **)
      malloc((StrArrayLen(DEST_PATHS) + 2) * sizeof(char *));
    rmArgs[0] = "-f";
    for(j = 0; DEST_PATHS[j] != NULL; j++)
      rmArgs[j + 1] = DEST_PATHS[j];
    rmArgs[j + 1] = NULL;
    ELBASE_Spawn(hostService, hostServer, "rm", NULL, rmArgs, NULL, NULL, NULL,
                 NULL, &pid);
    ELBASE_Poll(pid, 1, NULL);
    free(rmArgs);

  }

  return 0;

}
