/* $Id: copytest.c,v 1.7 2004/03/24 19:41:51 jhayes Exp $ */
/*
 * Copyright  2004 The Regents of the University of California. 
 * All Rights Reserved. 
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for educational, research and non-profit purposes, without
 * fee, and without a written agreement is hereby granted, provided that the
 * above copyright notice, this paragraph and the following three paragraphs
 * appear in all copies. 
 *
 * Permission to incorporate this software into commercial products may be
 * obtained by contacting
 * Technology Transfer Office 
 * 9500 Gilman Drive 
 * 411 University Center 
 * University of California 
 * La Jolla, CA 92093-0093
 * (858) 534-5815
 * invent@ucsd.edu
 *
 * This software program and documentation are copyrighted by The Regents of
 * the University of California. The software program and documentation are
 * supplied "as is", without any accompanying services from The Regents. The
 * Regents does not warrant that the operation of the program will be
 * uninterrupted or error-free. The end-user understands that the program was
 * developed for research purposes and is advised not to rely exclusively on
 * the program for any reason. 
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
 * LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION,
 * EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED
 * HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO
 * OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
 * MODIFICATIONS. 
 */

 /*
  * This is a test program for the ELAGI file copy services found in elbase.
  * It takes no arguments, but it does read server information from the file
  * servers.xml (see testutils.h) if available.  NOTE: this program will leave
  * test files in the home directory of remote servers.
  */

#include <stdio.h>  /* file functions */
#include <stdlib.h> /* atoi */
#include <oshseed.h>
#define ASSTR_SHORT_NAMES
#include <strseed.h>
#define ASXML_SHORT_NAMES
#include <xmlseed.h>
#include "config.h"
#include "elbase.h"
#include "testutils.h"


int
main(int argc,
     char **argv) {

  XmlNode doc;
  FILE *f;
  unsigned i;
  unsigned long idx;
  XmlNode n;
  int passed;
  ELBASE_Process pid;
  const char *server;
  XmlNodeList servers;
  ELBASE_StorageServiceTypes service;
  int status;

  if((f = fopen("servers.xml", "r")) == NULL)
    doc = ImplementationCreateDocument(NULL, NULL, "doc", NULL);
  else {
    doc = ParseServers(f, stderr);
    fclose(f);
    if(doc == NULL)
      return 1;
  }
 
  /* Add local disk if XML doesn't define it. */
  if(FindNodeByChildTag(doc, STORAGE_TAGS[ELBASE_CP]) == NULL) {
    n = NodeAppendChild(doc, DocumentCreateElement(doc, "storage"));
    n = NodeAppendChild(n, DocumentCreateElement(doc, "disk"));
    ElementSetAttribute(n, "id", "_local");
    n = NodeAppendChild
      (n, DocumentCreateElement(doc, (char *)STORAGE_TAGS[ELBASE_CP]));
    ElementSetAttribute(n, "server", "localhost");
  }

  servers = DocumentGetElementsByTagName(doc, "disk");

  for(i = 0; (n = NodeListItem(servers, i)) != NULL; i++) {
    char destPath[256];
    n = NodeFirstChild(n);
    if(n == NULL || !StrArrayFind(STORAGE_TAGS, NodeName(n), &idx))
      continue;
    server = ElementGetAttribute(n, "server");
    service = (ELBASE_StorageServiceTypes)idx;
    CreateTestFile(DEST_PATHS[0]);
    sprintf(destPath, "%s%s%s",
            service == ELBASE_GASS ? "/tmp/" : "", DEST_PATHS[0], NodeName(n));
    /* Copy, delete, copy back, and test content. */
    passed = ELBASE_CopyFile(ELBASE_CP, NULL, DEST_PATHS[0], service, server,
                             destPath, &pid) &&
             ELBASE_Poll(pid, 1, &status) &&
             unlink(DEST_PATHS[0]) == 0 &&
             status == 0 &&
             ELBASE_CopyFile(service, server, destPath, ELBASE_CP, NULL,
                             DEST_PATHS[0], &pid) &&
             ELBASE_Poll(pid, 1, &status) &&
             status == 0 &&
             CheckTestFile(DEST_PATHS[0]);
    fprintf(stdout, "%s copy via %s (%s)\n", passed ? "Pass" : "Fail",
            NodeName(n), ElementGetAttribute(NodeParentNode(n), "id"));
    unlink(DEST_PATHS[0]);
    passed = ELBASE_DeleteFile(service, server, destPath, &pid) &&
             ELBASE_Poll(pid, 1, &status) &&
             status == 0 &&
             ELBASE_CopyFile(service, server, destPath, ELBASE_CP, NULL,
                             DEST_PATHS[0], &pid) &&
             ELBASE_Poll(pid, 1, &status) &&
             unlink(DEST_PATHS[0]) != 0;
    fprintf(stdout, "%s delete via %s (%s)\n", passed ? "Pass" : "Fail",
            NodeName(n), ElementGetAttribute(NodeParentNode(n), "id"));
  }

  /* Report untested storage services. */
  for(i = 0; i < ELBASE_STORAGE_SERVICE_COUNT; i++) {
    if(FindNodeByChildTag(doc, STORAGE_TAGS[i]) == NULL)
      fprintf(stdout, "N/A copy via %s\n", STORAGE_TAGS[i]);
  }

  NodeListFree(servers);
  NodeFree(doc);

  return 0;

}
