/* $Id: spawntest.c,v 1.10 2004/03/19 18:40:52 jhayes Exp $ */
/*
 * Copyright  2004 The Regents of the University of California. 
 * All Rights Reserved. 
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for educational, research and non-profit purposes, without
 * fee, and without a written agreement is hereby granted, provided that the
 * above copyright notice, this paragraph and the following three paragraphs
 * appear in all copies. 
 *
 * Permission to incorporate this software into commercial products may be
 * obtained by contacting
 * Technology Transfer Office 
 * 9500 Gilman Drive 
 * 411 University Center 
 * University of California 
 * La Jolla, CA 92093-0093
 * (858) 534-5815
 * invent@ucsd.edu
 *
 * This software program and documentation are copyrighted by The Regents of
 * the University of California. The software program and documentation are
 * supplied "as is", without any accompanying services from The Regents. The
 * Regents does not warrant that the operation of the program will be
 * uninterrupted or error-free. The end-user understands that the program was
 * developed for research purposes and is advised not to rely exclusively on
 * the program for any reason. 
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
 * LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION,
 * EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED
 * HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO
 * OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
 * MODIFICATIONS. 
 */

 /*
  * This is a test program for the ELAGI process spawning services found in
  * elbase.  It takes no arguments, but it does read server information from
  * the file servers.xml if available.
  * file.
  */

#include <stdio.h>  /* File functions */
#include <oshseed.h>
#define ASSTR_SHORT_NAMES
#include <strseed.h>
#define ASXML_SHORT_NAMES
#include <xmlseed.h>
#include "config.h"
#include "elbase.h"
#include "testutils.h"


int
main(int argc,
     char **argv) {

  XmlNode doc;
  FILE *f;
  unsigned i;
  unsigned long idx;
  XmlNode n;
  int passed;
  ELBASE_Process pid;
  const char *server;
  XmlNodeList servers;
  ELBASE_ComputeServiceTypes service;
  int status;

  if((f = fopen("servers.xml", "r")) == NULL)
    doc = ImplementationCreateDocument(NULL, NULL, "doc", NULL);
  else {
    doc = ParseServers(f, stderr);
    fclose(f);
    if(doc == NULL)
      return 1;
  }
 
  if(FindNodeByChildTag(doc, COMPUTE_TAGS[ELBASE_FORK]) == NULL) {
    n = NodeAppendChild(doc, DocumentCreateElement(doc, "compute"));
    n = NodeAppendChild(n, DocumentCreateElement(doc, "host"));
    ElementSetAttribute(n, "id", "_local");
    n = NodeAppendChild
      (n, DocumentCreateElement(doc, (char *)COMPUTE_TAGS[ELBASE_FORK]));
    ElementSetAttribute(n, "server", "localhost");
  }

  servers = DocumentGetElementsByTagName(doc, "host");

  for(i = 0; (n = NodeListItem(servers, i)) != NULL; i++) {
    const char *badEnv[] = {"ELFILE=/notthere", NULL};
    const char *falseTestArgs[] = {"1", "!=", "1", NULL};
    const char *goodEnv[] = {"ELFILE=/etc/passwd", NULL};
    const char *trueTestArgs[] = {"1", "=", "1", NULL};
    n = NodeFirstChild(n);
    if(n == NULL ||
       !StrArrayFind(COMPUTE_TAGS, NodeName(n), &idx) ||
       NodeNextSibling(n) != NULL) /* batch host */
      continue;
    server = ElementGetAttribute(n, "server");
    service = (ELBASE_ComputeServiceTypes)idx;
    passed = ELBASE_Spawn(service, server, "true", NULL, NULL, NULL, NULL,
                          NULL, NULL, &pid) &&
             ELBASE_Poll(pid, 1, &status) &&
             status == 0 &&
             ELBASE_Spawn(service, server, "false", NULL, NULL, NULL, NULL,
                          NULL, NULL, &pid) &&
             ELBASE_Poll(pid, 1, &status) &&
             status != 0;
    fprintf(stdout, "%s compute via %s (%s)\n", passed ? "Pass" : "Fail",
            NodeName(n), ElementGetAttribute(NodeParentNode(n), "id"));
    passed = ELBASE_Spawn(service, server, "true", "/tmp", NULL, NULL, NULL,
                          NULL, NULL, &pid) &&
             ELBASE_Poll(pid, 1, &status) &&
             status == 0 &&
             ELBASE_Spawn(service, server, "true", "/notthere", NULL, NULL,
                          NULL, NULL, NULL, &pid) &&
             ELBASE_Poll(pid, 1, &status) &&
             status != 0;
    fprintf(stdout, "%s wd via %s (%s)\n", passed ? "Pass" : "Fail",
            NodeName(n), ElementGetAttribute(NodeParentNode(n), "id"));
    passed = ELBASE_Spawn(service, server, "test", NULL, trueTestArgs, NULL,
                          NULL, NULL, NULL, &pid) &&
             ELBASE_Poll(pid, 1, &status) &&
             status == 0 &&
             ELBASE_Spawn(service, server, "test", NULL, falseTestArgs, NULL,
                          NULL, NULL, NULL, &pid) &&
             ELBASE_Poll(pid, 1, &status) &&
             status != 0;
    fprintf(stdout, "%s args via %s (%s)\n", passed ? "Pass" : "Fail",
            NodeName(n), ElementGetAttribute(NodeParentNode(n), "id"));
    passed = ELBASE_Spawn(service, server, "wc", NULL, NULL,
                          "/etc/passwd", "/dev/null", NULL, NULL, &pid) &&
             ELBASE_Poll(pid, 1, &status) &&
             status == 0 &&
             ELBASE_Spawn(service, server, "wc", NULL, NULL,
                          "/notthere", NULL, "/dev/null", NULL, &pid) &&
             ELBASE_Poll(pid, 1, &status) &&
             status != 0;
    fprintf(stdout, "%s stdin via %s (%s)\n", passed ? "Pass" : "Fail",
            NodeName(n), ElementGetAttribute(NodeParentNode(n), "id"));
    passed = ELBASE_Spawn(service, server, "uname", NULL, NULL, NULL,
                          "/tmp/elout1", NULL, NULL, &pid) &&
             ELBASE_Poll(pid, 1, &status) &&
             status == 0 &&
             ELBASE_Spawn(service, server, "wc", NULL, NULL,
                          "/tmp/elout1", "/dev/null", NULL, NULL, &pid) &&
             ELBASE_Poll(pid, 1, &status) &&
             status == 0;
    fprintf(stdout, "%s stdout via %s (%s)\n", passed ? "Pass" : "Fail",
            NodeName(n), ElementGetAttribute(NodeParentNode(n), "id"));
    passed = ELBASE_Spawn(service, server, "sleep", NULL, NULL, NULL, NULL,
                          "/tmp/elout2", NULL, &pid) &&
             ELBASE_Poll(pid, 1, &status) &&
             status != 0 &&
             ELBASE_Spawn(service, server, "wc", NULL, NULL,
                          "/tmp/elout2", "/dev/null", NULL, NULL, &pid) &&
             ELBASE_Poll(pid, 1, &status) &&
             status == 0;
    fprintf(stdout, "%s stderr via %s (%s)\n", passed ? "Pass" : "Fail",
            NodeName(n), ElementGetAttribute(NodeParentNode(n), "id"));
    passed = ELBASE_Spawn(service, server, "wc", NULL, NULL,
                          "${ELFILE}", "/dev/null", NULL, goodEnv, &pid) &&
             ELBASE_Poll(pid, 1, &status) &&
             status == 0 &&
             ELBASE_Spawn(service, server, "wc", NULL, NULL,
                          "${ELFILE}", NULL, "/dev/null", badEnv, &pid) &&
             ELBASE_Poll(pid, 1, &status) &&
             status != 0;
    fprintf(stdout, "%s env via %s (%s)\n", passed ? "Pass" : "Fail",
            NodeName(n), ElementGetAttribute(NodeParentNode(n), "id"));
  }
  for(i = 0; i < ELBASE_COMPUTE_SERVICE_COUNT; i++) {
    if(FindNodeByChildTag(doc, COMPUTE_TAGS[i]) == NULL)
      fprintf(stdout, "N/A compute via %s\n", COMPUTE_TAGS[i]);
  }

  return 0;

}
