/* $Id: gmingrp.c,v 1.12 2004/03/19 00:09:56 jhayes Exp $ */
/*
 * Copyright  2004 The Regents of the University of California. 
 * All Rights Reserved. 
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for educational, research and non-profit purposes, without
 * fee, and without a written agreement is hereby granted, provided that the
 * above copyright notice, this paragraph and the following three paragraphs
 * appear in all copies. 
 *
 * Permission to incorporate this software into commercial products may be
 * obtained by contacting
 * Technology Transfer Office 
 * 9500 Gilman Drive 
 * 411 University Center 
 * University of California 
 * La Jolla, CA 92093-0093
 * (858) 534-5815
 * invent@ucsd.edu
 *
 * This software program and documentation are copyrighted by The Regents of
 * the University of California. The software program and documentation are
 * supplied "as is", without any accompanying services from The Regents. The
 * Regents does not warrant that the operation of the program will be
 * uninterrupted or error-free. The end-user understands that the program was
 * developed for research purposes and is advised not to rely exclusively on
 * the program for any reason. 
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
 * LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION,
 * EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED
 * HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO
 * OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
 * MODIFICATIONS. 
 */


#include "config.h"
#include <oshseed.h>
#include <stdio.h>  /* file funcs */
#include <stdlib.h> /* free malloc */
#include <ipseed.h>
#define ASSTR_SHORT_NAMES
#include <strseed.h>
#define ASV_SHORT_NAMES
#include <vectorseed.h>
#include "gminscripts.h"
#include "gmingrp.h"


struct GMINGRP__GroupInfo {
  char **contacts;
  char *gmindPath;
  char *initFilePath;
  ELBASE_Process *pids;
  unsigned port;
};


int
GMINGRP_GroupAdd(GMINGRP_GroupInfo group,
                 ELBASE_StorageServiceTypes storageType,
                 const char *storageServer,
                 ELBASE_ComputeServiceTypes computeType,
                 const char *computeServer) {

  const char *args[] = {NULL, "-p", NULL, "-f", NULL, "-d", NULL};
  FILE *f;
  unsigned i;
  char image[20];
  ELBASE_Process pid;
  ASIP_Address serverAddr;
  char *serverCopy;
  ASIP_Socket sock;
  int status;

  sprintf(image, "%d", (int)group->port);
  args[0] = group->gmindPath;
  args[2] = image;
  args[4] = group->initFilePath;

  if(computeType == ELBASE_SSH && strchr(computeServer, ';') != NULL)
    return 0; /* Hopped servers not supported. */
  if(storageType != ELBASE_CP &&
     (!ELBASE_CopyFile(ELBASE_CP, NULL, group->gmindPath, storageType,
                      storageServer, group->gmindPath, &pid) ||
      !ELBASE_Poll(pid, 1, &status) ||
      status != 0))
    return 0;
  if(!ELBASE_Spawn(computeType, computeServer, "perl", NULL, args,
                   NULL, NULL, NULL, NULL, &pid))
    return 0;

  serverCopy = strdup(computeServer);
  if(strchr(serverCopy, '/') != NULL)
    *strchr(serverCopy, '/') = '\0';
  if(strchr(serverCopy, ':') != NULL)
    *strchr(serverCopy, ':') = '\0';
  ASIP_AddressValue(serverCopy, &serverAddr);
  free(serverCopy);
  while(!ELBASE_Poll(pid, 0, NULL) &&
        (sock = ASIP_ConnectToPort(serverAddr, (ASIP_Port)group->port)) ==
         ASIP_NO_SOCKET)
    sleep(1);
  if(sock == ASIP_NO_SOCKET)
    return 0;
  ASIP_Disconnect(&sock);

  Append(group->pids, pid);
  Append(group->contacts, StrAppend(NULL, computeServer, ":", image, NULL));
  if((f = fopen(group->initFilePath, "w")) != NULL) {
    for(i = 1; i < Size(group->contacts); i++)
      fprintf(f, "%s\n", group->contacts[i]);
    fclose(f);
  }
  return 1;

}


void
GMINGRP_GroupFree(GMINGRP_GroupInfo group) {
  unsigned i;
  for(i = 0; i < Size(group->contacts); i++) {
    free(group->contacts[i]);
    ELBASE_Kill(group->pids[i]);
    ELBASE_Poll(group->pids[i], 1, NULL);
  }
  unlink(group->gmindPath);
  unlink(group->initFilePath);
  free(group->gmindPath);
  free(group->initFilePath);
  VectorFree(group->contacts);
  VectorFree(group->pids);
  free(group);
}


GMINGRP_GroupInfo
GMINGRP_GroupNew(unsigned port) {

  FILE *f;
  char image[256];
  GMINGRP_GroupInfo result =
    (GMINGRP_GroupInfo)malloc(sizeof(struct GMINGRP__GroupInfo));
  char *script;

  sprintf(image, "%d-%d", (int)getpid(), (int)port);
  result->contacts = VectorNew(char *);
  result->gmindPath = StrAppend(NULL, "./gmind", image, NULL);
  result->initFilePath = StrAppend(NULL, "gmindinit", image, NULL);
  result->pids = VectorNew(ELBASE_Process);
  result->port = port;

  if((f = fopen(result->gmindPath, "w")) == NULL) {
    GMINGRP_GroupFree(result);
    return NULL;
  }
  script = StrArrayJoin(gmind_script, "\n");
  fprintf(f, "%s\n", script);
  fclose(f);
  free(script);

  if(!GMINGRP_GroupAdd(result, ELBASE_CP, NULL, ELBASE_FORK, "localhost")) {
    GMINGRP_GroupFree(result);
    return NULL;
  }
  return result;

}


char *
GMINGRP_GroupServer(GMINGRP_GroupInfo group) {
  return group->contacts[0];
}
