/* $Id: gmingrp.h,v 1.4 2004/04/03 22:21:43 jhayes Exp $ */
/*
 * Copyright  2004 The Regents of the University of California. 
 * All Rights Reserved. 
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for educational, research and non-profit purposes, without
 * fee, and without a written agreement is hereby granted, provided that the
 * above copyright notice, this paragraph and the following three paragraphs
 * appear in all copies. 
 *
 * Permission to incorporate this software into commercial products may be
 * obtained by contacting
 * Technology Transfer Office 
 * 9500 Gilman Drive 
 * 411 University Center 
 * University of California 
 * La Jolla, CA 92093-0093
 * (858) 534-5815
 * invent@ucsd.edu
 *
 * This software program and documentation are copyrighted by The Regents of
 * the University of California. The software program and documentation are
 * supplied "as is", without any accompanying services from The Regents. The
 * Regents does not warrant that the operation of the program will be
 * uninterrupted or error-free. The end-user understands that the program was
 * developed for research purposes and is advised not to rely exclusively on
 * the program for any reason. 
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
 * LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION,
 * EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED
 * HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO
 * OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
 * MODIFICATIONS. 
 */


#ifndef GMINGRP_H
#define GMINGRP_H

/*
 * This package provides mechanisms for managing dynamic groups of
 * gmind's--daemons that use the Ganglia protocol to report a minimal set of
 * Grid metrics. Each daemon in a group listens to a port associated with that
 * group and reports metrics for its host; the local daemon also collects and
 * reports metrics for all other hosts in the group.  Thus, the local daemon
 * emulates Ganglia's gmetad and the remote daemons gmond.
 */

#ifdef __cplusplus
extern "C" {
#endif

#include "elbase.h"


/* Galglia's gmond default port. */
#define GMOND_DEFAULT_PORT 4646


/* Information about a group of gmind's. */
typedef struct GMINGRP__GroupInfo *GMINGRP_GroupInfo;


/*
 * Copies the necessary scripts to the #storageType# server #storageServer#
 * and starts a daemon in #group# on the #computeType# server #computeServer#.
 * Returns 1 if successful, else 0.
 */
int
GMINGRP_GroupAdd(GMINGRP_GroupInfo group,
                 ELBASE_StorageServiceTypes storageType,
                 const char *storageServer,
                 ELBASE_ComputeServiceTypes computeType,
                 const char *computeServer);


/* Terminates any daemons in #group#, then frees it. */
void
GMINGRP_GroupFree(GMINGRP_GroupInfo group);


/*
 * Returns a group running a daemon on the local machine.  That daemon, and any
 * daemons added subsequently, will listen on #port#.  Returns NULL on error.
 */
GMINGRP_GroupInfo
GMINGRP_GroupNew(unsigned port);


/*
 * Returns the contact string for the main server in #group#.  This server may
 * be contacted to retrieve information produced by all daemons in #group#.
 */
char *
GMINGRP_GroupServer(GMINGRP_GroupInfo group);


#ifdef __cplusplus
}
#endif


#endif
