/* $Id: rashgrp.h,v 1.18 2004/03/15 16:55:10 jhayes Exp $ */
/*
 * Copyright  2004 The Regents of the University of California. 
 * All Rights Reserved. 
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for educational, research and non-profit purposes, without
 * fee, and without a written agreement is hereby granted, provided that the
 * above copyright notice, this paragraph and the following three paragraphs
 * appear in all copies. 
 *
 * Permission to incorporate this software into commercial products may be
 * obtained by contacting
 * Technology Transfer Office 
 * 9500 Gilman Drive 
 * 411 University Center 
 * University of California 
 * La Jolla, CA 92093-0093
 * (858) 534-5815
 * invent@ucsd.edu
 *
 * This software program and documentation are copyrighted by The Regents of
 * the University of California. The software program and documentation are
 * supplied "as is", without any accompanying services from The Regents. The
 * Regents does not warrant that the operation of the program will be
 * uninterrupted or error-free. The end-user understands that the program was
 * developed for research purposes and is advised not to rely exclusively on
 * the program for any reason. 
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
 * LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION,
 * EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED
 * HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO
 * OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
 * MODIFICATIONS. 
 */


#ifndef RASHGRP_H
#define RASHGRP_H

/*
 * This package provides a mechanism for managing a group of RASH (Reverse
 * Access SHell) processes on a batch system.  A RASH group consists of a
 * proxy that runs on the head node and a set of agents which are launched via
 * a batch system and communicate with the proxy.  The client process can order
 * the agents, via the proxy, to run applications on the batch nodes.
 */

#ifdef __cplusplus
extern "C" {
#endif

#include "elbase.h"


/* Information about a rash proxy and associated agents. */
typedef struct RASHGRP__GroupInfo *RASHGRP_GroupInfo;


/*
 * Returns 1 or 0 depending on whether or not #group# has agents that have been
 * submitted to a batch queue but are not yet running.  Sets #status# to the
 * submission process status on error, else to 0.
 */
int
RASHGRP_AgentsPending(RASHGRP_GroupInfo group,
                      int *status);


/* Returns 1 or 0 depending on whether or not #group# has agents running. */
int
RASHGRP_AgentsRunning(RASHGRP_GroupInfo group);


/*
 * Returns a NULL-terminated allocated list of allocated contact strings for
 * all the agents running in #group#.
 */
char **
RASHGRP_GetAgentContacts(RASHGRP_GroupInfo group);


/* Terminates any proxy and agents in #group#, then frees it. */
void
RASHGRP_GroupFree(RASHGRP_GroupInfo group);


/*
 * Copies the necessary scripts to the #storageType# server #storageServer#
 * and returns information needed to start a proxy and agents on the
 * #computeType# server #computeServer#.  Any proxy started in #group# will
 * listen on #proxyPort#, and both proxys and agents will have
 * #workingDirectory# as their working directory.  Returns NULL on error.
 */
RASHGRP_GroupInfo
RASHGRP_GroupNew(ELBASE_StorageServiceTypes storageType,
                 const char *storageServer,
                 ELBASE_ComputeServiceTypes computeType,
                 const char *computeServer,
                 unsigned proxyPort,
                 const char *workingDirectory);


/* Initializes the module; must be called before any other function. */
int
RASHGRP_Init(void);


/* Returns 1 or 0 depending on whether or not #group# has a proxy running. */
int
RASHGRP_ProxyRunning(RASHGRP_GroupInfo group);


/* Terminates the module; call just before program exit. */
int
RASHGRP_Shutdown(void);


/* Launches a rash proxy in #group#.  Returns 1 on success, else 0. */
int
RASHGRP_StartProxy(RASHGRP_GroupInfo group);


/* Terminates any agents in #group#. */
void
RASHGRP_StopAgents(RASHGRP_GroupInfo group);


/* Terminates any proxy (killing any agents, as well) in #group#. */
void
RASHGRP_StopProxy(RASHGRP_GroupInfo group);


/*
 * Submits a #scheduler# batch request to start agents in #group#. #attributes#
 * contains batch attributes to be used in the submission; it must contain a
 * value for ELBASE_NODES (which indicates how many agents will be run).
 * Returns 1 on success, else 0.
 */
int
RASHGRP_SubmitAgents(RASHGRP_GroupInfo group,
                     ELBASE_SchedulerServiceTypes scheduler,
                     const char **attributes);


#ifdef __cplusplus
}
#endif


#endif
