const char *rash_script[] = {
  "# $Id: rash.pl,v 1.20 2004/03/24 19:30:48 jhayes Exp $",
  "",
  "my $USAGE = 'rash [-w] [name@host[:port] [command]]';",
  "",
  "# This is a rash client.  The name@host:port argument specifies the host to",
  "# connect to--the host:port portion is contact information for the rash daemon",
  "# and the name portion the host name.  (Instead of a name, a number can be used",
  "# to select a host positionally.)  The default is '0@localhost:7000', with the",
  "# leading '0' selecting the rash daemon.  If -w is specified, the client will",
  "# block until the specified host connects to the daemon; otherwise, it will",
  "# exit immediately with a non-zero value if the specified host is not already",
  "# connected.  If given a command, the client will execute that command and then",
  "# exit; otherwise, it will accept and execute commands interactively.  The",
  "# client exit code is the exit code of the last command completed.  When",
  "# running interactively, the '?' command will show the command history,",
  "# including the exit code for any commands that have completed.",
  "#",
  "# See rashd for description of the rash protocol.",
  "",
  "use IO::Select;",
  "use IO::Socket;",
  "",
  "my $CRLF = \"\\015\\012\";",
  "my $agent;",
  "my $command;",
  "my $foreground;",
  "my $hosts = '';",
  "my $pid;",
  "my %pidCommand;",
  "my %pidExit;",
  "my @pidOrder;",
  "my $sock;",
  "my $waitForHost;",
  "",
  "$waitForHost = shift if $#ARGV >= 0 && $ARGV[0] eq '-w';",
  "unshift @ARGV, '0@localhost' if $#ARGV < 0;",
  "die \"usage: $USAGE\" if $ARGV[0] !~ /^([^\\@]+)\\@([^:]+)(:(\\d+))?$/;",
  "$sock = IO::Socket->new",
  "  (Domain => AF_INET, PeerAddr => $2, PeerPort => defined $4 ? $4 : 7000)",
  "  or die \"socket: $!\";",
  "$agent = $1;",
  "shift and ($command = join(' ', @ARGV)) if $#ARGV > 0;",
  "",
  "$SIG{PIPE} = 'IGNORE';",
  "autoflush STDOUT 1;",
  "",
  "# Asks that #cmd# be executed; identifies the resulting process by #pid#.",
  "sub DoCommand {",
  "  my ($cmd, $pid) = @_;",
  "  my $host = $agent;",
  "  if($cmd=~/^(AUTOWHO|KILL \\S+|ME|QUIT|RENAME \\S+ \\S+|STOP \\S+|WHO)$/i) {",
  "    $cmd =~ s/(\\S+)//;",
  "    $cmd = uc($1) . $cmd;",
  "    $host = '_rashd';",
  "  }",
  "  if($cmd =~ /^KILL (\\S)/) {",
  "    $pidExit{$1} = 'K';",
  "    $foreground = undef if defined $foreground && $foreground eq $1;",
  "  } elsif($cmd !~ /^QUIT/) {",
  "    $pidCommand{$pid} = $cmd;",
  "    push(@pidOrder, $pid);",
  "    $cmd = \"RUN $host $pid $cmd\";",
  "  }",
  "  &Send($sock, $cmd);",
  "}",
  "",
  "# Disconnects from the daemon and exits with the appropriate code.",
  "sub Done {",
  "  my ($exitCode, $pid);",
  "  &DoCommand('QUIT');",
  "  $sock->close();",
  "  foreach $pid (@pidOrder) {",
  "    $exitCode = $pidExit{$pid}",
  "      if defined $pidExit{$pid} and $pidExit{$pid} ne 'K';",
  "  }",
  "  exit(defined $exitCode ? $exitCode : 0);",
  "}",
  "",
  "# Processes the newly-arrived #msg#.",
  "sub ProcessMessage {",
  "  my ($msg) = @_;",
  "  die \"Lost connection\\n\" if !defined $msg;",
  "  return if $msg !~ s/^(STDERR|STDOUT|FINISHED) (\\S+) //;",
  "  return if defined $pidExit{$2};",
  "  return print STDERR $msg if $1 eq 'STDERR';",
  "  return $2 eq 'AW' ? ($hosts = $msg) : print STDOUT $msg if $1 eq 'STDOUT';",
  "  $foreground = undef if defined $foreground && $foreground eq $2;",
  "  $pidExit{$2} = $msg;",
  "}",
  "",
  "# Returns a message received on #from#, minus the CRLF terminator and with",
  "# special characters decoded.",
  "sub Receive {",
  "  my ($from) = @_;",
  "  my $buf = ' ';",
  "  my $msg = '';",
  "  $msg .= $buf while $msg !~ s/$CRLF$// && sysread($from, $buf, 1);",
  "  $msg =~ s/\\\\n/\\n/g;",
  "  $msg =~ s/\\\\\\\\/\\\\/g;",
  "  return $msg =~ /./ ? $msg : undef;",
  "}",
  "",
  "# Sends #msg#, followed by CRLF and with special characters encoded, to #to#.",
  "sub Send {",
  "  my ($to, $msg) = @_;",
  "  $msg =~ s/\\\\/\\\\\\\\/g;",
  "  $msg =~ s/\\n/\\\\n/g;",
  "  $to->print(\"$msg$CRLF\");",
  "}",
  "",
  "# Wait (-w) or test (no -w) for the presence of the named host.",
  "&DoCommand('AUTOWHO', 'AW');",
  "while(1) {",
  "  my @names;",
  "  &ProcessMessage(&Receive($sock));",
  "  last if \" $hosts \" =~ / $agent /;",
  "  # Interpret a numeric host as an index into the host list.",
  "  ($agent = $names[$agent]) and last",
  "     if $agent =~ /^\\d+/ && (@names = split(' ', $hosts)) > $agent + 0;",
  "  exit 1 if !defined $waitForHost;",
  "}",
  "",
  "if(!defined $command) {",
  "  my $prompt = defined $ENV{'RASHPROMPT'} ? $ENV{'RASHPROMPT'} : 'rash> ';",
  "  $SIG{INT} = sub {",
  "    &DoCommand(\"KILL $foreground\", undef) if defined $foreground;",
  "  };",
  "  while(1) {",
  "    print $prompt if !defined $foreground;",
  "    IO::Select->new(\\*STDIN, $sock)->can_read();",
  "    &ProcessMessage(&Receive($sock)) while IO::Select->new($sock)->can_read(0);",
  "    if(IO::Select->new(\\*STDIN)->can_read(0)) {",
  "      my $text = <STDIN>;",
  "      last if !defined $text;",
  "      if(defined $foreground) {",
  "        &Send($sock, \"STDIN $foreground $text\");",
  "      } else {",
  "        $text =~ s/(^\\s+)|(\\s+$)//g;",
  "        next if $text eq '';",
  "        if($text eq '?') {",
  "          foreach $pid (@pidOrder) {",
  "            print STDOUT \"[$pid] $pidCommand{$pid}\";",
  "            print STDOUT \" [$pidExit{$pid}]\" if defined $pidExit{$pid};",
  "            print STDOUT \"\\n\";",
  "          }",
  "          next;",
  "        }",
  "        last if $text eq 'quit';",
  "        $pid = $#pidOrder + 1;",
  "        if($text =~ s/\\s*&\\s*$//) {",
  "          print \"[$pid]\\n\";",
  "        } elsif($text !~ /^\\s*kill /i) {",
  "          $foreground = $pid;",
  "        }",
  "        &DoCommand($text, $pid);",
  "      }",
  "    }",
  "  }",
  "} else {",
  "  $SIG{INT} = \\&Done;",
  "  $pid = '0';",
  "  $command =~ s/(^\\s+)|(\\s+$)//g;",
  "  &DoCommand($command, $pid);",
  "  $pidExit{$pid} = $pid if $command eq 'quit';",
  "  &ProcessMessage(&Receive($sock)) while !defined $pidExit{$pid};",
  "}",
  "",
  "&Done();",
  0};

const char *rashd_script[] = {
  "# $Id: rashd.pl,v 1.17 2004/03/24 19:30:48 jhayes Exp $",
  "",
  "my $USAGE = \"rashd [-d] [-v] [-p port]\";",
  "",
  "# This is the rash daemon, normally run on the gateway to a set of batch or",
  "# firewall-protected resources.  The default port is 7000.  The daemon accepts",
  "# and transmits the following CRLF-terminated text messages.",
  "#",
  "# FINISHED pid code",
  "#   From agent; forwarded to client.  Indicates that process #pid# has exited",
  "#   with code #code#.",
  "#",
  "# KILL pid",
  "#   From client; forwarded to agent.  Requests that process #pid# be terminated.",
  "#",
  "# QUIT",
  "#   From agent or client.  Requests that the connection be closed.",
  "#",
  "# RUN agent pid command",
  "#   From client; forwarded to agent.  Requests that the agent #agent# be asked",
  "#   to run #command#.  #pid# identifies the resulting process.  The daemon",
  "#   itself serves as the agent for these commands:",
  "#   AUTOWHO",
  "#     Returns a STDOUT message that contains the list of connection names and",
  "#     sends a new STDOUT message every time the list changes.  Never completes;",
  "#     the client must kill the process to terminate it.",
  "#   ME",
  "#     Returns a STDOUT message that contains the client's connection name",
  "#     followed by a FINISHED 0 message.",
  "#   RENAME old new",
  "#     Requests that the connection #old# be reidentified with #new#.",
  "#   STOP name",
  "#     Requests that the connection to #name# be closed.",
  "#   WHO",
  "#     Returns a STDOUT message that contains the list of connection names",
  "#     followed by a FINISHED 0 message.",
  "#",
  "# STDERR pid text",
  "#   From agent; forwarded to client.  Indicates that process #pid# has written",
  "#   #text# on standard error.",
  "#",
  "# STDIN",
  "#   From client; forwarded to agent.  Indicates that #text# should be written",
  "#   to process #pid# on standard input.",
  "#   ",
  "# STDOUT",
  "#   From agent; forwarded to client.  Indicates that process #pid# has written",
  "#   #text# on standard output.",
  "",
  "use IO::Select;",
  "use IO::Socket;",
  "",
  "my $CRLF = \"\\015\\012\";",
  "my $connection;",
  "my %connections;",
  "my $count = 0;",
  "my $debug;",
  "my $ear;",
  "my %order;",
  "my %pidAgents;",
  "my %pidClients;",
  "my $port = 7000;",
  "my $verbose;",
  "",
  "$debug = shift if $#ARGV >= 0 && $ARGV[0] eq '-d';",
  "$verbose = shift if $#ARGV >= 0 && $ARGV[0] eq '-v';",
  "if($#ARGV >= 0 && $ARGV[0] eq '-p') {",
  "  shift;",
  "  die \"usage: $USAGE\" if !defined ($port = shift);",
  "}",
  "die \"usage: $USAGE\" if $#ARGV >= 0;",
  "$ear = IO::Socket->new",
  "  (Domain => AF_INET, ReuseAddr => 1, LocalPort => $port, Listen => SOMAXCONN)",
  "  or die \"listen: $!\";",
  "$connections{'_rashd'} = $ear;",
  "$order{$ear} = $count++;",
  "$SIG{HUP} = $SIG{PIPE} = 'IGNORE';",
  "",
  "# Reports connection population to all AUTOWHO pids.",
  "sub DoAutoWhos {",
  "  my $pid;",
  "  foreach $pid (keys %pidAgents) {",
  "    &Reply($pid, 'STDOUT', &SortNames(), undef) if $pidAgents{$pid} == $ear;",
  "  }",
  "}",
  "",
  "# Performs the daemon-specific command #cmd# newly arrived on #connection#.",
  "# Labels all command output with #pid#.",
  "sub DoCommand {",
  "  my ($connection, $pid, $cmd) = @_;",
  "  if($cmd eq 'AUTOWHO') {",
  "    &Reply($pid, 'STDOUT', &SortNames(), undef);",
  "  } elsif($cmd eq 'ME') {",
  "    &Reply($pid, 'STDOUT', &GetConnectionName($connection), 0);",
  "  } elsif($cmd =~ /^RENAME (\\S+) (\\S+)$/) {",
  "    return &Reply($pid, 'STDERR', \"Unknown host '$1'\", 1)",
  "      if !defined $connections{$1};",
  "    return &Reply($pid, 'STDERR', \"'$2' in use\", 1) if defined $connections{$2};",
  "    $connections{$2} = $connections{$1};",
  "    delete $connections{$1};",
  "    &Reply($pid, undef, undef, 0);",
  "    &DoAutoWhos();",
  "  } elsif($cmd =~ s/^STOP (\\S+)$//) {",
  "    return &Reply($pid, 'STDERR', \"Unknown host '$1'\", 1)",
  "      if !defined $connections{$1};",
  "    &Reply($pid, undef, undef, 0);",
  "    return &Forget($connections{$1}) if $connections{$1} != $ear;",
  "    close $ear;",
  "    exit 0;",
  "  } elsif($cmd eq 'WHO') {",
  "    &Reply($pid, 'STDOUT', &SortNames(), 0);",
  "  } else {",
  "    &Reply($pid, 'STDERR', \"Unknown command '$cmd'\", 1);",
  "  }",
  "}",
  "",
  "# Drops #connection# after killing any associated processes.",
  "sub Forget {",
  "  my ($connection) = @_;",
  "  my $pid;",
  "  foreach $pid (keys %pidAgents) {",
  "    next if $pidAgents{$pid} != $connection && $pidClients{$pid} != $connection;",
  "    &Send($pidAgents{$pid}, \"KILL $pid\")",
  "      if $pidAgents{$pid} != $ear && $pidAgents{$pid} != $connection;",
  "    $pid =~ /#(.*)$/;",
  "    &Send($pidClients{$pid}, \"FINISHED $1 255\")",
  "      if $pidClients{$pid} != $connection;",
  "    delete $pidAgents{$pid};",
  "    delete $pidClients{$pid};",
  "  }",
  "  $connection->close();",
  "  delete $connections{&GetConnectionName($connection)};",
  "  delete $order{$connection};",
  "  &DoAutoWhos();",
  "}",
  "",
  "# Returns the identifier associated with #connection#.",
  "sub GetConnectionName {",
  "  my ($connection) = @_;",
  "  my $name;",
  "  return if !defined $connection;",
  "  foreach $name (keys %connections) {",
  "    return $name if $connections{$name} == $connection;",
  "  }",
  "}",
  "",
  "# Responds to the message #msg# newly-arrived on #connection#.",
  "sub ProcessMessage {",
  "  my ($connection, $message) = @_;",
  "  my $pid;",
  "",
  "  return &Forget($connection) if !defined $message;",
  "  print STDOUT",
  "    \"Received '$message' from \" . &GetConnectionName($connection) . \"\\n\";",
  "",
  "  if($message =~ s/^FINISHED (\\S+) //) {",
  "    $pid = $1;",
  "    $pid =~ /#(.*)$/;",
  "    &Send($pidClients{$pid}, \"FINISHED $1 $message\")",
  "      if defined $pidClients{$pid};",
  "    delete $pidAgents{$pid};",
  "    delete $pidClients{$pid};",
  "  } elsif($message =~ /^KILL (\\S+)$/) {",
  "    $pid = &GetConnectionName($connection) . \"#$1\";",
  "    &Send($pidAgents{$pid}, \"KILL $pid\")",
  "      if defined $pidAgents{$pid} && $pidAgents{$pid} != $ear;",
  "    delete $pidAgents{$pid};",
  "    delete $pidClients{$pid};",
  "  } elsif($message eq 'QUIT') {",
  "    &Forget($connection);",
  "  } elsif($message =~ s/^RUN (\\S+) (\\S+) //) {",
  "    if(!defined $connections{$1}) {",
  "      &Send($connection, \"STDERR $2 Unknown host '$1'\\\\n\");",
  "      &Send($connection, \"FINISHED $2 255\");",
  "      return;",
  "    }",
  "    $pid = &GetConnectionName($connection) . \"#$2\";",
  "    &Send($pidAgents{$pid}, \"KILL $pid\")",
  "      if defined $pidAgents{$pid} && $pidAgents{$pid} != $ear;",
  "    $pidAgents{$pid} = $connections{$1};",
  "    $pidClients{$pid} = $connection;",
  "    if($pidAgents{$pid} == $ear) {",
  "      &DoCommand($pidClients{$pid}, $pid, $message);",
  "    } else {",
  "      &Send($pidAgents{$pid}, \"RUN $1 $pid $message\");",
  "    }",
  "  } elsif($message =~ s/^(STDERR|STDOUT) (\\S+) //) {",
  "    my $stream = $1;",
  "    $pid = $2;",
  "    $pid =~ /#(.*)$/;",
  "    &Send($pidClients{$pid}, \"$stream $1 $message\")",
  "      if defined $pidClients{$pid};",
  "  } elsif($message =~ s/^STDIN (\\S+) //) {",
  "    $pid = &GetConnectionName($connection) . \"#$1\";",
  "    &Send($pidAgents{$pid}, \"STDIN $pid $message\")",
  "      if defined $pidAgents{$pid} && $pidAgents{$pid} != $ear;",
  "  }",
  "",
  "}",
  "",
  "# Returns a message received on #from#, minus the CRLF terminator.",
  "sub Receive {",
  "  my ($from) = @_;",
  "  my $buf = ' ';",
  "  my $msg = '';",
  "  $msg .= $buf while $msg !~ s/$CRLF$// && sysread($from, $buf, 1);",
  "  return $msg =~ /./ ? $msg : undef;",
  "}",
  "",
  "# A convenience function for generating a pair of messages about process #pid#.",
  "# The first is #stream# output containing #output#; the second a FINISHED",
  "# message that contains #exitCode#.",
  "sub Reply {",
  "  my ($pid, $stream, $output, $exitCode) = @_;",
  "  &ProcessMessage($ear, \"$stream $pid $output\\\\n\") if defined $stream;",
  "  &ProcessMessage($ear, \"FINISHED $pid $exitCode\") if defined $exitCode;",
  "}",
  "",
  "# Sends #msg# followed by CRLF to #to#. (Separated into a function for debug.)",
  "sub Send {",
  "  my ($to, $msg) = @_;",
  "  print STDOUT \"$msg\\n\";",
  "  $to->print(\"$msg$CRLF\");",
  "}",
  "",
  "# Returns the set of connection names, sorted by connection order.",
  "sub SortNames {",
  "  return join(' ', sort {$order{$connections{$a}} - $order{$connections{$b}}}",
  "                   keys %connections);",
  "}",
  "",
  "if(!defined $debug) {",
  "  # Put ourselves into the background.",
  "  exit 0 if fork() != 0;",
  "  setpgrp;",
  "}",
  "if(!defined $verbose) {",
  "  # Suppress debugging output.",
  "  close STDIN;",
  "  open STDOUT, '/dev/null';",
  "  open STDERR, '/dev/null';",
  "}",
  "",
  "while(1) {",
  "  my $sel = IO::Select->new(values %connections);",
  "  foreach $connection ($sel->can_read()) {",
  "    print STDOUT \"Read from $connection\\n\";",
  "    if($connection == $ear) {",
  "      my $newClient = $ear->accept();",
  "      print STDERR \"accept: $!\" and next if !defined $newClient;",
  "      my ($port, $addr) = sockaddr_in($newClient->peername());",
  "      my $host = gethostbyaddr($addr, AF_INET);",
  "      print STDOUT \"Connection from $host:$port\\n\";",
  "      $connections{\"$host:$port\"} = $newClient;",
  "      $order{$newClient} = $count++;",
  "      &DoAutoWhos();",
  "    } else {",
  "      &ProcessMessage($connection, &Receive($connection));",
  "    }",
  "  }",
  "}",
  0};

const char *rasha_script[] = {
  "# $Id: rasha.pl,v 1.25 2004/04/03 22:24:36 jhayes Exp $",
  "",
  "my $USAGE = \"rasha [-d] [host[:port]]\";",
  "",
  "# This is the rash agent that runs on a resource not normally accessible from",
  "# outside.  The host:port command-line argument gives contact information for",
  "# the rash daemon; if absent, it defaults to localhost:7000.  See rashd for a",
  "# description of the protocol.",
  "#",
  "# The command portion of the RUN message supported by rasha has the format",
  "# [preamble ';' ...] path [switch ...].  Each preamble is either a working",
  "# directory specification ('cd' [path]), an environment variable setting ('set'",
  "# name [word]), or a redirection ('redirect' fd path).  The default working",
  "# directory path is the home directory.  Environment variable values may be",
  "# enclosed in either single or double quotes, and escaping with backslash is",
  "# supported.  A command can also contain environment variable references of the",
  "# form '${' name '}'.  To provide some minimal security, the agent restricts",
  "# the programs it runs to those in its working directory and subdirectories,",
  "# plus a few in (/usr)/bin known to be safe.",
  "",
  "use Cwd;",
  "use IO::Handle;",
  "use IO::Select;",
  "use IO::Socket;",
  "",
  "my $CRLF = \"\\015\\012\";",
  "my $buffer = '';",
  "my $debug;",
  "my %exitCodes;",
  "my $name;",
  "my $pid;",
  "my %pids;",
  "my $pipeName = 'PIPE0';",
  "my $sock;",
  "my %stderrs;",
  "my %stdins;",
  "my %stdouts;",
  "my $wordPat = q#'((\\\\.|[^'])*)'|\"((\\\\.|[^\"])*)\"|([^'\";\\s]+)#;",
  "",
  "$debug = shift if $#ARGV >= 0 && $ARGV[0] eq '-d';",
  "push @ARGV, 'localhost' if $#ARGV < 0;",
  "die \"$USAGE\" if $#ARGV != 0 || $ARGV[0] !~ /^([^:]+)(:(\\d+))?$/;",
  "$sock = IO::Socket->new",
  "  (Domain => AF_INET, PeerAddr => $1, PeerPort => defined $3 ? $3 : 7000)",
  "  or die \"socket: $!\";",
  "$sock->autoflush(1);",
  "$SIG{PIPE} = 'IGNORE';",
  "",
  "if(!defined $debug) {",
  "  open STDERR, '/dev/null';",
  "  open STDOUT, '/dev/null';",
  "}",
  "",
  "# Splits #text# into a list of quoted and unquoted words and returns the",
  "# result.  If #strip# is non-zero, quoted words are stripped of quotes and",
  "# escaped characters.",
  "sub GetWords {",
  "  my ($text, $strip) = @_;",
  "  my @result;",
  "  while($text =~ s/\\s*($wordPat)//) {",
  "    push(@result, $1) and next if !$strip || defined $6;",
  "    my $word = defined $2 ? $2 : $4;",
  "    $word =~ s/\\\\(.)/$1/g;",
  "    push(@result, $word);",
  "  }",
  "  return @result;",
  "}",
  "",
  "# Returns a message received on #from#, minus the CRLF terminator and with",
  "# special characters decoded.",
  "sub Receive {",
  "  my ($from) = @_;",
  "  my $buf = ' ';",
  "  my $msg = '';",
  "  $msg .= $buf while $msg !~ s/$CRLF$// && sysread($from, $buf, 1);",
  "  $msg =~ s/\\\\n/\\n/g;",
  "  $msg =~ s/\\\\\\\\/\\\\/g;",
  "  return $msg =~ /./ ? $msg : undef;",
  "}",
  "",
  "# Sends #msg#, followed by CRLF and with special characters encoded, to #to#.",
  "sub Send {",
  "  my ($to, $msg) = @_;",
  "  $msg =~ s/\\\\/\\\\\\\\/g;",
  "  $msg =~ s/\\n/\\\\n/g;",
  "  print STDOUT \"$msg\\n\";",
  "  $to->print(\"$msg$CRLF\");",
  "}",
  "",
  "while(1) {",
  "  IO::Select->new($sock, values %stderrs, values %stdouts)->can_read();",
  "  while(IO::Select->new($sock)->can_read(0)) {",
  "    my $msg = &Receive($sock);",
  "    if(!defined $msg) {",
  "      kill KILL, values %pids;",
  "      die \"Lost connection\\n\";",
  "    }",
  "    print STDOUT \"Received '$msg'\\n\";",
  "    if($msg =~ /^KILL (\\S+)/) {",
  "      kill(KILL, $pids{$1}) if defined $pids{$1};",
  "    } elsif($msg =~ s/^RUN \\S+ (\\S+) //) {",
  "      my ($errRead, $errWrite, $exitRead, $exitWrite, $inRead, $inWrite,",
  "          $outRead, $outWrite);",
  "      $name = $1;",
  "      pipe($errRead = ++$pipeName, $errWrite = ++$pipeName);",
  "      pipe($exitRead = ++$pipeName, $exitWrite = ++$pipeName);",
  "      pipe($inRead = ++$pipeName, $inWrite = ++$pipeName);",
  "      pipe($outRead = ++$pipeName, $outWrite = ++$pipeName);",
  "      if(($pid = fork()) == 0) {",
  "        # The blocking can_read() above will remain blocked when the command",
  "        # exits if both stderr and stdout are redirected.  To handle this case,",
  "        # we interpose another process that holds the pipes open, waits for the",
  "        # command to exit, and forwards the exit code.",
  "        if(($pid = fork()) > 0) {",
  "          wait();",
  "          exit $? >> 8;",
  "        }",
  "        open(STDERR, \">&$errWrite\");",
  "        open(STDIN,  \"<&$inRead\");",
  "        open(STDOUT, \">&$outWrite\");",
  "        close $errRead;",
  "        close $errWrite;",
  "        close $exitRead;",
  "        close $exitWrite;",
  "        close $inRead;",
  "        close $inWrite;",
  "        close $outRead;",
  "        close $outWrite;",
  "        while($msg =~ s/^((($wordPat)\\s*)+)(;\\s*|$)//) {",
  "          my $word;",
  "          my @words = &GetWords($1, 0);",
  "          foreach $word (@words) {",
  "            next if $word =~ /^'/;",
  "            $word = $1 . (defined $ENV{$2} ? $ENV{$2} : \"\") . $3",
  "              while $word =~ /^(.*)\\${([^}]+)}(.*)$/;",
  "          }",
  "          @words = &GetWords(join(' ', @words), 1);",
  "          if($words[0] eq 'cd' && defined $words[1]) {",
  "            chdir $words[1] or (print STDERR \"cd failed\\n\" and exit 1);",
  "          } elsif($words[0] eq 'cd') {",
  "            chdir or (print STDERR \"cd failed\\n\" and exit 1);",
  "          } elsif($words[0] eq 'set' and defined $words[1]) {",
  "            if(defined $words[2]) {",
  "              $ENV{$words[1]} = $words[2];",
  "            } else {",
  "              delete $ENV{$words[1]};",
  "            }",
  "          } elsif($words[0] eq 'redirect') {",
  "            print STDERR \"Missing fd\\n\" and exit 1 if !defined $words[1];",
  "            print STDERR \"Bad fd\\n\" and exit 1 if $words[1] !~ /^\\d+$/;",
  "            print STDERR \"Missing path\\n\" and exit 1 if !defined $words[2];",
  "            open(STDIN, \"<$words[2]\") or",
  "              (print STDERR \"stdin failed\\n\" and exit 1) if $words[1] eq '0';",
  "            open(STDOUT, \">$words[2]\") or",
  "              (print STDERR \"stdout failed\\n\" and exit 1) if $words[1] eq '1';",
  "            open(STDERR, \">$words[2]\") or",
  "              (print STDERR \"stderr failed\\n\" and exit 1) if $words[1] eq '2';",
  "          } else {",
  "            autoflush STDERR 1;",
  "            autoflush STDOUT 1;",
  "            if($words[0] eq '?\?') {",
  "              foreach $name (sort keys %pids) {",
  "                print STDOUT \"$name ($pids{$name})\" .",
  "                  (defined $exitCodes{$name} ? \" [$exitCodes{$name}]\\n\" : \"\\n\");",
  "              }",
  "              exit 0;",
  "            }",
  "            # Restrict execs to rasha wd & subdirs, plus a few known to be safe.",
  "            if($words[0] !~ m#/#) {",
  "              foreach $dir (split(':', $ENV{\"PATH\"})) {",
  "                $words[0] = \"$dir/$words[0]\" and last",
  "                  if -x \"$dir/$words[0]\" || -x \"$dir/$words[0].exe\";",
  "              }",
  "            }",
  "            while($words[0] =~ s#(^|/)\\./#$1#) { }",
  "            while($words[0] =~ s#[^/]+/\\.\\./##) { }",
  "            my $cwd = cwd();",
  "            $words[0] = \"$cwd/$words[0]\" if $words[0] !~ m#^/#;",
  "            print STDERR \"Permission denied\\n\" and exit 1",
  "              if $words[0] !~ m#^(/usr)?/bin/(date|df|false|hostname|ls|sleep|test|touch|true|pwd|uname|uptime|vmstat|w|wc|who)(\\.exe)?$# &&",
  "                 $words[0] !~ m#^$cwd#;",
  "            print STDERR \"Permission denied\\n\" and exit 1",
  "              if ! -x $words[0] && ! -x \"$words[0].exe\";",
  "            exec {$words[0]} @words or",
  "              (print STDERR \"exec failed\\n\" and exit 88);",
  "          }",
  "        }",
  "        exit 0; # No application to exec, just useless preamble.",
  "      }",
  "      $stderrs{$name} = IO::Handle->new_from_fd(fileno($errRead), \"r\");",
  "      $stdins{$name} = IO::Handle->new_from_fd(fileno($inWrite), \"w\");",
  "      $stdouts{$name} = IO::Handle->new_from_fd(fileno($outRead), \"r\");",
  "      $stdins{$name}->autoflush(1);",
  "      $pids{$name} = $pid;",
  "      close $errWrite;",
  "      close $exitWrite;",
  "      close $inRead;",
  "      close $outWrite;",
  "    } elsif($msg =~ s/STDIN (\\S+) //) {",
  "      $stdins{$1}->print($msg) if defined $stdins{$1};",
  "    }",
  "  }",
  "  foreach $name (keys %stderrs) {",
  "    next if !IO::Select->new($stderrs{$name})->can_read(0);",
  "    if($stderrs{$name}->sysread($buffer, 1024) > 0) {",
  "      print STDERR \"Read from stderrs{$name}\\n\";",
  "      &Send($sock, \"STDERR $name $buffer\");",
  "    } else {",
  "      print STDERR \"Close stderrs{$name}\\n\";",
  "      $stderrs{$name}->close();",
  "      delete $stderrs{$name};",
  "      # Drain stdout pipe before forwarding exit code.",
  "      &Send($sock, \"STDOUT $name $buffer\")",
  "        while defined $stdouts{$name} &&",
  "        $stdouts{$name}->sysread($buffer, 1024) > 0;",
  "      $stdins{$name}->close();",
  "      delete $stdins{$name};",
  "      waitpid($pids{$name}, 0);",
  "      $exitCodes{$name} = $?;",
  "      &Send($sock, \"FINISHED $name \". ($exitCodes{$name} >> 8));",
  "    }",
  "  }",
  "  foreach $name (keys %stdouts) {",
  "    next if !IO::Select->new($stdouts{$name})->can_read(0);",
  "    if($stdouts{$name}->sysread($buffer, 1024) > 0) {",
  "      print STDERR \"Read from stdouts{$name}\\n\";",
  "      &Send($sock, \"STDOUT $name $buffer\");",
  "    } else {",
  "      print STDERR \"Close stdouts{$name}\\n\";",
  "      $stdouts{$name}->close();",
  "      delete $stdouts{$name};",
  "    }",
  "  }",
  "}",
  "",
  "$sock->close();",
  0};
