#! /usr/bin/perl -w
# $Id: ggal.c,v 1.7 2002/07/10 00:01:47 abe Exp $

my $USAGE = 'tarp.pl -A|c|d|r|t|u|x [-Ovz] [-f archive] [-T file]';

# TODO: Comment

my $blockSize = 512;
my $verbosity = 0;

sub Bomb {
  my ($msg) = @_;
  print STDERR "tarp: $msg\n";
  exit(1);
}

sub GetStat {
  my ($path) = @_;
  my %result;
  my @status = stat($path);
  return undef if !defined($status[0]);
  ($result{'dev'}, $result{'ino'}, $result{'mode'}, $result{'nlink'},
   $result{'uid'}, $result{'gid'}, $result{'rdev'}, $result{'size'},
   $result{'atime'}, $result{'mtime'}, $result{'ctime'}, $result{'blksize'},
   $result{'blocks'}) = @status;
  return \%result;
}

sub HeaderFromBlock {
  my ($block) = @_;
  my %result;
  $result{'name'} = substr($block, 0, 100);
  $result{'mode'} = substr($block, 100, 8);
  $result{'uid'} = substr($block, 108, 8);
  $result{'gid'} = substr($block, 116, 8);
  $result{'size'} = substr($block, 124, 12);
  $result{'mtime'} = substr($block, 136, 12);
  $result{'chksum'} = substr($block, 148, 8);
  $result{'typeflag'} = substr($block, 156, 1);
  $result{'linkname'} = substr($block, 157, 100);
  $result{'magic'} = substr($block, 257, 6);
  $result{'version'} = substr($block, 263, 2);
  $result{'uname'} = substr($block, 265, 32);
  $result{'gname'} = substr($block, 297, 32);
  $result{'devmajor'} = substr($block, 329, 8);
  $result{'devminor'} = substr($block, 337, 8);
  $result{'prefix'} = substr($block, 345, 155);
  foreach $field (keys %result) {
    $result{$field} =~ s/\000//g;
    $result{$field} = oct($result{$field}) if $field !~ /chksum|name|link/;
  }
  return \%result;
}

sub StatToBlock {
  my ($path, $status) = @_;
  my $block = "\000" x $blockSize;
  my $ch;
  my $chksum = 0;
  my $formatted;
  my $name;
  substr($block, 0, length($path)) = $path;
  $formatted = sprintf("0%06o\000", $status->{'mode'});
  substr($block, 100, 8) = $formatted;
  $formatted = sprintf("0%06o\000", $status->{'uid'});
  substr($block, 108, 8) = $formatted;
  $formatted = sprintf("0%06o\000", $status->{'gid'});
  substr($block, 116, 8) = $formatted;
  $formatted = sprintf("0%010o\000", $status->{'size'});
  substr($block, 124, 12) = $formatted;
  $formatted = sprintf("0%010o\000", $status->{'mtime'});
  substr($block, 136, 12) = $formatted;
  substr($block, 148, 8) = ' ' x 8;
  substr($block, 156, 1) = '0';
  substr($block, 257, 6) = 'ustar ';
  substr($block, 263, 2) = " \000";
  $name = (getpwuid($status->{'uid'}))[0];
  substr($block, 265, length($name)) = $name;
  $name = (getgrgid($status->{'gid'}))[0];
  substr($block, 297, length($name)) = $name;
  foreach $ch (split(//, $block)) {
    $chksum += ord($ch);
  }
  $formatted = sprintf("0%o\000", $chksum);
  substr($block, 148, length($formatted)) = $formatted;
  return $block;
}

my $action;
my $archive;
my $block = '';
my $compressed = 0;
my $extractToStdout = 0;
my $header;
my $path;
my @paths;
my $pathStat;
my $size;

$ARGV[0] = "-$ARGV[0]" if $#ARGV >= 0 && $ARGV[0] !~ /^-/;
while($#ARGV >= 0) {
  my $arg = shift;
  push(@paths, $arg) and next if $arg !~ s/^-//;
  while($arg =~ /./) {
    if($arg =~ s/^(A|c|d|r|t|u|x)//) {
      &Bomb('You may not specify more than one -Acdtrux option')
        if defined($action);
      $action = $1;
    } elsif($arg =~ s/^f//) {
      $archive = ($arg =~ /./ ? $arg : $#ARGV >= 0 ? shift : &Bomb($USAGE));
      last;
    } elsif($arg =~ s/^O//) {
      $extractToStdout = 1;
    } elsif($arg =~ s/^T//) {
      my $file = ($arg =~ /./ ? $arg : $#ARGV >= 0 ? shift : &Bomb($USAGE));
      open(FILE, "<$file") or &Bomb("$file: cannot open");
      my @moreFiles = <FILE>;
      close(FILE);
      push(@paths, split(/\s+/, join(' ', @moreFiles)));
      last;
    } elsif($arg =~ s/^v//) {
      $verbosity++;
    } elsif($arg =~ s/^z//) {
      $compressed = 1;
    } else {
      &Bomb($USAGE);
    }
  }
}
&Bomb($USAGE) if !defined($action);

if($action =~ /A|c|r|u/) {

  if(!defined($archive)) {
    open(ARCHIVE, ">&STDOUT");
  } elsif($action ne 'c' && -e $archive) {
    open(ARCHIVE, "+<$archive") or &Bomb("$archive: cannot open");
    while(sysread(ARCHIVE, $block, $blockSize) > 0) {
      $header = &HeaderFromBlock($block);
      last if $header->{'name'} eq '';
      $size = $header->{'size'};
      $size -= $blockSize
        while $size > 0 && sysread(ARCHIVE, $block, $blockSize) > 0;
    }
    sysseek(ARCHIVE, -$blockSize, 1);
  } else {
    open(ARCHIVE, ">$archive") or &Bomb("$archive: cannot open");
  }
  foreach $path (@paths) {
    print "$path\n" if $verbosity > 0;
    print STDERR "tarp: $path: cannot stat\n" and next
      if !defined(($pathStat = &GetStat($path)));
    # TODO Test for links and dirs
    print STDERR "tarp: $path: cannot open\n" and next
      if !open(FILE, "<$path");
    $size = $pathStat->{'size'};
    $size -= 2 * $blockSize if $action eq 'A';
    syswrite(ARCHIVE, &StatToBlock($path, $pathStat), $blockSize)
      if($action ne 'A');
    syswrite(ARCHIVE, $block, $blockSize) and $size -= $blockSize
      while $size >= $blockSize && sysread(FILE, $block, $blockSize) > 0;
    syswrite(ARCHIVE, ($block .= "\000" x ($blockSize - $size)), $blockSize)
      if $size > 0 && sysread(FILE, $block, $size) > 0;
    close(FILE);
  }
  $block = "\000" x $blockSize;
  syswrite(ARCHIVE, $block, $blockSize);
  syswrite(ARCHIVE, $block, $blockSize);
  close(ARCHIVE);

} elsif($action =~ /d|t|x/) {

  my $pathPat = $#paths < 0 ? '*' : join('|', @paths);
  $pathPat =~ s/\./\\./g;
  $pathPat =~ s/\?/./g;
  $pathPat =~ s/\*/.*/g;
  $pathPat = "^($pathPat)\$";
  if(!defined($archive)) {
    open(ARCHIVE, "<&STDIN");
  } else {
    open(ARCHIVE, "<$archive") or &Bomb("$archive: cannot open");
  }

  while(sysread(ARCHIVE, $block, $blockSize) > 0) {
    my $writeBlocks = 0;
    $header = &HeaderFromBlock($block);
    last if ($path = $header->{'name'}) eq '';
    $size = $header->{'size'};
    if($path =~ /$pathPat/) {
      print "$path\n" if $verbosity > 0 || $action eq 't';
      if($action eq 'd') {
        if(defined($pathStat = &GetStat($path))) {
          foreach $field (sort keys %$pathStat) {
            print "$path: $field differs\n"
              if defined($header->{$field}) &&
                 $pathStat->{$field} != $header->{$field};
          }
        } else {
          print STDERR "tarp: $path: cannot stat\n";
        }
      } elsif($action eq 'x') {
        my $dir = '';
        while($path =~ s#^([^/]+/)##) {
          $dir .= $1;
          mkdir($dir) if ! -d $dir;
        }
        if($path eq '') {
          # empty
        } elsif(open(FILE, $extractToStdout ? ">&STDOUT" : ">$dir$path")) {
          $writeBlocks = 1;
        } else {
          print STDERR "tarp: $dir$path: cannot open\n";
        }
      }
    }
    while($size > 0 && sysread(ARCHIVE, $block, $blockSize) > 0) {
      syswrite(FILE, $block, $size > $blockSize ? $blockSize : $size)
        if($writeBlocks);
      $size -= $blockSize;
    }
    close(FILE) if $writeBlocks;
  }
  close(ARCHIVE);

}
