/* $Id: bulkcopy.h,v 1.3 2004/03/04 19:10:00 jhayes Exp $ */
/*
 * Copyright  2004 The Regents of the University of California. 
 * All Rights Reserved. 
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for educational, research and non-profit purposes, without
 * fee, and without a written agreement is hereby granted, provided that the
 * above copyright notice, this paragraph and the following three paragraphs
 * appear in all copies. 
 *
 * Permission to incorporate this software into commercial products may be
 * obtained by contacting
 * Technology Transfer Office 
 * 9500 Gilman Drive 
 * 411 University Center 
 * University of California 
 * La Jolla, CA 92093-0093
 * (858) 534-5815
 * invent@ucsd.edu
 *
 * This software program and documentation are copyrighted by The Regents of
 * the University of California. The software program and documentation are
 * supplied "as is", without any accompanying services from The Regents. The
 * Regents does not warrant that the operation of the program will be
 * uninterrupted or error-free. The end-user understands that the program was
 * developed for research purposes and is advised not to rely exclusively on
 * the program for any reason. 
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
 * LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION,
 * EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED
 * HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO
 * OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
 * MODIFICATIONS. 
 */


#ifndef BULKCOPY_H
#define BULKCOPY_H

/*
 * This package provides a method for copying multiple files at a time, with
 * any luck saving time over individual calls to ELBASE_CopyFile.
 */

#ifdef __cplusplus
extern "C" {
#endif


#include "elbase.h"


/* A link over which bulk copies can be made. */
typedef struct ELBC__BulkLink *ELBC_BulkLink;


/* Releases any resources associated with #link# and frees #link#. */
void
ELBC_BulkLinkFree(ELBC_BulkLink link);


/*
 * Returns a link for bulk copies from the #sourceStorageService# server
 * #sourceStorageServer# to the #destStorageService# server
 * #destStorageServer#.  The disks which these two servers can access are also
 * available to the #sourceComputeService# server #sourceComputeServer# and the
 * #destComputeService# server #destComputeServer#, respectively.  Returns NULL
 * if a link for bulk copies cannot be established.
 */
ELBC_BulkLink
ELBC_BulkLinkNew(ELBASE_StorageServiceTypes sourceStorageService,
                 const char *sourceStorageServer,
                 ELBASE_ComputeServiceTypes sourceComputeService,
                 const char *sourceComputeServer,
                 ELBASE_StorageServiceTypes destStorageService,
                 const char *destStorageServer,
                 ELBASE_ComputeServiceTypes destComputeService,
                 const char *destComputeServer);


/*
 * Starts a process to copy on #link# each file listed in the #pathCount#-long
 * array #sourcePaths# to the corresponding path in the #pathCount#-long array
 * #destPaths#.  If successful, returns 1 and sets #pid# to the process id;
 * otherwise, returns 0.
 */
int
ELBC_BulkCopy(ELBC_BulkLink link,
              const char **sourcePaths,
              const char **destPaths,
              unsigned pathCount,
              ELBASE_Process *pid);


#ifdef __cplusplus
}
#endif


#endif
