/* $Id: elbase.h,v 1.31 2004/03/24 19:40:19 jhayes Exp $ */
/*
 * Copyright  2004 The Regents of the University of California. 
 * All Rights Reserved. 
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for educational, research and non-profit purposes, without
 * fee, and without a written agreement is hereby granted, provided that the
 * above copyright notice, this paragraph and the following three paragraphs
 * appear in all copies. 
 *
 * Permission to incorporate this software into commercial products may be
 * obtained by contacting
 * Technology Transfer Office 
 * 9500 Gilman Drive 
 * 411 University Center 
 * University of California 
 * La Jolla, CA 92093-0093
 * (858) 534-5815
 * invent@ucsd.edu
 *
 * This software program and documentation are copyrighted by The Regents of
 * the University of California. The software program and documentation are
 * supplied "as is", without any accompanying services from The Regents. The
 * Regents does not warrant that the operation of the program will be
 * uninterrupted or error-free. The end-user understands that the program was
 * developed for research purposes and is advised not to rely exclusively on
 * the program for any reason. 
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
 * LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION,
 * EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED
 * HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO
 * OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
 * MODIFICATIONS. 
 */


#ifndef ELBASE_H
#define ELBASE_H

/*
 * This package provides ELAGI basic services--file copying and process
 * spawning--using a variety of Grid infrastructures.
 */

#ifdef __cplusplus
extern "C" {
#endif


/* Supported compute infrastructures. */
typedef enum {
  ELBASE_FORK, ELBASE_GRAM, ELBASE_NETSOLVE, ELBASE_RASH, ELBASE_SSH,
  ELBASE_COMPUTE_SERVICE_COUNT
} ELBASE_ComputeServiceTypes;

/* Supported batch scheduling infrastructures. */
typedef enum {
  ELBASE_SHELL, ELBASE_CONDOR, ELBASE_DQS, ELBASE_LOADLEVELER, ELBASE_LSF,
  ELBASE_PBS, ELBASE_SGE,
  ELBASE_SCHEDULER_SERVICE_COUNT
} ELBASE_SchedulerServiceTypes;

/* Supported file copy infrastructures. */
typedef enum {
  ELBASE_CP, ELBASE_FTP, ELBASE_GASS, ELBASE_SCP, ELBASE_SFTP, ELBASE_SRB,
  ELBASE_STORAGE_SERVICE_COUNT
} ELBASE_StorageServiceTypes;

/* A spawned process. */
typedef void *ELBASE_Process;

/* Recognized elements of ELBASE_Submit's #schedulerAttributes# parameter. */
/* Account to charge */
#define ELBASE_ACCOUNT   "project"
/* email to notify about status */
#define ELBASE_EMAIL     "email"
/* Minimum required megabytes */
#define ELBASE_MEMORY    "min_memory"
/* Total # of nodes to request */
#define ELBASE_NODES     "host_count"
/* Type of node */
#define ELBASE_NODETYPE  "node_type"
/* When to send email--always|complete|error|never */
#define ELBASE_NOTIFY    "notify"
/* # of copies to run */
#define ELBASE_PROCESSES "count"
/* Name of batch queue */
#define ELBASE_QUEUE     "queue"
/* Job stderr path */
#define ELBASE_STDERR    "stderr"
/* Job stdin path */
#define ELBASE_STDIN     "stdin"
/* Job stdout path */
#define ELBASE_STDOUT    "stdout"
/* Maximum wall clock minutes */
#define ELBASE_TIME      "max_wall_time"


/*
 * Equivalent to calling ELBASE_SpawnScript with a script constructed by
 * calling ELBASE_ScriptForCopy with all the same parameters.
 */
int
ELBASE_CopyFile(ELBASE_StorageServiceTypes sourceService,
                const char *sourceServer,
                const char *sourcePath,
                ELBASE_StorageServiceTypes destService,
                const char *destServer,
                const char *destPath,
                ELBASE_Process *process);


/*
 * Equivalent to calling ELBASE_SpawnScript with a script constructed by
 * calling ELBASE_ScriptForDelete with all the same parameters.
 */
int
ELBASE_DeleteFile(ELBASE_StorageServiceTypes service,
                  const char *server,
                  const char *path,
                  ELBASE_Process *process);


/*
 * Kills the process #process#, which must have been returned from a successful
 * call to a function in this package.  Returns 1 if successful, else 0.
 */
int
ELBASE_Kill(ELBASE_Process process);


/*
 * Returns 1 if #process#, which must have been returned from a successful call
 * to a function in this package, has terminated, else 0.  The function blocks
 * until the process terminates if #waitForTermination# is non-zero.  The
 * process' exit status is copied into #exitStatus# if it is non-NULL.
 */
int
ELBASE_Poll(ELBASE_Process process,
            int waitForTermination,
            int *exitStatus);


/*
 * Returns in an allocated string a /bin/sh script to copy the file
 * #sourcePath# accessed via the #sourceService# server #sourceServer# to the
 * file #destPath# accessed via the #destService# server #destServer#.
 */
char *
ELBASE_ScriptForCopy(ELBASE_StorageServiceTypes sourceService,
                     const char *sourceServer,
                     const char *sourcePath,
                     ELBASE_StorageServiceTypes destService,
                     const char *destServer,
                     const char *destPath);


/*
 * Returns in an allocated string a /bin/sh script that asks the #service#
 * server #server# to delete the file #path#.
 */
char * 
ELBASE_ScriptForDelete(ELBASE_StorageServiceTypes service,
                       const char *server,
                       const char *path);


/*
 * Returns in an allocated string a /bin/sh script that asks the #service#
 * server #server# to execute or submit #executablePath# using #scheduler#.
 * #schedulerAttributes# is a NULL-terminated array of name=value elements
 * indicating parameters for batch submissions.  Execution is performed in
 * #workingDirectory#, #arguments# are passed to the execution, and #stdinPath#,
 * #stdoutPath#, and #stderrPath# redirect the standard files to file paths on
 * #server#.  #environment# is a NULL-terminated array of name=value elements
 * indicating local environment variable settings.  Any or all the the last
 * seven parameters may be NULL.
 */
char *
ELBASE_ScriptForSubmit(ELBASE_ComputeServiceTypes service,
                       const char *server,
                       ELBASE_SchedulerServiceTypes scheduler,
                       const char **schedulerAttributes,
                       const char *executablePath,
                       const char *workingDirectory,
                       const char **arguments,
                       const char *stdinPath,
                       const char *stdoutPath,
                       const char *stderrPath,
                       const char **environment);


/*
 * Instructs the module to use #command# to launch #service# jobs, overriding
 * the default that relies on the user's PATH setting. Returns 1 on success,
 * else 0.
 */
int
ELBASE_SetComputeCommand(ELBASE_ComputeServiceTypes service, 
                         const char *command);


/*
 * Launches a process to run the /bin/sh script #script#.  Returns 1 if
 * successful, else 0.  If successful and #process# is non-NULL, sets it to a
 * process id that can be passed to ELBASE_Kill and ELBASE_Poll.
 */
int
ELBASE_SpawnScript(const char *script,
                   ELBASE_Process *process);


/*
 * Equivalent to calling ELBASE_SpawnScript with a script constructed by
 * calling ELBASE_ScriptForSubmit with all the same parameters.
 */
int
ELBASE_Submit(ELBASE_ComputeServiceTypes service,
              const char *server,
              ELBASE_SchedulerServiceTypes scheduler,
              const char **schedulerAttributes,
              const char *executablePath,
              const char *workingDirectory,
              const char **arguments,
              const char *stdinPath,
              const char *stdoutPath,
              const char *stderrPath,
              const char **environment,
              ELBASE_Process *process);
/* Convenience macro for spawning a non-batch process. */
#define ELBASE_Spawn(type,server,exec,wd,args,in,out,err,env,proc) \
  ELBASE_Submit(type,server,ELBASE_SHELL,0,exec,wd,args,in,out,err,env,proc)
/* Convenience macro for forking a local process. */
#define ELBASE_Fork(exec,wd,args,in,out,err,env,proc) \
  ELBASE_Spawn(ELBASE_FORK, "", exec, wd, args, in, out, err, env, proc)


#ifdef __cplusplus
}
#endif


#endif
