/* $Id: ApstClient.java,v 1.2 2003/09/08 23:16:10 jhayes Exp $ */
/*
 * Copyright  2002 The Regents of the University of California. 
 * All Rights Reserved. 
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for educational, research and non-profit purposes, without
 * fee, and without a written agreement is hereby granted, provided that the
 * above copyright notice, this paragraph and the following three paragraphs
 * appear in all copies. 
 *
 * Permission to incorporate this software into commercial products may be
 * obtained by contacting
 * Technology Transfer Office 
 * 9500 Gilman Drive 
 * 411 University Center 
 * University of California 
 * La Jolla, CA 92093-0093
 * (858) 534-5815
 * invent@ucsd.edu
 *
 * This software program and documentation are copyrighted by The Regents of
 * the University of California. The software program and documentation are
 * supplied "as is", without any accompanying services from The Regents. The
 * Regents does not warrant that the operation of the program will be
 * uninterrupted or error-free. The end-user understands that the program was
 * developed for research purposes and is advised not to rely exclusively on
 * the program for any reason. 
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
 * LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION,
 * EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED
 * HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO
 * OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
 * MODIFICATIONS. 
 */


import java.io.*;
import java.net.*;


public class ApstClient {


/* Creates an ApstClient connected to the server at the other end of #sock#. */
public ApstClient(Socket sock) throws IOException {
  initFromSocket(sock);
}


/* Creates an ApstClient connected to the server listening on #addr#:#port#. */
public ApstClient(InetAddress addr,
                  int port) throws IOException {
  Socket s;
  try {
    s = new Socket(addr, port);
  } catch(Exception x) {
    throw new IOException("Unable to contact " + addr + ":" + port);
  }
  initFromSocket(s);
}


/*
 * Creates an ApstClient connected to the server accessed via #server#, which
 * is a host name or IP address image optionally followed by a colon and a port.
 */
public ApstClient(String server) throws IOException {
  int colonPos = server.indexOf(":");
  int port;
  Socket s;
  if(colonPos < 0)
    port = 6660;
  else {
    port = Integer.parseInt(server.substring(colonPos + 1));
    server = server.substring(0, colonPos);
  }
  try {
    s = new Socket(InetAddress.getByName(server), port);
  } catch(Exception x) {
    throw new IOException("Unable to contact " + server + ":" + port);
  }
  initFromSocket(s);
}


/*
 * Asks the server to add the resources and/or tasks described in #xml#.
 * Returns true if successful, else false.
 */
public boolean
addXml(String xml) {
  return dialog(MSG_ADD, xml) != null;
}


/*
 * Asks the server to add the resources and/or tasks described in the local
 * file #path#.  * Returns true if successful, else false.
 */
public boolean
addFile(String path) {

  String atPath;
  FileReader f;
  long fLen;
  String localPath;
  boolean result;

  try {
    localPath = new File(path).getCanonicalPath();
  } catch(Exception x) {
    localPath = path;
  }
  try {
    f = new FileReader(localPath);
  } catch(Exception x) {
    try {
      localPath += ".xml";
      f = new FileReader(localPath);
    } catch(Exception e) {
      this.error = "No such file " + path;
      return false;
    }
  }

  fLen = new File(localPath).length();
  System.out.println("Length is " + fLen);
  if(this.localServer) {
    atPath = "@" + localPath;
    result = dialog(MSG_ADD, atPath) != null;
  }
  else if(fLen < 10000) {
    char[] contents = new char[(int)fLen];
    try {
      new BufferedReader(f).read(contents, 0, (int)fLen);
      result = dialog(MSG_ADD, new String(contents)) != null;
    } catch(Exception x) {
      this.error = "Read error";
      result = false;
    }
  }
  else {
    String serverPath = "/tmp/apstclient" + this.myId + ".xml";
    atPath = "@" + serverPath;
    if(copyFile(localPath, serverPath))
      result = dialog(MSG_ADD, atPath) != null;
    else {
      this.error = "File copy failed";
      result = false;
    }
  }

  try {
    f.close();
  } catch(Exception x) {
    /* Nothing to do. */
  }
  return result;

}


/*
 * When the server rejects a request sent over the socket, the client will
 * resubmit it via a copied file if #allow# is true.  Returns the previous
 * value of the flag.
 */
public boolean
allowFile(boolean allow) {
  boolean old = this.tryFile;
  this.tryFile = allow;
  return old;
}


/*
 * Asks the server to cancel the tasks indicated by #tasks#.  Returns true if
 * successful, else false.
 */
public boolean
cancelTasks(String tasks) {
  return dialog(MSG_CANCEL_TASKS, tasks) != null;
}


/*
 * Asks the server to discard all task and resource information.  Returns true
 * if successful, else false.
 */
public boolean
clear() {
  return dialog(MSG_CLEAR, null) != null;
}


/*
 * Asks the server to disable the resource identified by #id#.  Returns true
 * iff successful.
 */
public boolean
disable(String id) {
  return dialog(MSG_DISABLE_HOSTS, id) != null;
}


/*
 * Asks the server to enable the resource identified by #id#.  Returns true iff
 * successful.
 */
public boolean
enable(String id) {
  return dialog(MSG_ENABLE_HOSTS, id) != null;
}


/*
 * Asks the server to copy the file identified by #id# to its local directory.
 * Returns true iff successful.
 */
public boolean
download(String id) {
  return dialog(MSG_DOWNLOAD_FILE, id) != null;
}


/* Returns a message describing the reason for the last failed request. */
public String
getError() {
  return this.error;
}


/* Returns the client id and server working directory; null on failure. */
public String
getId() {
  return dialog(MSG_GET_ID, null);
}


/* Returns the server log file contents; null on failure. */
public String
getLog() {
  return dialog(MSG_GET_FILE, null);
}


/* Returns XML describing the server state; null on failure. */
public String
getState(String tasks) {
  return dialog(MSG_GET_STATE, tasks);
}


/*
 * Returns server statistics in an array of 7 ints; null on failure.  The
 * elements of the array are:
 *   [0] seconds the server has been running
 *   [1] apst commands performed
 *   [2] total tasks
 *   [3] tasks not yet running
 *   [4] tasks running
 *   [5] tasks finished
 *   [6] tasks canceled
 */
public int[]
getStatus() {
  int i;
  int pos;
  String reply = dialog(MSG_GET_STATUS, null);
  int[] result;
  if(reply == null)
    return null;
  result = new int[7];
  reply += " ";
  for(i = 0; i < 7 && (pos = reply.indexOf(" ")) >= 0; i++) {
    result[i] = Integer.parseInt(reply.substring(0, pos));
    reply = reply.substring(pos + 1);
  }
  return result;
}


/* Asks the server to do nothing; returns true iff successful. */
public boolean
noop() {
  return dialog(MSG_NOOP, null) != null;
}


/*
 * Asks the server to send email to #addr# on completion; returns true iff
 * successful.
 */
public boolean
notify(String emailAddr) {
  return dialog(MSG_NOTIFY, emailAddr) != null;
}


/*
 * Asks the server to mark as new the tasks listed in #tasks#; returns true iff
 * successful.
 */
public boolean
resetTasks(String tasks) {
  return dialog(MSG_RESET_TASKS, tasks) != null;
}


/*
 * Asks the server to perform #count# scheduling steps before pausing.  A
 * #count# of 0 instructs the server to pause immediately; a count < 0
 * instructs it to never pause.  Returns true iff successful.
 */
public boolean
step(int count) {
  return dialog(MSG_STEP, count + "") != null;
}


/* Asks the server to terminate.  Returns true iff successful. */
public boolean
stop() {
  return dialog(MSG_STOP, null) != null;
}


/*
 * Uses scp to transfer the local file #localPath# to #remotePath# on the
 * server's machine.  Returns true iff successful.
 */
protected boolean copyFile(String localPath,
                           String remotePath) {
  try {
    Process proc = Runtime.getRuntime().exec
      ("scp " + localPath + " " + serverHost + ":" + remotePath);
    if(proc.waitFor() != 0)
      return false;
  } catch(Exception x) {
    return false;
  }
  return true;
}


/*
 * Sends #command# to the server, followed by #args# if it is not null.
 * Returns the (possibly-empty) contents of a successful reply; null on error.
 */
protected String
dialog(String command,
       String args) {

  String content;
  String fileName;
  String localPath;
  int pos;
  String received;
  boolean result;
  String serverPath;

  try {
    String toSend = (args == null ? command : (command + " " + args)) + "\r\n";
    toServer.write(toSend.getBytes(), 0, toSend.length());
  } catch(Exception x) {
    this.error = "Send error";
    return null;
  }
  try {
    StringBuffer buf = new StringBuffer();
    while(true) {
      int c = fromServer.read();
      if(c <= 0) {
        this.error = "Receive error";
        return null;
      }
      if(c == '\r') {
        if((c = fromServer.read()) == '\n') {
          received = buf.toString();
          break;
        }
        else
          buf.append('\r');
      }
      buf.append((char)c);
    }
  } catch(Exception x) {
    this.error = "Receive error";
    return null;
  }

  pos = received.indexOf("\n");
  content = pos < 0 ? "" : received.substring(pos + 1);
  if(received.startsWith(SUCCEED))
    return content;
  this.error = content;
  if(!this.tryFile || received.indexOf("denied") < 0)
    return null;

  fileName =  "apstclient" + this.myId + ".txt";
  serverPath = this.serverDir + "/" + fileName;
  localPath = !this.localServer ? "/tmp/" + fileName : serverPath;
  try {
    FileWriter f = new FileWriter(localPath);
    f.write(args != null ? command + " " + args : command);
    f.close();
    /* TODO Set file permissions */
  } catch(Exception x) {
    this.error = x.toString();
    return null;
  }
  if(!this.localServer && !copyFile(localPath, serverPath))
    return null;
  return dialog(MSG_READ, fileName);

}


/* Initialization routine for constructors. */
protected void initFromSocket(Socket s) throws IOException {
  InetAddress addr = s.getInetAddress();
  int i;
  String idInfo;
  this.fromServer = new DataInputStream(s.getInputStream());
  this.toServer = new DataOutputStream(s.getOutputStream());
  this.localServer = addr.equals(InetAddress.getLocalHost()) ||
                     addr.equals(InetAddress.getByName("localhost"));
  this.serverHost = addr.getHostName();
  if((idInfo = dialog(MSG_GET_ID, null)) == null)
    throw new IOException("Unable to talk to server");
  i = idInfo.indexOf(" ");
  this.myId = idInfo.substring(0, i);
  this.serverDir = idInfo.substring(i + 1);
}


protected final static String FAIL = "FAIL";
protected final static String SUCCEED = "OK";

protected final static String MSG_ADD = "ADD";
protected final static String MSG_CANCEL_TASKS = "CANCEL";
protected final static String MSG_CLEAR = "CLEAR";
protected final static String MSG_DISABLE_HOSTS = "DISABLE";
protected final static String MSG_DOWNLOAD_FILE = "DOWNLOAD";
protected final static String MSG_ENABLE_HOSTS = "ENABLE";
protected final static String MSG_GET_FILE = "FILE";
protected final static String MSG_GET_ID = "ID";
protected final static String MSG_NOOP = "NOOP";
protected final static String MSG_NOTIFY = "NOTIFY";
protected final static String MSG_READ = "READ";
protected final static String MSG_RESET_TASKS = "RESET";
protected final static String MSG_GET_STATE = "STATE";
protected final static String MSG_GET_STATUS = "STATUS";
protected final static String MSG_STEP = "STEP";
protected final static String MSG_STOP = "STOP";

protected String error = null;
protected DataInputStream fromServer = null;
protected boolean localServer = false;
protected String myId = null;
protected String serverDir = null;
protected String serverHost = null;
protected DataOutputStream toServer = null;
protected boolean tryFile = false;


protected static final String USAGE =
  "USAGE: apst [-hv] [--host <machine>[:<port>]] [<command>]\n" +
     "\tadd [<file> ...]\n" +
     "\tcancel <task> [...] | all\n" +
     "\tcheckpoint [<file>]\n" +
     "\tclear\n" +
     "\tdisable [<resource> | all]\n" +
     "\tdownload <file>\n" +
     "\tenable [<resource> | all]\n" +
     "\tgetlog\n" +
     "\tnotify <e-mail address>\n" +
     "\treset <task> [...] | all\n" +
     "\tresources\n" +
     "\tstatus\n" +
     "\tstep n\n" +
     "\tstop\n" +
     "\ttasks [<task> [...] | all]";

protected static void printTags(String xml,
                                String tag) {
  int pos;
  String endTag = "</" + tag + ">";
  tag = "<" + tag;
  int tagLen = tag.length();
  int endTagLen = endTag.length();
  while((pos = xml.indexOf(tag)) >= 0) {
    int endPos = xml.indexOf(">", pos);
    if(pos + tagLen != endPos && xml.charAt(pos + tagLen) != ' ') {
      xml = xml.substring(endPos + 1);
      continue;
    }
    int endTagPos =
      xml.charAt(endPos - 1) == '/' ? endPos - 1 : xml.indexOf(endTag, endPos);
    String content = xml.substring(pos + 1, endTagPos);
    xml = xml.substring(endPos + endTagLen);
    while((pos = content.indexOf("<")) >= 0)
      content = content.substring(0, pos) + " " + content.substring(pos + 1);
    while((pos = content.indexOf("/>")) >= 0)
      content = content.substring(0, pos) + " " + content.substring(pos + 2);
    while((pos = content.indexOf(">")) >= 0)
      content = content.substring(0, pos) + " " + content.substring(pos + 1);
    System.out.println(content);
  }
}

protected static boolean doCommand(ApstClient client,
                                   String command,
                                   String args) {
  boolean result;
  if(command.equals("add")) {
    if(args == null) {
      BufferedReader stdin =
        new BufferedReader(new InputStreamReader(System.in));
      String xml = "";
      try {
        do {
          xml += stdin.readLine() + "\n";
        } while(xml.indexOf("</apst>") < 0 && xml.indexOf("<apst/>") < 0);
        result = client.addXml(xml);
      } catch(Exception x) {
        result = false;
      }
    }
    else {
      int i;
      result = true;
      while((i = args.indexOf(" ")) >= 0) {
        String one = args.substring(0, i);
        args = args.substring(i + 1).trim();
        if(!(result = client.addFile(one)))
          break;
      }
      if(result)
        result = client.addFile(args);
    }
  }
  else if(command.equals("checkpoint")) {
    String contents = client.getState("all");
    result = contents != null;
    if(result) {
      /* TODO Pretty-print */
      System.out.println(contents);
    }
  }
  else if(command.equals("clear")) {
    result = client.clear();
  }
  else if(command.equals("getlog")) {
    String contents = client.getLog();
    result = contents != null;
    if(result)
      System.out.println(contents);
  }
  else if(command.equals("id")) {
    String contents = client.getId();
    result = contents != null;
    if(result)
      System.out.println(contents);
  }
  else if(command.equals("noop")) {
    result = client.noop();
  }
  else if(command.equals("resources")) {
    String state = client.getState("");
    result = state != null;
    if(result) {
      printTags(state, "disk");
      printTags(state, "host");
    }
  }
  else if(command.equals("schedule")) {
    String state = client.getState("");
    result = state != null;
    if(result)
      printTags(state, "schedule");
  }
  else if(command.equals("status")) {
    int[] counts = client.getStatus();
    result = counts != null;
    if(result)
      System.out.println(
        "Time running = " + counts[0] + "\n" +
        "apst command performed = " + counts[1] + "\n" +
        "Tasks total: " + counts[2] + "  new: " + counts[3] +
         "  running: " + counts[4] + "  done: " + counts[5] +
         "  canceled: " + counts[6]);
  }
  else if(command.equals("step")) {
    result = client.step(Integer.parseInt(args));
  }
  else if(command.equals("stop")) {
    result = client.stop();
  }
  else if(command.equals("tasks")) {
    String contents =
      client.getState(args == null || args == "" ? "all" : args);
    result = contents != null;
    if(result)
      printTags(contents, "task");
  }
  else if(args == null && !command.equals("help")) {
    /* All other commands require an argument. */
    System.out.println(USAGE);
    return false;
  }
  else if(command.equals("cancel")) {
    result = client.cancelTasks(args);
  }
  else if(command.equals("disable")) {
    result = client.disable(args);
  }
  else if(command.equals("download")) {
    result = client.download(args);
  }
  else if(command.equals("enable")) {
    result = client.enable(args);
  }
  else if(command.equals("notify")) {
    result = client.notify(args);
  }
  else if(command.equals("reset")) {
    result = client.resetTasks(args);
  }
  else {
    System.out.println(USAGE);
    return command.equals("help");
  }
  if(!result && client.getError() != null)
    System.err.println(client.getError());
  return result;
}

public static void main(String args[]) {
  String command = null;
  String serverImage = "localhost";
  String params = "";
  ApstClient client = null;
  int i;
  boolean result = false;
  for(i = 0; i < args.length; i++) {
    if(args[i].equals("--host") && i + 1 < args.length)
      serverImage = args[++i];
    else if(command == null)
      command = args[i];
    else
      params += " " + args[i];
  }
  params = params.trim();
  try {
    client = new ApstClient(serverImage);
  }
  catch(IOException x) {
    System.err.println(x.toString());
    System.exit(1);
  }
  client.allowFile(true);
  if(command == null) {
    java.util.Vector history = new java.util.Vector();
    BufferedReader stdin = new BufferedReader(new InputStreamReader(System.in));
    int c;
    while(true) {
      System.out.print("ApstClient> ");
      command = "";
      try {
        command = stdin.readLine();
      } catch(Exception x) {
        break;
      }
      if(command == null)
        break;
      command = command.trim();
      if(command.equals(""))
        continue;
      history.addElement(command);
      while((i = command.indexOf("!{")) >= 0) {
        int end = command.indexOf("}", i + 2);
        int which;
        if(end < 0)
          break;
        String ref = command.substring(i + 2, end);
        try {
          which = Integer.parseInt(ref);
          if(which > history.size() - 1)
            which = -1;
        } catch(Exception x) {
          for(which = history.size() - 1; which >= 0; which--) {
            if(((String)history.elementAt(which)).startsWith(ref))
              break;
          }
        }
        if(which < 0) {
          System.err.println("Unknown history reference '" + ref + "'");
          command = "";
          break;
        }
        command = command.substring(0, i) +
                  (String)history.elementAt(which) +
                  command.substring(end + 1);
      }
      if(command.equals(""))
        continue;
      if(command.equals("quit"))
        break;
      if((i = command.indexOf(" ")) < 0)
        params = null;
      else {
        params = command.substring(i).trim();
        command = command.substring(0, i);
      }
      if(command.equals("history")) {
        for(i = 0; i < history.size(); i++)
          System.out.println(i + " " + (String)history.elementAt(i));
      }
      else
        System.out.println(doCommand(client, command, params) ? "ok" : "fail");
    }
  }
  else
    System.out.println(doCommand(client, command, params) ? "ok" : "fail");
}


}
