/* $Id: client.c,v 1.40 2003/09/15 19:56:23 jhayes Exp $ */
/*
 * Copyright  2002 The Regents of the University of California. 
 * All Rights Reserved. 
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for educational, research and non-profit purposes, without
 * fee, and without a written agreement is hereby granted, provided that the
 * above copyright notice, this paragraph and the following three paragraphs
 * appear in all copies. 
 *
 * Permission to incorporate this software into commercial products may be
 * obtained by contacting
 * Technology Transfer Office 
 * 9500 Gilman Drive 
 * 411 University Center 
 * University of California 
 * La Jolla, CA 92093-0093
 * (858) 534-5815
 * invent@ucsd.edu
 *
 * This software program and documentation are copyrighted by The Regents of
 * the University of California. The software program and documentation are
 * supplied "as is", without any accompanying services from The Regents. The
 * Regents does not warrant that the operation of the program will be
 * uninterrupted or error-free. The end-user understands that the program was
 * developed for research purposes and is advised not to rely exclusively on
 * the program for any reason. 
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
 * LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION,
 * EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED
 * HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO
 * OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
 * MODIFICATIONS. 
 */


#include "config.h"
#include <stdio.h>   /* file functions */
#include <string.h>  /* string functions */
#include <unistd.h>  /* getcwd getpid */
#include <fileseed.h>
#define ASIP_SHORT_NAMES
#include <ipseed.h>
#define ASSTR_SHORT_NAMES
#include <strseed.h>
#include <elbase.h>
#include "misc.h"
#include "protocol.h"
#include "client.h"


static char *error = NULL;
static int tryFile = 1;


/* A convenience.  Stores a copy of #s# in the error global, then returns 0. */
static int
Fail(const char *s) {
  if(error != NULL)
    FREE(error);
  error = STRDUP(s);
  return 0;
}


/*
 * Sends #command# on #sock#, with #args# appended if it isn't NULL.  Returns 1
 * on success, 0 on failure.  Handles communication via file if enabled. If
 * #content# is not NULL, any reply returned by the peer is stored in it; this
 * should be freed after use.
 */
static int
Dialog(Socket sock,
       const char *command,
       const char *args,
       char **content) {

  char *c;
  char *daemonPath;
  int daemonIsLocal;
  FILE *f;
  char *fileName;
  char *localPath;
  ELBASE_Process process;
  char *reply;
  int result;

  if(args == NULL)
    result = SendString(sock, command);
  else {
    char *commandAndArgs = StrAppend(NULL, command, " ", args, NULL);
    result = SendString(sock, commandAndArgs);
    FREE(commandAndArgs);
  }

  if(!result)
    return Fail("Send error");
  else if(!ReceiveString(sock, &reply))
    return Fail("Receive error");

  result = strncmp(reply, APST_CLIENT_OK, strlen(APST_CLIENT_OK)) == 0;
  c = strchr(reply, '\n');
  if(!result)
    Fail(c == NULL ? "" : (c + 1));
  else if(content != NULL)
    *content = STRDUP(c == NULL ? "" : (c + 1));
  FREE(reply);
  if(result || !tryFile || strstr(error, "denied") == NULL ||
     strcmp(command, APST_CLIENT_GET_ID) == 0)
    return result;

  if(!Dialog(sock, APST_CLIENT_GET_ID, NULL, &reply))
    return 0;
  else if((c = strchr(reply, ' ')) == NULL) {
    FREE(reply);
    return Fail("Receive error");
  }

  *c = '\0';
  daemonIsLocal = ASIP_SocketAddress(sock) == ASIP_PeerAddress(sock);
  fileName = StrAppend(NULL, "client", reply, ".txt", NULL);
  daemonPath = StrAppend(NULL, c + 1, "/", fileName, NULL);
  localPath = daemonIsLocal ?
              STRDUP(daemonPath) : StrAppend(NULL, "/tmp/", fileName, NULL);
  FREE(reply);
  result = (f = fopen(localPath, "w")) != NULL;
  if(result) {
    fprintf(f, "%s %s", command, args == NULL ? "" : args);
    fclose(f);
    ASFILE_SetFilePermission(localPath, AS_GROUP, 1, 0, 0);
    ASFILE_SetFilePermission(localPath, AS_OTHERS, 1, 0, 0);
    ASFILE_SetFilePermission(localPath, AS_OWNER, 1, 1, 0);
    if(!daemonIsLocal) {
      char *addrImage = ASIP_AddressImage(ASIP_PeerAddress(sock));
      result =
        ELBASE_CopyFile
          (ELBASE_SCP, NULL, localPath, ELBASE_SCP, addrImage, daemonPath,
           &process) &&
        ELBASE_Poll(process, 1, NULL);
      FREE(addrImage);
    }
  }
  FREE(localPath);
  FREE(daemonPath);
  if(result)
    result = Dialog(sock, APST_CLIENT_READ, fileName, content);
  FREE(fileName);
  return result;

}


int
APSTClient_AddXml(Socket sock,
               const char *xml) {
  return Dialog(sock, APST_CLIENT_ADD, xml, NULL);
}


int
APSTClient_AddFile(ASIP_Socket sock,
                   const char *path) {

  char *atPath;
  ASFILE_FileFacts facts;
  char *localPath;
  int result;
  char wd[1024];

  getcwd(wd, sizeof(wd));
  localPath = AbsolutePath(path, wd);

  if(!ASFILE_GetFileFacts(localPath, &facts)) {
    localPath = StrAppend(localPath, ".xml", NULL);
    if(!ASFILE_GetFileFacts(localPath, &facts)) {
      FREE(localPath);
      return Fail("No such file");
    }
  }

  if(ASIP_SocketAddress(sock) == ASIP_PeerAddress(sock)) {
    atPath = StrAppend(NULL, "@", localPath, NULL);
    result = Dialog(sock, APST_CLIENT_ADD, atPath, NULL);
    FREE(atPath);
  }
  else if(facts.size <= 10000) { /* Arbitrary limit on enclosed XML size. */
    char *contents = ASFILE_FileContents(localPath);
    result = Dialog(sock, APST_CLIENT_ADD, contents, NULL);
    FREE(contents);
  }
  else {
    char *addrImage;
    char pidImage[20];
    ELBASE_Process process;
    sprintf(pidImage, "%d", (int)getpid());
    atPath = StrAppend(NULL, "@/tmp/apstclient", pidImage, ".xml", NULL);
    addrImage = ASIP_AddressImage(ASIP_PeerAddress(sock));
    result =
      ELBASE_CopyFile
        (ELBASE_SCP, NULL, localPath, ELBASE_SCP, addrImage, atPath + 1,
         &process) &&
      ELBASE_Poll(process, 1, NULL);
    FREE(addrImage);
    if(!result)
      Fail("File copy failed");
    else
      result = Dialog(sock, APST_CLIENT_ADD, atPath, NULL);
    FREE(atPath);
  }

  FREE(localPath);
  return result;

}


int
APSTClient_AllowFile(int allow) {
  int old = tryFile;
  tryFile = allow;
  return old;
}


int
APSTClient_CancelTasks(Socket sock,
                       const char *tasks) {
  return Dialog(sock, APST_CLIENT_CANCEL_TASKS, tasks, NULL);
}


int
APSTClient_Clear(ASIP_Socket sock) {
  return Dialog(sock, APST_CLIENT_CLEAR, NULL, NULL);
}


int
APSTClient_Disable(Socket sock,
                   const char *resource) {
  return Dialog(sock, APST_CLIENT_DISABLE_HOSTS, resource, NULL);
}


int
APSTClient_Enable(Socket sock,
                  const char *resource) {
  return Dialog(sock, APST_CLIENT_ENABLE_HOSTS, resource, NULL);
}


int
APSTClient_Download(Socket sock,
                    const char *resource) {
  return Dialog(sock, APST_CLIENT_DOWNLOAD_FILE, resource, NULL);
}


const char *
APSTClient_Error(void) {
  return error;
}


int
APSTClient_GetId(ASIP_Socket sock,
                 char **id) {
  return Dialog(sock, APST_CLIENT_GET_ID, NULL, id);
}


int
APSTClient_GetLog(Socket sock,
                  char **log) {
  return Dialog(sock, APST_CLIENT_GET_FILE, NULL, log);
}


int
APSTClient_GetState(Socket sock,
                    const char *tasks,
                    char **state) {
  return Dialog(sock, APST_CLIENT_GET_STATE, tasks, state);
}


int
APSTClient_GetStatus(Socket sock,
                     unsigned *secondsRunning,
                     unsigned *commands,
                     unsigned *tasksTotal,
                     unsigned *tasksToDo,
                     unsigned *tasksRunning,
                     unsigned *tasksFinished,
                     unsigned *tasksFailed) {
  char *status;
  if(!Dialog(sock, APST_CLIENT_GET_STATUS, NULL, &status))
    return 0;
  if(sscanf(status, "%u %u %u %u %u %u %u",
            secondsRunning, commands, tasksTotal, tasksToDo, tasksRunning,
            tasksFinished, tasksFailed) != 7) {
    FREE(status);
    return Fail("sscanf failure\n");
  }
  FREE(status);
  return 1;
}


int
APSTClient_Noop(ASIP_Socket sock) {
  return Dialog(sock, APST_CLIENT_NOOP, NULL, NULL);
}


int
APSTClient_Notify(Socket sock,
                  const char *address) {
  return Dialog(sock, APST_CLIENT_NOTIFY, address, NULL);
}


int
APSTClient_ResetTasks(Socket sock,
                      const char *tasks) {
  return Dialog(sock, APST_CLIENT_RESET_TASKS, tasks, NULL);
}


int
APSTClient_SetPriority(ASIP_Socket sock,
                       unsigned priority,
                       const char *tasks) {
  char *args;
  char priorityImage[10];
  int result;
  sprintf(priorityImage, "%d", priority);
  args = StrAppend(NULL, priorityImage, " ", tasks, NULL);
  result = Dialog(sock, APST_CLIENT_PRIORITIZE, args, NULL);
  FREE(args);
  return result;
}


int
APSTClient_Step(Socket sock,
                const char *count) {
  return Dialog(sock, APST_CLIENT_STEP, count, NULL);
}


int
APSTClient_Stop(Socket sock) {
  return Dialog(sock, APST_CLIENT_STOP, NULL, NULL);
}
