/* $Id: clientmain.c,v 1.66 2004/04/07 21:39:55 jhayes Exp $ */
/*
 * Copyright  2002 The Regents of the University of California. 
 * All Rights Reserved. 
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for educational, research and non-profit purposes, without
 * fee, and without a written agreement is hereby granted, provided that the
 * above copyright notice, this paragraph and the following three paragraphs
 * appear in all copies. 
 *
 * Permission to incorporate this software into commercial products may be
 * obtained by contacting
 * Technology Transfer Office 
 * 9500 Gilman Drive 
 * 411 University Center 
 * University of California 
 * La Jolla, CA 92093-0093
 * (858) 534-5815
 * invent@ucsd.edu
 *
 * This software program and documentation are copyrighted by The Regents of
 * the University of California. The software program and documentation are
 * supplied "as is", without any accompanying services from The Regents. The
 * Regents does not warrant that the operation of the program will be
 * uninterrupted or error-free. The end-user understands that the program was
 * developed for research purposes and is advised not to rely exclusively on
 * the program for any reason. 
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
 * LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION,
 * EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED
 * HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO
 * OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
 * MODIFICATIONS. 
 */


#include "config.h"
#include <signal.h>  /* signal */
#include <stdlib.h>  /* strtol */
#include <stdio.h>   /* file functions */
#include <string.h>  /* string functions */
#ifdef HAVE_READLINE_H
#  include <readline.h>
#  include <history.h>
#  define HAVE_READLINE
#endif
#ifdef HAVE_READLINE_READLINE_H
#  include <readline/readline.h>
#  include <readline/history.h>
#  define HAVE_READLINE
#endif
#include <elbase.h>
#define ASENV_SHORT_NAMES
#include <envseed.h>
#include <fileseed.h>
#define ASIP_SHORT_NAMES
#include <ipseed.h>
#include <propertyseed.h>
#define ASSTR_SHORT_NAMES
#include <strseed.h>
#include <vectorseed.h>
#define ASXML_SHORT_NAMES
#include <xmlseed.h>
#include "client.h"
#include "misc.h"


/*
 * Reads a line of input from the user and returns it in the #max#-long
 * array #command#.  Returns 0 on end-of-file, else 1.
 */
static int
GetCommand(char *command,
           unsigned max) {
  static const char *prompt = "apst> ";
#ifdef HAVE_READLINE
  char *c = readline((char *)prompt);
  if(c == NULL)
    return 0;
  strncpy(command, c, max - 1);
  command[max - 1] = '\0';
  FREE(c);
#else
  fprintf(stdout, "%s", prompt);
  if(fgets(command, max - 1, stdin) == NULL)
    return 0;
  command[strlen(command) - 1] = '\0'; /* Strip newline */
#endif
  return 1;
}


/*
 * Prints to stdout the contents of each #tag# tag within #xml# in a somewhat
 * human-friendly format.  Returns 1 if successful, else 0.
 */
static int
PrintXML(char *xml,
         const char *tag) {

  XmlNamedNodeMap attrs;
  char *c;
  char *d;
  XmlDocument doc;
  unsigned i;
  char *image;
  XmlNodeIterator it;
  XmlNode node;
  XmlNode x;

  if((doc = XmlFromString(xml, NULL, 0, 0, NULL, NULL, NULL)) == NULL) {
    fprintf(stderr, "Unable to parse xml\n");
    return 0;
  }
  /* NOTE: NodeLists and NodeIterators are interchangable in xmlseed. */
  it = (XmlNodeIterator)DocumentGetElementsByTagName(doc, (char *)tag);
  while((node = NodeIteratorNextNode(it)) != NULL) {
    attrs = NodeAttributes(node);
    for(i = 0; (x = NamedNodeMapItem(attrs, i)) != NULL; i++)
      fprintf(stdout, "%s='%s'\n", NodeName(x), NodeValue(x));
    for(x = NodeFirstChild(node); x != NULL; x = NodeNextSibling(x)) {
      image = XmlToString(x, NULL);
      while((c = strstr(image, "id=")) != NULL)
        StrReplace(&image, c - image, 3, "");
      while((c = strstr(image, "</")) != NULL) {
        d = strchr(c, '>');
        StrReplace(&image, c - image, d - c + 1, "");
      }
      while((c = strchr(image, '<')) != NULL)
        StrReplace(&image, c - image, 1, " ");
      while((c = strstr(image, "/>")) != NULL)
        StrReplace(&image, c - image, 2, "");
      while((c = strchr(image, '>')) != NULL)
        StrReplace(&image, c - image, 1, "");
      fprintf(stdout, "%s\n", image);
      FREE(image);
    }
  }
  NodeIteratorDetach(it);
  NodeFree(doc);
  return 1;

}


const char *USAGE = "USAGE: apst [<option> ...] [<command>]";
const char *COMMANDS =
  "add [<file> ...]\n"
  "cancel <tasks>\n"
  "checkpoint [<file>]\n"
  "clear\n"
  "disable [<resource> | all]\n"
  "download <file>\n"
  "enable [<resource> | all]\n"
  "getlog\n"
  "notify <e-mail address>\n"
  "prioritize <priority> <tasks>\n"
  "reset <tasks>\n"
  "resources\n"
  "status\n"
  "step n\n"
  "stop\n"
  "tasks [<tasks>]";
const char *VALID =
  "-h     void           display held and exit\n"
  "-v     void           display version and exit\n"
  "--host machine[:port] apstd contact information\n"
  "       string         command (see below)";


/*
 * Asks #server# to execute #command# using #args#.  Returns 1 if successful,
 * else 0.
 */
static int
DoCommand(Socket server,
          const char *command,
          const char **args) {

  char *contents;
  unsigned i;
  int result;
  char *tasks;

  if(strcmp(command, "add") == 0) {
    if(*args == NULL) { /* Read from stdin. */
      char line[256];
      char **lines = ASV_VectorNew(char *);
      ASV_GrowCapacity(lines, 100);
      ASV_SetIncrement(lines, 100);
      while(fgets(line, sizeof(line), stdin) != NULL) {
        ASV_Append(lines, STRDUP(line));
        if(strstr(line, "</apst>") != NULL || strstr(line, "<apst/>") != NULL)
          break;
      }
      contents = StrArrayNJoin((const char **)lines, ASV_Size(lines), NULL);
      for(i = 0; i < ASV_Size(lines); i++)
        FREE(lines[i]);
      ASV_VectorFree(lines);
      result = APSTClient_AddXml(server, contents);
      FREE(contents);
    }
    else {
      do {
        result = APSTClient_AddFile(server, *args);
        args++;
      } while(result && *args != NULL);
    }
  }
  else if(strcmp(command, "checkpoint") == 0) {
    XmlDocument doc;
    FILE *f;
    result = APSTClient_GetState(server, "all", &contents);
    if(result) {
      if((doc = XmlFromString(contents,NULL,0,0,NULL,NULL,NULL)) == NULL) {
        fprintf(stderr, "Unable to parse XML\n");
        result = 0;
      }
      else {
        FREE(contents);
        contents = XmlToString(doc, "  ");
        if(*args == NULL)
          fprintf(stdout, "%s\n", contents);
        else if((f = fopen(*args, "w")) == NULL)
          result = 0;
        else {
          fprintf(f, "%s\n", contents);
          fclose(f);
        }
        NodeFree(doc);
      }
      FREE(contents);
    }
  }
  else if(strcmp(command, "clear") == 0)
    result = APSTClient_Clear(server);
  else if(strcmp(command, "getlog") == 0) {
    result = APSTClient_GetLog(server, &contents);
    if(result) {
      fprintf(stdout, "%s", contents);
      FREE(contents);
    }
  }
  else if(strcmp(command, "help") == 0) {
    fprintf(stderr, "%s\n", COMMANDS);
    result = 1;
  }
  else if(strcmp(command, "resources") == 0) {
    result = APSTClient_GetState(server, "", &contents);
    if(result) {
      result = PrintXML(contents, "storage") && PrintXML(contents, "compute");
      FREE(contents);
    }
  }
  else if(strcmp(command, "status") == 0) {
    unsigned commands;
    unsigned done;
    unsigned failed;
    unsigned running;
    unsigned seconds;
    unsigned toDo;
    unsigned total;
    result = APSTClient_GetStatus
      (server, &seconds, &commands, &total, &toDo, &running, &done, &failed);
    if(result)
      fprintf(stdout,
              "Time running = %d\napst commands performed = %d\n"
              "Tasks total: %d  new: %d  running: %d  done: %d  "
              "failed: %d  canceled: %d\n",
              seconds, commands, total, toDo, running, done, failed,
              total - toDo - running - done - failed);
  }
  else if(strcmp(command, "step") == 0)
    result = APSTClient_Step(server, *args == NULL ? "1" : *args);
  else if(strcmp(command, "stop") == 0)
    result = APSTClient_Stop(server);
  else if(strcmp(command, "tasks") == 0) {
    tasks = *args == NULL ? STRDUP("all") : StrArrayJoin(args, " ");
    result = APSTClient_GetState(server, tasks, &contents);
    FREE(tasks);
    if(result) {
      result = PrintXML(contents, "tasks");
      FREE(contents);
    }
  }
  else if(*args == NULL) { /* All other commands require an argument. */
    fprintf(stderr, "%s\n", COMMANDS);
    return 0;
  }
  else if(strcmp(command, "cancel") == 0) {
    tasks = StrArrayJoin(args, " ");
    result = APSTClient_CancelTasks(server, tasks);
    FREE(tasks);
  }
  else if(strcmp(command, "disable") == 0)
    result = APSTClient_Disable(server, *args);
  else if(strcmp(command, "download") == 0)
    result = APSTClient_Download(server, *args);
  else if(strcmp(command, "enable") == 0)
    result = APSTClient_Enable(server, *args);
  else if(strcmp(command, "notify") == 0)
    result = APSTClient_Notify(server, *args);
  else if(strcmp(command, "prioritize") == 0) {
    unsigned priority = strtol(*args, NULL, 10);
    if(args[1] == NULL) {
      fprintf(stderr, "%s\n", COMMANDS);
      return 0;
    }
    tasks = StrArrayJoin(args + 1, " ");
    result = APSTClient_SetPriority(server, priority, tasks);
    FREE(tasks);
  }
  else if(strcmp(command, "reset") == 0) {
    tasks = StrArrayJoin(args, " ");
    result = APSTClient_ResetTasks(server, tasks);
    FREE(tasks);
  }
  else {
    fprintf(stderr, "%s\n", COMMANDS);
    return 0;
  }

  if(!result)
    fprintf(stderr, "%s\n", APSTClient_Error());
  return result;

}


int
main(int argc,
     const char **argv) { 

  Address address;
  FILE *apstrc;
  char *colonPlace;
  char command[256] = "";
  char *homeDir;
  char hostName[256] = "";
  int i;
  ParsedArgv pa;
  char *path;
  ASPROP_PropertyList pl = NULL;
  Port port;
  ASPROP_Property prop;
  int result;
  Socket sock = NO_SOCKET;
  char **words;

  if((pa = ParseArgv(argv, VALID, NULL)) == NULL || SwitchPresent(pa, "-h")) {
    fprintf(stderr, "%s\n%s\n%s\n", USAGE, VALID, COMMANDS);
    if(pa != NULL)
      ParsedArgvFree(pa);
    return 1;
  }
  else if(SwitchPresent(pa, "-v")) {
    fprintf(stderr,
            "AppLeS Parameter Sweep Template (APST) Client " APST_VERSION "\n");
    ParsedArgvFree(pa);
    return 0;
  }

  if((homeDir = getenv("HOME")) != NULL) {
    path = StrAppend(NULL, homeDir, "/.apstrc", NULL);
    if((apstrc = fopen(path, "r")) != NULL) {
      if((pl = ASPROP_ReadPropertyList(apstrc)) == NULL)
        fprintf(stderr, "Warning: unable to read %s\n", path);
      fclose(apstrc);
    }
    free(path);
  }

  strncpy(hostName, SwitchValue(pa, "--host", "localhost"), sizeof(hostName)-1);
  if((colonPlace = strchr(hostName, ':')) != NULL) {
    *colonPlace = '\0';
    port = (Port)strtol(colonPlace + 1, NULL, 10);
  }
  else if(pl != NULL &&
          (prop = ASPROP_FindPropertyByName(pl, "APSTD_PORT")) != NULL)
    port = strtol(ASPROP_PropertyValue(prop), NULL, 10);
  else
    port = strtol(DEFAULT_APSTD_PORT, NULL, 10);
  if(!AddressValue(hostName, &address))
    fprintf(stderr, "Failed to resolve host name '%s'", hostName);
  else if((sock = ConnectToTcpPort(address, port)) == NO_SOCKET)
    fprintf(stderr, "Failed to contact apstd %s:%d\n", hostName, (int)port);
  if(pl != NULL)
    ASPROP_PropertyListFree(&pl);
  if(sock == NO_SOCKET) {
    ParsedArgvFree(pa);
    return 1;
  }
  signal(SIGPIPE, SIG_IGN);

  if(SwitchValueN(pa, "", 1, NULL) != NULL) {
    for(i = 2; SwitchValueN(pa, "", i, NULL) != NULL; i++)
      ; /* empty */
    words = MALLOC(i * sizeof(char *));
    for(i = 1;
        (words[i - 1] = (char *)SwitchValueN(pa, "", i, NULL)) != NULL;
        i++)
      ; /* empty */
    result = DoCommand(sock, words[0], (const char **)&words[1]);
    fprintf(stdout, "%s\n", result ? "ok" : "failed");
    FREE(words);
    ParsedArgvFree(pa);
    return result ? 0 : 1;
  }

  ParsedArgvFree(pa);
#ifdef HAVE_READLINE
  using_history();
#endif

  while(GetCommand(command, sizeof(command)) && strcmp(command, "quit") != 0) {
    if(*(words = StrSplit(command, NULL)) != NULL) {
#ifdef HAVE_READLINE
      add_history(command);
#endif
      result = DoCommand(sock, words[0], (const char **)&words[1]);
      fprintf(stdout, "%s\n", result ? "ok" : "failed");
    }
    StrArrayFree(words);
  }

  return 0;

}
