/*
 * Copyright (c) 2000-2003 Illinois Institute of Technology.
 *                         All rights reserved.
 *
 * This file is part of the GHS software package.  For license
 * information, see the LICENSE file in the top level directory of the
 * GHS source distribution.
 *
 * Released Date: August 18 2005
 * This is the GHS I release by the
 *   SCS Group,
 *   http://www.cs.iit.edu/~scs
 *   Department of Computer Science,
 *   Illinois Institute of Technology.
 *
 * This release has been tested under SUN OS 5.9.
 *
 * Supervisor
 * ----------
 * Illiniois Institute of Technology:
 * -Dr. Xian-He Sun
 *
 * Author(s)
 * -----------------
 * Illinois Institute of Technology:
 * -Ming Wu
 * -Xian-He Sun
 *
 */

#include <stdio.h>
#include <math.h>
//#include <unistd.h>
#include <sys/types.h>
#include <signal.h>

#define NPROCS 10000

//double lifetime[NPROCS];

/* Drand: choose a rand floating-point number between zero and one (including both),
 * based on a 31-bit rand integer
 */

double 
Drand ()
{
  return (double) rand() / 32767;
}

/* CHOOSE FROM LOG UNIFORM : low and high are the exponents of the
 * range; i.e. low = 0 and high = 3 would have a range from 1 second
 * to exp(3) seconds
 */
double 
ChooseFromLogUniform(double low, double high)
{
  double x = Drand () * (high - low) + low;
  return exp(x);
}

double 
ChooseFromSerial(double maxValue)
{
  double x;
  int l = 0;

  while (l == 0)
  {
    x = Drand();
    if ( (2.0 / x) < maxValue ) l = 1;
  }

  return (2.0 / x);

}

/* CHOOSE LIFETIME : log uniform distribution between tmin and tmax */

double 
ChooseLifeTime(double maxValue)
{
  return ChooseFromSerial(maxValue);
}

void 
initial_sand(int value)
{
  srand(value);
}

/*  // generate process time following the Log distribution
void 
GenerateLogLifeTime(double *lifeTime, double logMax, int procNum)
{
  int i;

  for (i = 0; i < procNum; i++)
  {
    lifeTime[i] = ChooseLifeTime(logMax);
  }

} */

void 
GenerateRaLifeTime(double *lifeTime, double maxvalue, int procNum)
{
  int i;

  for (i = 0; i < procNum; i++)
  {
    lifeTime[i] = Drand() * maxvalue;
  }

}

void 
GenerateSeLifeTime(double *lifeTime, double maxvalue, int procNum)
{
  int i;

  for (i = 0; i < procNum; i++)
  {
    lifeTime[i] = ChooseLifeTime (maxvalue);
  }

}

// generate process time following the Exponential distribution
void 
GenerateExLifeTime(double *lifeTime, double serv, int procNum)
{
  int i;

  for (i = 0; i < procNum; i++)
  {
    lifeTime[i] = -1 * (1.0 / serv) * log(Drand());
  }

}

// generate process time following the Log Normal distribution
void 
GenerateLNLifeTime(double *lifeTime, double miu, double std, int procNum)
{
  int i;
  int k, valid = 0;
  double u1, u2, u3, x, y;

  for (i = 0; i < procNum; i++)
  {
    valid = 0;
    while (valid == 0) 
    {
      u1 = Drand();
      u2 = Drand();
      x = -1.0 * log(u1);
      //printf(" x is %5.2f exp is %5.2f \n", x, -1 * pow(x - 1, 2.0)/2);
      if (u2 <= exp(-1.0 * pow((x - 1), 2.0) / 2.0)) valid = 1;
    }

    u3 = Drand();
    if (u3 > 0.5) y = x;
    else y = -1.0 * x;
    lifeTime[i] = exp(miu + std * y);
  }

}

// generate process time following the Erlang distribution
void 
GenerateErLifeTime(double *lifeTime, double scaleP, int shapeP, int procNum)
{
  int i;
  int k;
  double tmpproduct = 1.0;

  for (i = 0; i < procNum; i++)
  {
    for (k = 0; k < shapeP; k++) tmpproduct *= Drand(); 
    lifeTime[i] = -1 * scaleP * log(tmpproduct);
    tmpproduct = 1.0;
  }

}

// generate process time following the Gamma distribution
void 
GenerateGaLifeTime(double *lifeTime, double scaleP, double shapeP, int procNum)
{
  int shapeInt;
  int i, k, valid = 0;
  double tmpProduct = 1.0, u1, u2, xb, yb, x, y;

  for (i = 0; i < procNum; i++)
  {
    shapeInt = (int) shapeP;
    if (shapeP == shapeInt)
    {
      for (k = 0; k < shapeP; k++) tmpProduct *= Drand();
      lifeTime[i] = -1 * scaleP * log(tmpProduct);
      tmpProduct = 1.0;
    } 
    else if (shapeP < 1)
    {
      valid = 0;
      while ( valid == 0) 
      {
        u1 = Drand();
        u2 = Drand();
        xb = pow(u1, (1/shapeP));
        yb = pow(u2, (1/(1 - shapeP)));
        if ( (xb + yb) <= 1) valid = 1;
      }		
      x = xb / (xb + yb); 
      y = -1 * log(Drand());
      lifeTime[i] = scaleP * x * y;
    } 
    else
    {
      tmpProduct = 1.0;
      for (k = 0; k < shapeP; k++) tmpProduct *= Drand();
      lifeTime[i] = -1 * scaleP * log(tmpProduct);

      shapeP = shapeP - shapeInt;
      valid = 0;
      while ( valid == 0) 
      {
        u1 = Drand();
        u2 = Drand();
        xb = pow(u1, (1.0 / shapeP));
        yb = pow(u2, (1.0/(1 - shapeP)));
        if ( (xb + yb) <= 1) valid = 1;
      }		
      x = xb / (xb + yb); 
      y = -1 * log(Drand());
      lifeTime[i] += scaleP * x * y;
    }
  }

}

// generate process time following the Truncated Normal distribution
void 
GenerateTNLifeTime(double *lifeTime, double serv, int procNum)
{
  int i;

  for (i = 0; i < procNum; i++)
  {
    lifeTime[i] = -1 * (1.0 / serv) * log(Drand());
  }

}

void 
GenerateInterarri(double *interArri, double arri, int procNum)
{
  int i;
  int l= 0;

  for (i = 0; i < procNum; i++)
  {
    l = 0;
    while (l == 0)
    {
      interArri[i] = (-1) * (1.0 / arri) * log(Drand());
      if (interArri[i] < 1000000) l=1;
    }
  }
	//printf("I am here");
}

/*int 
GetServiceTime(char *configFile) 
{
  int i= 0;

  FILE *stream;
  char line[100];
  float ftemp;

  if ( (stream = fopen(configFile,"r")) == NULL) 
  {
    printf("can't open the service time file\n");
    exit(1);
  }

  i = 0;
  while ( fgets( line, 100, stream ) != NULL) 
  {
    sscanf(line, "%f ", &ftemp);
    lifetime[i] = (double) ftemp;
    i++;
  }

  return(i);

}*/

/* This function calculates the mean value of all the values passed
 *      by the parameter *values. The number of values is also passed in
 *      parameter num_elements
 */

double 
CalculateMean(double *values, int numElements)
{
  int i = 0;
  double mean = 0.0, sd = 0.0, temp = 0.0, var = 0.0;
      
  for(i = 0; i < numElements; i++)
    mean += values[i];
  mean /= numElements;

  return mean;

}

/* This function calculates the standard deviation of all the values passed
 * by the parameter *values. The number of values is also passed in 
 * parameter numElements
 */

double 
CalculateSTD(double *values, int numElements)
{
  int i = 0;
  double mean = 0.0, sd = 0.0, temp = 0.0, var = 0.0;

  mean = CalculateMean(values, numElements);

  for(i = 0; i < numElements; i++)
  {
    temp = values[i] - mean;
    temp = pow(temp, 2.0);
    var += temp;
  }

  if (numElements > 1) sd = var / (numElements - 1);
  else sd = var / numElements;

  sd = sqrt(sd);
  return sd;

}

/* 
main ()
{
  char rtime[10];
  int i, pid, beta=100, miu=10;
  int runtime, sleeptime;
  double unitime;

  srand(32767);

  for (i = 1 ; i < NPROCS; i++)
  {
    lifetime[i] = ChooseLifeTime();
  }
        
  for (i = 1; i < NPROCS; i++)
  {
    printf("%f ", lifetime[i]);
    if (i%10 == 0) printf("\n");
  }

  return(0);

}
*/

