/* $Id: misc.h,v 1.36 2003/09/17 18:14:41 jhayes Exp $ */
/*
 * Copyright  2002 The Regents of the University of California. 
 * All Rights Reserved. 
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for educational, research and non-profit purposes, without
 * fee, and without a written agreement is hereby granted, provided that the
 * above copyright notice, this paragraph and the following three paragraphs
 * appear in all copies. 
 *
 * Permission to incorporate this software into commercial products may be
 * obtained by contacting
 * Technology Transfer Office 
 * 9500 Gilman Drive 
 * 411 University Center 
 * University of California 
 * La Jolla, CA 92093-0093
 * (858) 534-5815
 * invent@ucsd.edu
 *
 * This software program and documentation are copyrighted by The Regents of
 * the University of California. The software program and documentation are
 * supplied "as is", without any accompanying services from The Regents. The
 * Regents does not warrant that the operation of the program will be
 * uninterrupted or error-free. The end-user understands that the program was
 * developed for research purposes and is advised not to rely exclusively on
 * the program for any reason. 
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
 * LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION,
 * EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED
 * HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO
 * OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
 * MODIFICATIONS. 
 */


#ifndef MISC_H
#define MISC_H

#include <sys/time.h> /* struct timeval */
#include <hashseed.h> /* ASH_HashTable */
#include <ipseed.h>   /* ASIP_Port ASIP_Socket */


/* memory management functions and macros */
void my_free(void*);
void *my_malloc(size_t);
void *my_realloc(void*,size_t);
char *my_strdup(const char *);

#undef FREE
#define FREE(x) my_free(x)
#undef MALLOC
#define MALLOC(n) my_malloc(n)
#undef REALLOC
#define REALLOC(x,n) my_realloc(x,n)
#undef STRDUP
#define STRDUP(s) my_strdup(s)


/*
 * If #path# is absolute, returns a copy of it; otherwise, returns #path# with
 * #dir# prepended.
 */
char *
AbsolutePath(const char *path,
             const char *dir);


/*
 * Receives a string on #sock#.  If successful, returns 1 and sets #value# to
 * an allocated copy of the string; otherwise, returns 0.
 */
#undef ReceiveString
int
ReceiveString(ASIP_Socket sock,
              char **value);
/* Sends #value# on #sock#.  Returns 1 if successful, else 0. */
int
SendString(ASIP_Socket sock,
           const char *value);

/*
 * "Adds" #amount# to the value in *#s#.  The value must contain only
 * alphanumeric characters; the result of incrementing other characters is
 * undefined.  The function preserves the letter/digit class of each column.
 * For example, incrementing "Tz9" by one yields "Ua0".  Any carry from the
 * leftmost column is done using the class of the leftmost column.  For
 * example, incrementing "Zz9" by one yields "AAa0".
 */
void
StrIncr(char **s,
        unsigned amount);

/* sendmail */
void
AddEmailNotification(const char *address);

unsigned
SendEmailNotifications(const char *subject,
                       const char *body);

/*
 * Begins capture of process stderr output.  Returns a key that can be passed
 * to the other Stderr functions.
 */
int
StderrCaptureBegin(void);

/*
 * Ends capture of process stderr output.  #key# must be a value previously
 * returned from StderrCaptureBegin; after this function is called, stderr
 * will again be sent to where it was going before that function was called.
 */
void
StderrCaptureEnd(int key);

/*
 * Retrieves the process stderr output captured between the time
 * StderrCaptureBegin returned #key# and the time StderrCaptureEnd was called
 * with that key.  The returned value is allocated and should be freed after
 * use.  Can only be called once with any given #key#; subsequent calls will
 * return NULL.
 */
char *
StderrCaptureRetrieve(int key);


#endif
