/* $Id: predictor.c,v 1.39 2004/03/26 19:02:08 jhayes Exp $ */
/*
 * Copyright  2002 The Regents of the University of California. 
 * All Rights Reserved. 
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for educational, research and non-profit purposes, without
 * fee, and without a written agreement is hereby granted, provided that the
 * above copyright notice, this paragraph and the following three paragraphs
 * appear in all copies. 
 *
 * Permission to incorporate this software into commercial products may be
 * obtained by contacting
 * Technology Transfer Office 
 * 9500 Gilman Drive 
 * 411 University Center 
 * University of California 
 * La Jolla, CA 92093-0093
 * (858) 534-5815
 * invent@ucsd.edu
 *
 * This software program and documentation are copyrighted by The Regents of
 * the University of California. The software program and documentation are
 * supplied "as is", without any accompanying services from The Regents. The
 * Regents does not warrant that the operation of the program will be
 * uninterrupted or error-free. The end-user understands that the program was
 * developed for research purposes and is advised not to rely exclusively on
 * the program for any reason. 
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
 * LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION,
 * EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED
 * HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO
 * OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
 * MODIFICATIONS. 
 */


#include "config.h"
#include <math.h>   /* isnan */
#include <stdlib.h> /* strtod */
#include <time.h>   /* time */
#include <unistd.h> /* getpid */
#include <elmetric.h>
#include <gmingrp.h>
#define ASH_SHORT_NAMES
#include <hashseed.h>
#define ASSTR_SHORT_NAMES
#include <ipseed.h>
#include <strseed.h>
#include <vectorseed.h>
#include "corestructures.h"
#include "global.h"
#include "misc.h"
#include "predictor.h"
#include "ghs.h"


/*
 * About the best we can say about this implementation is that it compiles.  It
 * contains a number of assumptions that are no more than guesses.  Significant
 * testing is needed.
 */


#define MAX_USEFUL_HISTORY 28800 /* 8 hours, completely arbitrary */
#define FEEDBACK_THRESHOLD 100


static HashTable cachedMetrics = NULL;
static GMINGRP_GroupInfo gminGroup = NULL;
static HashTable idsWithNoKey = NULL;
static char *localName;

static char *METRIC_IMAGES[ELMETRIC_METRIC_COUNT + 1] = {
  "cpuCount", "cpuLoad", "bandwidth", "latency", "arrRate", "timeDev", "compCapa", NULL
};
static const char *SERVICE_IMAGES[ELMETRIC_METRIC_SERVICE_COUNT + 1] = {
  "local", "ganglia", "mds", "nws", "ghs", NULL
};

static void
CachedMetricReclaimer(KeyType id,
                      ValueType value) {
  ELMETRIC_Metric m = (ELMETRIC_Metric)value;
  FREE(id);
  ELMETRIC_FreeMetric(m);
}


static void
IdReclaimer(KeyType id,
            ValueType value) {
  FREE(id);
}


static char *
GetMetricValue(ELMETRIC_MetricTypes metric,
               const char *entity,
               Host h) {

  char *id = StrAppend(NULL, entity, ".", METRIC_IMAGES[metric], NULL);
  HashIterator it;
  ELMETRIC_Key k;
  ELMETRIC_Metric m;

  if (APST_DEBUG_ON == 1) printf("the entity is %s \n", entity);
  if (APST_DEBUG_ON == 1) printf("the id is %s \n", id);

  if(GetValue(idsWithNoKey, id, NULL) != NULL) {
    FREE(id);
    return NULL;
  }
  else if((m = (ELMETRIC_Metric)GetValue(cachedMetrics,id,NULL)) == NULL) {
    k = NULL;
    ForEachMapping(global.gridInfoSources, it) {
      GridInfoSource gis = (GridInfoSource)IteratorValue(it);
      if((k = ELMETRIC_GetKey(gis->method,gis->server,entity,metric)) != NULL) {
	if (APST_DEBUG_ON == 1) printf("the key is found \n");
        //printf("the server is %s and the entity is %s \n", k->server, k->entity);
        break;
      }
    }
    if(k == NULL && gminGroup != NULL && h != NULL &&
       GMINGRP_GroupAdd(gminGroup, h->disk->method, h->disk->server, h->method,
                        entity))
      k = ELMETRIC_GetKey
        (ELMETRIC_GANGLIA, GMINGRP_GroupServer(gminGroup), entity, metric);
    if(k == NULL) {
      PutMapping(idsWithNoKey, id, (void *)1);
      return NULL;
    }
  }
  else if(ELMETRIC_GetMetricExpiration(m) > time(NULL)) {
    FREE(id);
    return ELMETRIC_GetMetricValue(m);
  }
  else {
    k = ELMETRIC_GetMetricKey(m);

//if(metric != ELMETRIC_JOB_ARRIVAL_RATE)
if( (metric != ELMETRIC_CPU_LOAD) && (metric != ELMETRIC_JOB_ARRIVAL_RATE) && (metric != ELMETRIC_SERVICE_TIME_DEVIATION) && (metric != ELMETRIC_COMPUTING_CAPACITY) )
    RemoveMapping(cachedMetrics, id, NULL);
  }

  if (APST_DEBUG_ON == 1) printf("I am here before getMetric(k) \n");
  if((m = ELMETRIC_GetMetric(k)) == NULL) {
    if (APST_DEBUG_ON == 1) printf("I am in null at ELMETRIC_GetMetric(k) \n");
    ELMETRIC_FreeKey(k);
    return NULL;
  }
  if (APST_DEBUG_ON == 1) printf("I am here after getMetric(k) \n");
//if(metric != ELMETRIC_JOB_ARRIVAL_RATE)
if( (metric != ELMETRIC_CPU_LOAD) && (metric != ELMETRIC_JOB_ARRIVAL_RATE) && (metric != ELMETRIC_SERVICE_TIME_DEVIATION) && (metric != ELMETRIC_COMPUTING_CAPACITY) )
  PutMapping(cachedMetrics, id, m);
  LOG2("New metric value %s for %s\n", ELMETRIC_GetMetricValue(m), id);
  return ELMETRIC_GetMetricValue(m);

}

int GetParasValue(double *utils, double *arris, double *servstds, double *speeds, int hostNum, Host *hosts) {
  int i,j,k;
  char *value;
  Host h;

  k = 0;
  for (i = 0; i< hostNum; i++) {
    h = hosts[i];
    for (j = 0; j < h->cpus; j++) {
    	if (APST_DEBUG_ON == 1) printf("the host id is %s\n", h->id);
    	value = GetMetricValue(ELMETRIC_CPU_LOAD,
                         h->method == ELBASE_FORK ? localName :
                         h->dnsName == NULL ? h->server : h->dnsName, h);
    	if (APST_DEBUG_ON == 1) printf("the value is %s \n", value);
    	utils[k] = strtod(value, NULL);
    	value = GetMetricValue(ELMETRIC_JOB_ARRIVAL_RATE,
                         h->method == ELBASE_FORK ? localName :
                         h->dnsName == NULL ? h->server : h->dnsName, h);
    	if (APST_DEBUG_ON == 1) printf("the value is %s \n", value);
    	arris[k] = strtod(value, NULL);
   	value = GetMetricValue(ELMETRIC_SERVICE_TIME_DEVIATION,
                         h->method == ELBASE_FORK ? localName :
                         h->dnsName == NULL ? h->server : h->dnsName, h);
    	if (APST_DEBUG_ON == 1) printf("the value is %s \n", value);
    	servstds[k] = strtod(value, NULL);
    	value = GetMetricValue(ELMETRIC_COMPUTING_CAPACITY,
                         h->method == ELBASE_FORK ? localName :
                         h->dnsName == NULL ? h->server : h->dnsName, h);
    	if (APST_DEBUG_ON == 1) printf("the value is %s \n", value);
    	speeds[k] = strtod(value, NULL);
	k++;
    }
  }
  return k;
}

unsigned long
PredictFileTransfer(FileTransfer ft) {

  FileTransfer *last;
  char *linkName;
  FileTransfer *next;
  long now = time(NULL);
  FileTransfer old;
  double secsPerByte;
  double totalSize = 0.0;
  double totalTime = 0.0;
  char *value;

  /*
   * Compute the number of seconds per byte that transfers already completed on
   * this link have taken.
   */
  last = ft->source->transfersCompleted +
         ASV_Size(ft->source->transfersCompleted) - 1;
  for(next = ft->source->transfersCompleted; next <= last; next++) {
    old = *next;
    if(old->source != ft->dest && old->dest != ft->dest)
      continue; /* Different link */
    if(now - old->meta.timeCompleted > MAX_USEFUL_HISTORY)
      continue; /* Stale data */
    totalSize += old->file->size;
    totalTime += old->meta.timeCompleted - old->meta.timeStarted;
  }
  secsPerByte = totalSize == 0 ? 0.0001 : (totalTime / totalSize);

  linkName = ASSTR_StrAppend
    (NULL, ft->source->method == ELBASE_CP ? localName : ft->source->server,
     "-", ft->dest->method == ELBASE_CP ? localName : ft->dest->server, NULL);
  if((value = GetMetricValue(ELMETRIC_TCP_LATENCY, linkName, NULL)) == NULL)
    ft->meta.latency = 0.0;
  else {
    ft->meta.latency = strtod(value, NULL) / 1000.0;
    if(isnan(ft->meta.latency))
      ft->meta.latency = 0.0;
  }
  if((value = GetMetricValue(ELMETRIC_TCP_BANDWIDTH, linkName, NULL)) == NULL)
    ft->meta.bandwidth = 1000.0;
  else {
    ft->meta.bandwidth = strtod(value, NULL) / 8.0;
    if(isnan(ft->meta.bandwidth))
      ft->meta.bandwidth = 1000.0;
  }
  FREE(linkName);

  return (unsigned long)(ft->meta.latency+ft->file->size/ft->meta.bandwidth);

}


static GridInfoSource
ProcessInfoSourceXml(ASXML_XmlNode node) {

  unsigned long i;
  char *id;
  char *server;
  ELMETRIC_MetricServiceTypes service;

  if (APST_DEBUG_ON == 1) printf("I am in the begining of ProcessInfoSourceXml \n");
  if(*(id = ASXML_ElementGetAttribute(node, "id")) == '\0')
    FAIL("'infosource' tag requires an 'id' attribute\n");
  if((node = ASXML_NodeFirstChildByType(node, ASXML_SHOW_ELEMENT)) == NULL)
    i = (int)ELMETRIC_LOCAL;
  else if(!ASSTR_StrArrayFind(SERVICE_IMAGES, ASXML_NodeName(node), &i))
    FAIL1("Unknown infosource subtag '%s'\n", ASXML_NodeName(node));
  if((service = (ELMETRIC_MetricServiceTypes)i) != ELMETRIC_LOCAL) {
    server = ASXML_ElementGetAttribute(node, "server");
    if (APST_DEBUG_ON == 1) printf("I am here in ProcessInfoSourceXml \n");
  }
  else if(gminGroup == NULL) {
    if((gminGroup = GMINGRP_GroupNew(getpid() % 5000 + 5000)) == NULL)
      FAIL("Unable to establish local gridinfo server\n");
    server = GMINGRP_GroupServer(gminGroup);
    service = ELMETRIC_GANGLIA;
  }

  if(service == ELMETRIC_GHS) {
    //server = "myGHS";
    server = ASXML_ElementGetAttribute(node, "server");
    if (APST_DEBUG_ON == 1) printf("I got GHS!\n");
    return GridInfoSource_New(STRDUP(id), service, STRDUP(server));
  }

  if(!ELMETRIC_ValidServer(service, server)) {
    if (APST_DEBUG_ON == 1) printf("I am here in validServer ProcessInfoSourceXml \n");
    FAIL1("Unable to contact infosource server %s\n", server);
  }
  return GridInfoSource_New(STRDUP(id), service, STRDUP(server));

}


int
predictor_init(void) {
  cachedMetrics = HashTableNew
    (MEDIUM_HASH, &StringCompare, &StringHash, &CachedMetricReclaimer);
  idsWithNoKey = HashTableNew
    (MEDIUM_HASH, &StringCompare, &StringHash, &IdReclaimer);
  localName = ASIP_MyMachineName();
  return 1;
}


int
predictor_process_xml(ASXML_XmlNode node) {
  ASXML_XmlNode child;
  int oldCount = ASH_GetMappingCount(global.gridInfoSources);
  int result = 1;
  GridInfoSource source;
  for(child = ASXML_NodeFirstChildByType(node, ASXML_SHOW_ELEMENT);
      child != NULL;
      child = ASXML_NodeNextSiblingByType(child, ASXML_SHOW_ELEMENT)) {
    if(strcmp(ASXML_NodeName(child), "infosource") == 0) {
      if((source = ProcessInfoSourceXml(child)) == NULL)
        result = 0;
      else
	{
        GridInfoSourceList_Add(global.gridInfoSources, source);
	}
    }
    else {
      ERROR1("Unknown gridinfo subtag '%s'\n", ASXML_NodeName(child));
      result = 0;
    }
  }
  /* Add a source for local info if any others have been defined. */
  if(oldCount == 0 && ASH_GetMappingCount(global.gridInfoSources) > 0)
    GridInfoSourceList_Add
      (global.gridInfoSources,
       GridInfoSource_New(STRDUP("_localinfo"), ELMETRIC_LOCAL, STRDUP("")));
  return result;
}


int
predictor_shutdown(void) {
  HashTableFree(cachedMetrics);
  cachedMetrics = NULL;
  if(gminGroup != NULL) {
    GMINGRP_GroupFree(gminGroup);
    gminGroup = NULL;
  }
  HashTableFree(idsWithNoKey);
  idsWithNoKey = NULL;
  FREE(localName);
  return 1;
}


unsigned long
PredictTaskOnHost(Task t,
                  Host h) {

  double cpuSecsPerCostUnit;
  long lastCompleted = 0;
  Task *last;
  Task *next;
  long now = time(NULL);
  double taskCost;
  double totalCostUnits = 0.0;
  double totalCpuTime = 0.0;
  char *value;

  /*
   * Compute the number of CPU seconds per cost unit that tasks already
   * completed on #host# have taken.
   */
  last = h->tasksCompleted + ASV_Size(h->tasksCompleted) - 1;
  for(next = h->tasksCompleted; next <= last; next++) {
    Task completed = *next;
    if(now - completed->meta.timeCompleted > MAX_USEFUL_HISTORY)
      continue; /* Stale data */
    totalCostUnits += completed->meta.cost == 0 ? 1 : completed->meta.cost;
    totalCpuTime +=
      (completed->meta.timeCompleted - completed->meta.timeStarted) *
      completed->meta.cpuShare;
    if(completed->meta.timeCompleted > lastCompleted)
      lastCompleted = completed->meta.timeCompleted;
  }
  /*
   * TODO If totalCostUnits == 0, should we base a computation on tasks
   * completed by other hosts?
   */
  cpuSecsPerCostUnit =
    totalCostUnits == 0 ? 1.0 : (totalCpuTime / totalCostUnits);

  value = GetMetricValue(ELMETRIC_CPU_LOAD,
                         h->method == ELBASE_FORK ? localName :
                         h->dnsName == NULL ? h->server : h->dnsName, h);
/*
  value = GetMetricValue(ELMETRIC_JOB_ARRIVAL_RATE,
                         h->method == ELBASE_FORK ? localName :
                         h->dnsName == NULL ? h->server : h->dnsName, h);
  if (APST_DEBUG_ON == 1) printf("\nJOB_ARRIVAL_RATE for %s is %s.\n", h->server, value);
*/

  if(value == NULL)
    t->meta.cpuShare = 1.0; /* Assume unloaded. */
  else {
    t->meta.cpuShare = 1.0 / (strtod(value, NULL) + 1.0);
    if(isnan(t->meta.cpuShare))
      t->meta.cpuShare = 1.0;
    else if((now - lastCompleted) < FEEDBACK_THRESHOLD) {
      /* Remove the effect of our previous task from the estimate. */
      int nonApstProcesses = (int)(1.0 / t->meta.cpuShare) - 2;
      t->meta.cpuShare =
        nonApstProcesses < 0 ? 1.0 : (1.0 / (nonApstProcesses + 1.0));
    }
    if(t->meta.cpuShare > 1.0)
      t->meta.cpuShare = 1.0; /* A single task can't use > 1 CPU */
  }

  taskCost = t->meta.cost == 0 ? 1 : t->meta.cost;
  return (unsigned long)(cpuSecsPerCostUnit * taskCost / t->meta.cpuShare);

}
