/*
 * Copyright (c) 2000-2003 Illinois Institute of Technology.
 *                         All rights reserved.
 *
 * This file is part of the GHS software package.  For license
 * information, see the LICENSE file in the top level directory of the
 * GHS source distribution.
 *
 * Released Date: August 18 2005
 * This is the GHS I release by the
 *   SCS Group,
 *   http://www.cs.iit.edu/~scs
 *   Department of Computer Science,
 *   Illinois Institute of Technology.
 *
 * This release has been tested under SUN OS 5.9.
 *
 * Supervisor
 * ----------
 * Illiniois Institute of Technology:
 * -Dr. Xian-He Sun
 *
 * Author(s)
 * -----------------
 * Illinois Institute of Technology:
 * -Ming Wu
 * -Xian-He Sun
 *
 */

#include<stdio.h>
#include<math.h>
#include<stdlib.h>
#include<time.h>
#define MAX_WS_NUM 20000

extern double GammaCDF(double x, double a, double b);

int ptWsNum;
double ptUtil[MAX_WS_NUM], ptArri[MAX_WS_NUM], ptServSTD[MAX_WS_NUM], ptWork[MAX_WS_NUM];
double a[MAX_WS_NUM], b[MAX_WS_NUM];

void 
ReadTaskPara(double *ut, double *ar, double *ss, double *wo, int mNum) 
{
  int i;

  ptWsNum = mNum;

  for (i = 0; i < ptWsNum; i++) 
  {
    ptUtil[i] = ut[i];
    ptArri[i] = ar[i];
    ptServSTD[i] = ss[i];
    ptWork[i] = wo[i];
  }
}

double 
GetSubMean(double sUtil, double sArri, double sServ, double sWork)
{
  double subMean;

  subMean = (1 / (1 - exp(-1 * sArri * sWork))) * (sUtil / (1 - sUtil)) * sWork;
  //printf("the subMean is %f %f %f %f\n", subMean, sUtil, sArri, sWork);
  return(subMean);

}

double 
GetSubVari(double sUtil, double sArri, double sServ, double sWork)
{
  double subVari;
  double capa, theta, mu;

  capa = 1 - sUtil;
  mu = sArri / sUtil;
  theta = sServ * mu;
  subVari = (1/(1 - exp(-sArri * sWork))) * (sUtil/(capa * capa * capa)) * ((theta * theta + 1)/mu) * sWork;
  //printf("the subVari is %f %f %f %f\n", subVari, sUtil, sArri, sWork);
  return(subVari);

}

void 
CalculateAB()
{
  int i;
  double atemp,btemp;

  for (i = 0; i < ptWsNum; i++)
  {
    btemp = GetSubVari(ptUtil[i], ptArri[i], ptServSTD[i], ptWork[i]) / GetSubMean(ptUtil[i], ptArri[i], ptServSTD[i], ptWork[i]);
    b[i] = btemp;
    atemp = GetSubMean(ptUtil[i], ptArri[i], ptServSTD[i], ptWork[i]) / btemp;
    a[i] = atemp;
  }

}

double 
CalculateA(double sUtil, double sArri, double sServ, double sWork)
{
  double aTemp, tmp;

  tmp = GetSubMean(sUtil, sArri, sServ, sWork);
  aTemp = (tmp * tmp) / GetSubVari(sUtil, sArri, sServ, sWork);
  return(aTemp);

}

double 
CalculateB(double sUtil, double sArri, double sServ, double sWork)
{
  double bTemp, tmp;

  tmp = GetSubMean(sUtil, sArri, sServ, sWork);
  bTemp = GetSubVari(sUtil, sArri, sServ, sWork) / tmp;
  return(bTemp);

}

/* calculate the subtask completion time CDF */
//double GetSkCDF(double sUtil, double sArri, double sServ, double sWork, double taskTime)
double 
GetSkCDF(double a, double b, double sWork, double taskTime)
{
  double skCDF;

  //b = GetSubVari(sUtil, sArri, sServ, sWork) / GetSubMean(sUtil, sArri, sServ, sWork);
  //a = GetSubMean(sUtil, sArri, sServ, sWork) / b;

  //printf("a is %f and b is %f \n", a, b);

  skCDF = (double) GammaCDF((double)(taskTime - sWork), (double)a, (double)b);

  if (skCDF < 0)
  {
    printf("skCDF is less than 0");
    exit(0);
  }
	//printf("the skCDF is %f \n", skCDF);
  return(skCDF);

}

// calculate the CDF of the first part of formula: max{T(swork1), migrtime)
double 
GetNewSkCDF(double sUtil, double sArri, double sServ, double sWork1, double migrTime, double taskTime)
{
  double skCDF, elambda;
  double aTemp, bTemp;
  double maxTemp;

  aTemp = CalculateA(sUtil, sArri, sServ, sWork1);
  bTemp = CalculateB(sUtil, sArri, sServ, sWork1);

  if (sWork1 <= 0)
  {
    printf("skCDF is less than 0");
    exit(0);
  }

  if (sWork1 > migrTime) maxTemp = sWork1;
  else maxTemp = migrTime;

  if ( taskTime >= maxTemp)
  {
    elambda = exp(-sArri * sWork1);
    skCDF = elambda + (1 - elambda) * GetSkCDF(aTemp, bTemp, sWork1, taskTime);
  }
  else
  {
    skCDF = 0;
  }
	//printf("the new skCDF for %f (sWork1) %f (migrTime) and %f (taskTime)is %f \n", sWork1, migrTime, taskTime, skCDF);
  return(skCDF);

}

/* 
double get_CDF(double taskTime, double a, double b)
{
	double skCDF;

	skCDF = (double) GammaCDF((double)taskTime, (double)a,(double)b);

	if (skCDF < 0)
	{
		printf("skCDF is less than 0");
		exit(0);
	}

	return(skCDF);
} */

/*
double get_variance(int numOfinte, double a, double b)
{
	double paramean=0, mean=0, timeInterval, temp;
	double maxwork=0, tasktime;

	int i;

	mean = a*b;
	timeInterval = (10 * 100)/numOfinte;
	//printf("the maxwork is %f and the timeInterval is %f \n", maxwork, timeInterval);

	for (i=0;i<numOfinte;i++) 
	{
		tasktime = timeInterval*i;
		temp=(tasktime+0.5*timeInterval-mean)*(tasktime+0.5*timeInterval-mean);
		paramean += temp*(get_CDF(tasktime+timeInterval, a, b)-get_CDF(tasktime, a, b));
		//printf("the paramean is %f \n", paramean);
	}

	return(paramean);
	
} */

/* calculate the parallel task completion time CDF */
double 
GetParaCDF(double taskTime)
{
  double paraCDF = 1, elambda;
  int i;

  for (i = 0; i < ptWsNum; i++)
  {
    if (ptWork[i] != 0)
    {
      elambda = exp(-ptArri[i] * ptWork[i]);
      paraCDF = paraCDF * (elambda + (1 - elambda) * GetSkCDF(a[i], b[i], ptWork[i], taskTime));
    }
  }

  return(paraCDF);

}

/* calculate the expectation time of parallel task completion time*/
double 
GetParaMean(int numOfInte)
{
  double paramean = 0, timeInterval;
  double maxWork = 0, taskTime;
  double newCDF, oldCDF;

  int i;

  CalculateAB();

  for (i = 0; i < ptWsNum; i++)
  {
    if (maxWork < ptWork[i]) maxWork = ptWork[i];
  }

  timeInterval = (10 * maxWork) / numOfInte;
  //printf("the maxWork is %f and the timeInterval is %f \n", maxWork, timeInterval);

  oldCDF = GetParaCDF(maxWork);

  for (i = 0; i < numOfInte; i++) 
  {
    taskTime = (maxWork + timeInterval * i);
    newCDF = GetParaCDF(taskTime + timeInterval);
    paramean += (taskTime + 0.5 * timeInterval) * (newCDF - oldCDF);
    oldCDF = newCDF;
    //printf("the paramean is %f \n", paramean);
  }

  return(paramean);

}

// return the mean of the first part of formula: max{T(swork1), migrtime)
double 
GetSkMean(double sUtil, double sArri, double sServ, double sWork1, double migrTime, int numOfInte)
{
  int i;
  double skMean = 0, taskTime, timeInterval, maxWork;
  double newCDF, oldCDF;

  maxWork = sWork1;
  if (maxWork < migrTime)
  {
    maxWork = migrTime;
  }
  timeInterval = (10 * maxWork) / numOfInte;

  oldCDF = GetNewSkCDF(sUtil, sArri, sServ, sWork1, migrTime, maxWork);

  for (i = 0; i < numOfInte; i++) 
  {
    taskTime = maxWork + timeInterval * i;
    newCDF = GetNewSkCDF(sUtil, sArri, sServ, sWork1, migrTime, (taskTime+timeInterval));
    skMean += (taskTime + 0.5 * timeInterval) * (newCDF - oldCDF);
    //printf("the paramean is %f and diff is %f\n", skMean, (newCDF-oldCDF));
    oldCDF = newCDF;
  }

  return(skMean);

}

// return the mean of the whole formula: max{T(sWork1), migrTime)+T(swork2)
double 
GetNewSkMean(double sUtil, double sArri, double sServ, double sWork1, double swork2, double migrTime, int numOfInte)
{
  double skMean = 0;

  skMean = GetSkMean(sUtil, sArri, sServ, sWork1, 200, numOfInte);
  //printf("the skMean 1 is %f \n", skMean);
  skMean += GetSubMean(sUtil, sArri, sServ, swork2);
  //printf("the skMean 2 is %f \n", skMean);

  return(skMean);

}

double 
GetParaVari(double paraMean, int numOfInte)
{

  double paraVari = 0, timeInterval;
  double maxWork = 0, taskTime;
  double xTemp = 0, xVari = 0;

  int i;

  for (i = 0; i< ptWsNum; i++)
  {
    if (maxWork < ptWork[i]) maxWork = ptWork[i];
  }

  timeInterval = (10 * maxWork) / numOfInte;
  //printf("the maxWork is %f and the timeInterval is %f \n", maxWork, timeInterval);

  for (i = 0; i < numOfInte; i++)
  {
    taskTime = (maxWork + timeInterval * i);
    xTemp = taskTime + 0.5 * timeInterval;
    xVari = (xTemp - paraMean) * (xTemp - paraMean);
    paraVari += xVari * (GetParaCDF(taskTime + timeInterval) - GetParaCDF(taskTime));
    //printf("the paraMean is %f \n", paraMean);
  }

  return(paraVari);

}

// return the variance of the first part of formula: max{T(swork1), migrtime)
double 
GetSkVari(double sUtil, double sArri, double sServ, double sWork1, double migrTime, int numOfInte)
{
  int i;
  double skVari = 0, skMean, taskTime, timeInterval, maxWork;
  double xTemp = 0, xVari = 0;
  double tmp1, tmp2;

  maxWork = sWork1;
  timeInterval = (10 * maxWork) / numOfInte;

  skMean = GetSkMean(sUtil, sArri, sServ, sWork1, migrTime, numOfInte);

  for (i = 0; i < numOfInte; i++) 
  {
    taskTime = maxWork + timeInterval * i;
    xTemp = taskTime + 0.5 * timeInterval;
    xVari = (xTemp - skMean) * (xTemp - skMean);
    tmp1 = GetNewSkCDF(sUtil, sArri, sServ, sWork1, migrTime, (taskTime + timeInterval));
    tmp2 = GetNewSkCDF(sUtil, sArri, sServ, sWork1, migrTime, taskTime);
    skVari += xVari * (tmp1 - tmp2);
  }

  return(skVari);

}

// calculate the variance of the whole formula: max{T(sWork1), migrTime) + T(sWork2)
double 
get_new_sk_vari(double sUtil, double sArri, double sServ, double sWork1, double sWork2, double migrTime, int numOfInte)
{
  double skVari = 0;

  skVari = GetSkVari(sUtil, sArri, sServ, sWork1, migrTime, numOfInte);
  skVari += GetSubVari(sUtil, sArri, sServ, sWork2);

  return(skVari);

}
/*
int main(int argc, char **argv)
{

	//double d1 = -0.5772156649015328605195174;
	double paramean=0.0, paravari=0.0;
	clock_t start, finish;

	get_mappara(argv[1]);
	start = clock();

	CalculateAB();

	paramean = GetParaMean(100);
	paravari = GetParaVari(paramean, 100);
	

	finish = clock();

	//paramean = get_variance(100, 5, 3);

	printf("the paramean is %f \n", paramean/60.0);
	printf("the paravari is %f \n", paravari/3600.0);
	printf("\nthe total time is %2.4f ", (double)(finish-start)/CLOCKS_PER_SEC );

	//paramean = get_variance(100, 3, 5);
	//printf("the paramean is %f", paramean);

	return(0);
}
*/
