/*
 * Copyright (c) 2000-2003 Illinois Institute of Technology.
 *                         All rights reserved.
 *
 * This file is part of the GHS software package.  For license
 * information, see the LICENSE file in the top level directory of the
 * GHS source distribution.
 *
 * Released Date: August 18 2005
 * This is the GHS I release by the
 *   SCS Group,
 *   http://www.cs.iit.edu/~scs
 *   Department of Computer Science,
 *   Illinois Institute of Technology.
 *
 * This release has been tested under SUN OS 5.9.
 *
 * Supervisor
 * ----------
 * Illiniois Institute of Technology:
 * -Dr. Xian-He Sun
 *
 * Author(s)
 * -----------------
 * Illinois Institute of Technology:
 * -Ming Wu
 * -Xian-He Sun
 *
 */

#include<stdio.h>
#include<math.h>
#include<stdlib.h>
#include<time.h>
#include "ghs.h"


int wsNum, gJobNum;

extern double GetParaMean(int numOfinte);
extern double GetParaVari(double paramean, int numOfinte);
extern void ReadTaskPara(double *ut, double *ar, double *ss, double *wo, int mnum);

double serv_p[MAX_WS_NUM], servtmp[MAX_WS_NUM], arri_p[MAX_WS_NUM], servstd_p[MAX_WS_NUM], work_p[MAX_WS_NUM], speed_p[MAX_WS_NUM], util_p[MAX_WS_NUM];

double mUtil[MAX_WS_NUM], mArri[MAX_WS_NUM], mServSTD[MAX_WS_NUM], mWork[MAX_WS_NUM];

double totalWorkload;

// a list of machines status, 1 stands for the machine will be used
int machID[MAX_WS_NUM];

// 
double optimalWorkload[MAX_WS_NUM];

// sort the machines with (1-utilization)*speed
int machsOrder[MAX_WS_NUM];

int firstItemIndex[MAX_WS_NUM], lastItemIndex[MAX_WS_NUM];

char machName[MAX_WS_NUM][30];
char taskName[MAX_WS_NUM][30];

/*  read resource parameters
 *	resource name, utilization, arrival rate, servicestd
 */
void 
GetResoPara(char *configFile) 
{
  int i = 0;
  FILE *stream;
  char line[100];
  float tmpUtil, tmpArri, tmpServ, tmpSpeed;

  if ( (stream = fopen(configFile, "r")) == NULL) 
  {
    printf("can't open the resource parameter file\n");
    exit(1);
  }

  while ( fgets(line, 100, stream) != NULL) 
  {
    sscanf(line, "%s %f %f %f %f", &machName[i], &tmpUtil, &tmpArri, &tmpServ, &tmpSpeed);
    util_p[i] = tmpUtil;
    arri_p[i] = tmpArri;
    serv_p[i] = util_p[i] / arri_p[i];
    servstd_p[i] = tmpServ;
    speed_p[i] = tmpSpeed;
    //printf(" %f %f %f %f \n", util_p[i], arri_p[i], servstd_p[i], speed_p[i]);
    i++;
  }

  wsNum = i;
  fclose(stream);

}

// 0->1->01->11->001->101->011->111
void 
AddArray()
{
  int i = 0;
  int endSign;

  endSign = 0;
  while (i < wsNum && (endSign == 0) )
  {
    if ( machID[i]== 0)
    {
      machID[i] = 1;
      endSign = 1;
    }
    else // machID is 1 already
    {
      machID[i] = 0;
    }
    i++;
  }

}

int 
IsArrayFull()
{
  int i = 0;
  int full = 0;

  while (i < wsNum && machID[i] == 1)
  {
    i++;
  }

  if (i == wsNum) full = 1;
  return(full);

}

// task partition according to (1-utilization)* speed 
void 
PartitionWorkload(int *machinearray)
{
  int i = 0, j = 0;
  // the sum of (1-utilization)* speed
  double utilspeed = 0;

  for (i = 0; i < wsNum; i++) 
  {
    if (machinearray[i] == 1) 
    {
      utilspeed += (1 - arri_p[i] * serv_p[i]) * speed_p[i];
    }
  }

  for (i = 0; i < wsNum; i++) 
  {
    if (machinearray[i] == 1) 
    {
      work_p[i] = totalWorkload * ((1 - arri_p[i] * serv_p[i]) * speed_p[i]) / utilspeed;
    }
  }
}

/* record the current scheduling plan as the best scheduling plan
 */
void 
PlanRecord()
{
  int i,j;

  for (i = 0; i < wsNum; i++) 
  {
    optimalWorkload[i] = work_p[i];
  }

}

/* calcuate the mean+variance of the parallel task runtime
 * using model formulas 
 */
double 
CalculateRuntime(int *machineArray)
{
  int i = 0, j = 0;
  float meanTmp, variTmp;
  double subtaskTime;

  for (i = 0; i < wsNum; i++) work_p[i] = 0;

  PartitionWorkload(machineArray);

  j = 0;
  for (i = 0; i < wsNum; i++) 
  {
    if (machineArray[i] == 1) 
    {
      mUtil[j] = arri_p[i] * serv_p[i];
      mArri[j] = arri_p[i];
      mServSTD[j] = servstd_p[i];
      mWork[j] = work_p[i] / speed_p[i];
      //printf("%f %f %f %f \n", mUtil[j], mArri[j], mServSTD[j], mWork[j]);
      j++;
    }
  }

  ReadTaskPara(mUtil, mArri, mServSTD, mWork, j);
  meanTmp = GetParaMean(XNUM);
  //printf("the meanTmp is %f\n", meanTmp);
  //variTmp = GetParaVari(meanTmp, XNUM);
  //return(meanTmp+variTmp);
  /* if ( meanTmp < 2)
  {
    for (i = 0; i < wsNum; i++) 
    {
      if (machineArray[i] == 1) 
      {
        subtaskTime = work_p[i] / speed_p[i];
        printf("the machine is %d and subtaskTime is %f \n", i, subtaskTime);
      }
    }
    //printf("..............\n");
  } */

  return(meanTmp);

}

/* sort the machine list according to (1-utilization)* speed 
 * the result is stored in machsOrder array
 */
void 
SortMachID()
{
  int i = 0, j = 0, orderTmp;
  double uspeed[MAX_WS_NUM], ustemp;

  for (i = 0; i < wsNum; i++)
  {
    machsOrder[i] = i;
    uspeed[i] = (1-arri_p[i] * serv_p[i]) * speed_p[i];
  }

  for (i = 0; i < wsNum - 1; i++)
  {
    for (j = i + 1; j < wsNum; j++)
    {
      if ( uspeed[i] < uspeed[j] ) 
      {
        ustemp = uspeed [i];
        uspeed[i] = uspeed[j];
        uspeed[j] = ustemp;

        orderTmp = machsOrder[i];
        machsOrder[i] =  machsOrder[j];
        machsOrder[j] = orderTmp;
      }
    }
  }

}

/* generate the machine id array according to the machsOrder array */
void 
GenerateMachID(int machNum)
{

  int i;
  for (i = 0; i < wsNum; i++) machID[i] = 0;

  for (i = 0; i < machNum; i++)
  {
    machID[machsOrder[i]] = 1;
  }

}

/* optimal scheduling algorithm, return parallel task run time */
double 
OptimalSchedule()
{
  int i;
  double evMin, evTmp = 0.0;

  for (i = 0; i < wsNum; i++) machID[i] = 0;

  i = 0;
  while (IsArrayFull() == 0)
  {
    AddArray();
    evTmp = CalculateRuntime(machID);
    if (i == 0) 
    {
      evMin = evTmp;
      PlanRecord();
    }
    if ( evMin > evTmp )
    {
      evMin = evTmp;
      PlanRecord();
    }
    i++;
  }

  return(evMin);

}

/* heuristic scheduling algorithm, return parallel task run time */
double 
HeuristicSchedule()
{
  int i;
  int a, b, c, maxNum;
  double maxServ = 0;
  double runa, runb, runc, runc1;
  double evMin;
  double subtaskTime;


  SortMachID();

  for (i = 0; i < wsNum; i++)
  {
    if (maxServ < serv_p[i] ) maxServ = serv_p[i];
  }

  //**********************need modification ****************************
  maxNum = (int)(totalWorkload / (2 * maxServ));

  a = 1;
  if (wsNum > maxNum)
  {
    b = maxNum;
  } else b = wsNum;


  while ( a + 1 < b) 
  {
    c = (a + b) / 2;
    GenerateMachID(a);
    runa = CalculateRuntime(machID);
    GenerateMachID(b);
    runb = CalculateRuntime(machID);
    GenerateMachID(c);
    runc = CalculateRuntime(machID);

    if ( (runa < runb) && (runa < runc))
    {
      b = c;
    }
    else if ( (runb < runa) && (runb < runc) )
    {
      a = c;
    }
    else
    {
      GenerateMachID(c + 1);
      runc1 = CalculateRuntime(machID);
      if ( runc < runc1) 
      {
        b = c;
      }
      else a = c;
    }
  }

  GenerateMachID(a);
  runa = CalculateRuntime(machID);
  PlanRecord();

  GenerateMachID(b);
  runb = CalculateRuntime(machID);

  if ( runa < runb) evMin = runa;
  else 
  {
    evMin = runb;
    PlanRecord();
  }

  if (  evMin == runb) GenerateMachID(b);
  else GenerateMachID(a);

  for (i = 0; i < wsNum; i++) 
  {
    if (machID[i] == 1) 
    {
      subtaskTime = work_p[i] / speed_p[i];
      //printf("the machine is %d and subtaskTime is %f \n", i, subtaskTime);
    }
  }
  //printf("..............\n");
  //printf("the machines order \n");
  //for (i = 0; i <wsNum; i++) printf("%d ", machsOrder[i]);
  //printf("end of machines order \n");
  return(evMin);

}

/* write the task scheduling plan into a result file
 * which name is provided by argv[3]
 */

void 
OutputSchedule(char *outputfile)
{
  int i = 0, j;
  FILE *stream;
  char line[100];
  double tmpWork;

  if ( (stream = fopen(outputfile, "w")) == NULL) 
  {
    printf("can't open output file\n");
    exit(1);
  }

  for (i = 0; i < wsNum; i++) 
  {
    sprintf(line, "%s: %f\n", machName[i], optimalWorkload[i]);
    fputs(line, stream);
  }

  fclose(stream);

}

int 
main(int argc, char **argv)
{
  int i, j;

  double paraMean = 0.0, paraVari = 0.0;
  double optiev = 0, heurev = 0;

  float wl;
  int scheTag;

  if (argc != 5)
  {
    printf("pgsc resopara.txt workload result.txt schetag(0/1) \n");
    exit(1);
  }

  for (i = 0; i < wsNum; i++) machID[i] = 1;
	
  GetResoPara(argv[1]);
  sscanf(argv[2], "%f", &wl);
  totalWorkload = wl;
  sscanf(argv[4], "%d", &scheTag);

  /* printf("machine name: utilization, arrival-rate, service_std, relative speed \n");
  for (i = 0; i <wsNum; i++)
  {
    printf("%s %f %f %f %f \n", machName[i], arri_p[i]*serv_p[i], arri_p[i], servstd_p[i], speed_p[i]);
  } */

  if ( scheTag == 0)
  {
    OptimalSchedule();
  } else if (scheTag == 1)
  {
    HeuristicSchedule();
  } else 
  {
    printf("wrong scheduling tag, it must either 0 (optimal) or 1 (heuristic) \n");
    exit(1);
  }

  OutputSchedule(argv[3]);

  return(0);

}
