/*
 * Copyright (c) 2000-2003 Illinois Institute of Technology.
 *                         All rights reserved.
 *
 * This file is part of the GHS software package.  For license
 * information, see the LICENSE file in the top level directory of the
 * GHS source distribution.
 *
 * Released Date: August 18 2005
 * This is the GHS I release by the
 *   SCS Group,
 *   http://www.cs.iit.edu/~scs
 *   Department of Computer Science,
 *   Illinois Institute of Technology.
 *
 * This release has been tested under SUN OS 5.9.
 *
 * Supervisor
 * ----------
 * Illiniois Institute of Technology:
 * -Dr. Xian-He Sun
 *
 * Author(s)
 * -----------------
 * Illinois Institute of Technology:
 * -Ming Wu
 * -Xian-He Sun
 *
 */

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <time.h>
#include "ghs.h"

char measureLog[120]; 

double paraDay[MAX_DAY_SIZE];
double paraHour[MAX_HOUR_SIZE];
double paraAll[MAX_HOUR];

double alpha = 0.5, beta = 0.5;
int hourSample, daySample;
int hourNum, dayNum;

/* This array is used to store the configuration parameters */
char configArray[100][100];

double 
GetRandomAver(double *values, int maxNum, int numElements)
{
  int i = 0, elementIndex;
  double mean = 0.0, sd = 0.0, temp = 0.0, var = 0.0;

  //for(i=maxNum-1;i>=maxNum-numElements;i--)
  // mean += values[i];

  for (i = 0; i < numElements; i++)
  {
    elementIndex = (int) (1.0 * rand() * (maxNum-1) / RAND_MAX);
    mean += values[elementIndex];
  }
  
  mean /= numElements;

  return mean;
}

/* generate two arrays for store related measurement in history 
 * according to the users input "interval" which gives the time period 
 * over which the system parameter will be predicted
 */ 
void
GetParameters(char *paraName, int interval, int *maxDayNum, int *maxHourNum)
{  
  time_t currHour=0, currHourTmp=0;
  struct timeval tv;
  int dayNum, interNum;
  char bufConfig[100];
  char *str;
  int i, j, k1, k2, k3;
  int initHour, paraPosi=0, lastHour;
  int currParaIndex, lastParaIndex;
  int invalidTag = 0; 
  FILE *fdLog;
  float paraValue;

  if ( (fdLog = fopen(measureLog,"r") ) == NULL )
  {
    printf("\ncannot open log file to read");
    exit(1);
  }

  gettimeofday(&tv, NULL);
  currHour = tv.tv_sec / 3600;

  // check which column data should be read 
  if (!strcmp(paraName, "utilization")) paraPosi = 1;
  else if (!strcmp(paraName, "arrival")) paraPosi = 2;
  else if (!strcmp(paraName, "service")) paraPosi = 3;
  else if (!strcmp(paraName, "std")) paraPosi = 4;

  if (fgets(bufConfig, 100, fdLog) == NULL)
  {
    printf("No measurement in the file, check the filename input \n");
    exit(1);
  }

  i = 0;
  while ( (fgets(bufConfig, 100, fdLog) != NULL) && (i < MAX_HOUR) )
  {
    str = strtok(bufConfig, " ");
    if (i == 0) sscanf(str, "%d", &initHour);
    // ******************* the set of currHour ********************
    else sscanf(str, "%d", &currHourTmp);
    
    j = 0;
    while (j < paraPosi)
    { 
      str = strtok(NULL, " "); 
      j++;
    }
    sscanf(str, "%f", &paraValue);
    //printf("the paravalue is %f \n", paraValue);
    paraAll[i] = paraValue;
    i++;
  }

  if (currHour > (currHourTmp + 1)) currHour = currHourTmp;

  // check whether there is any measurement data available
  if (i == 0) 
  {
    printf("No measurment available by far\n");
    exit(0);
  }

  dayNum = (currHour - initHour) / 24 + 1;
  //printf("currHour is %d and initHour is %d \n", currHour, initHour);
  if( ((dayNum-1)*24) == (currHour-initHour) ) dayNum--;

  // nonsense for the following two statements
  interNum = (currHour - initHour) / interval + 1;
  if( ((interNum-1)*interval) == (currHour-initHour) ) interNum--;

  // may have some problem if two measurements in the same hour
  lastHour = initHour + i -1; //the time when last measurement is collected

  lastParaIndex = i-1; //sample size
  if (DEBUG_ON == 1) printf("the lastParaIndex is %d \n", lastParaIndex);
  
  currParaIndex = lastParaIndex;

  k1 = 0; 

  // calculate the hour samples
  while (currParaIndex >= 0) 
  {
    paraHour[k1] = 0;
    j = 0;
    invalidTag = 0;

    while ((currParaIndex >= 0) && (j < interval))
    {
      paraHour[k1] += paraAll[currParaIndex];
      if (paraAll[currParaIndex] == INVALID_PARA_VALUE) invalidTag = 1;
      currParaIndex--;
      j++;
    }

    paraHour[k1] = paraHour[k1] / j;
    if (DEBUG_ON == 1) printf("the paraHour[ %d ] is %f \n", k1, paraHour[k1]);
    if (invalidTag == 0) k1++;
    if (k1 == MAX_HOUR_SIZE-1) break;
  }

  currParaIndex = lastParaIndex; 
  k2 = 0;
  k3 = 0;

  // calculate the day samples
  while ( (k3 < dayNum) && (currParaIndex >= 0) )
  {
    currParaIndex = lastParaIndex - 24*k3;

    paraDay[k2] = 0;
    j = 0;
    invalidTag = 0;
    
    while ((currParaIndex >= 0) && (j < interval))
    {
      paraDay[k2] += paraAll[currParaIndex];
      if (paraAll[currParaIndex] == INVALID_PARA_VALUE) invalidTag = 1;
      currParaIndex--;
      j++;
    }
    
    paraDay[k2] = paraDay[k2] / j;
    if (DEBUG_ON == 1) printf("the paraDay[ %d ] is %f \n", k2, paraDay[k2]);
    if (invalidTag == 0) k2++;
    if (k2 == MAX_DAY_SIZE-1) break;
    k3++;
  }	
	
  *maxDayNum = k2; //return current number of day samples
  *maxHourNum = k1; //return current number of hour samples

  fclose(fdLog);

}

double
GetPrediction(int dayIndex, int hourIndex)
{

  int i, j;
  double predValue = 0, valueDay = 0, valueHour = 0;

  if ( (dayIndex <=0) || (hourIndex <= 0) ) 
  {
    printf("Data is missing for prediction \n");
    exit(1);
  }
	
  i=0;
  while ( (i<dayIndex) && (i<30) )
  {
    valueDay += paraDay[i];
    //printf("paraDay is %f \n", paraDay[i]);
    i++;
  }

  //printf("i is %d and dayIndex is %d \n", i, dayIndex);

  valueDay = valueDay/i;
  //printf("valueDay is %f \n", valueDay);

  j = 0;
  while ( (j < hourIndex) && (j < 30) )
  {
    valueHour += paraHour[j];
    j++;
  }

  //printf("j is %d and dayIndex is %d \n", j, hourIndex);
  valueHour = valueHour/j;
  //printf("valueHour is %f \n", valueHour);

  predValue = alpha*valueDay + beta*valueHour;
  return (predValue);

}

void 
GetMeasureLog(char *logFileName) 
{

  FILE *fdConfig;
  char bufConfig[100];
  char ghsConfig[100];
  char fileName[40];
  char hostName[80];
  char *temp, *temp1;
  int i = 0, j = 0;

  sprintf(ghsConfig, "%s%s", getenv("GHS_HOME"), "/ghs_config.dat");
  if ( (fdConfig = fopen(ghsConfig, "r") ) != NULL )
  {
    while (fgets(bufConfig, 100, fdConfig) != NULL)
    {
      if ((bufConfig[0] == 47)&&(bufConfig[1] == 47))
      { /*Process the comments here */
      }
      else {
        strcpy(configArray[i], bufConfig);
        i++; // number of configuration line
      }
    }
  }
  else {
    if (DEBUG_ON == 1) printf("\nCannot open the configuration file");
  }

  for(j = 0; j < i; j++)
  {
    temp = strtok(configArray[j]," ");
    temp1 = strtok(NULL," ");
    if (strcmp(temp,"LOGFILE_NAME") == 0) // unit_time, usually it is an hour
    {  
      //sscanf(temp1, "%s ", &fileName);
      if (gethostname(hostName, 80) != 0)
      {
        printf("Can not get the host name to generate the measure log file \n");
        exit(0);
      }
      sprintf(logFileName, "%s/%s%s", getenv("GHS_HOME"), hostName, temp1);
      //sprintf(logFileName, "%s/%s", getenv("GHS_HOME"), fileName); 
      //printf("\n the log file's name is: %s \n", logFileName);
    }

  }

  fclose(fdConfig);
}

/*
 * main program
 */

int 
main(int argc, char **argv)
{

  int periodTmp;
  char paraTmp[10];
  int dayIndex, hourIndex;
  double predResult;

  // read input parameters

  if ( argc < 3 )
  {
    printf("syspred utilization|arrival|service|std period [filename]\n");
    return (0);
  }

  sscanf(argv[1], "%s", &paraTmp);
  sscanf(argv[2], "%d", &periodTmp);
	
  if ( argc == 4) sscanf(argv[3], "%s", measureLog);
  else GetMeasureLog(measureLog); 

  //printf("the file is %s \n", paraTmp);

  GetParameters(paraTmp, periodTmp, &dayIndex, &hourIndex);

  //printf("dayIndex is %d and hourIndex is %d \n", dayIndex, hourIndex);

  predResult = GetPrediction(dayIndex, hourIndex);

  //printf("the prediction of %s for the next %d hours is %f \n", paraTmp, periodTmp, predResult);
  printf("%f \n", predResult);

}
